//
//  ATMPPR_Task.h
//
//  Created by tyler on 10/5/14.
//  Copyright (c) 2014 Atomic Pepper LLC. All rights reserved.
//

#import <Foundation/Foundation.h>

@interface ATMPPR_Task : NSObject

// 'find' methods
// searchPath must be specified

// fileExtensions case-insensitive NSArray of file extensions (e.g. @[@"mov",@"m4v",@"JPG"]
// fileExtensions can be nil (will match all files)

// age - NSNumber of at least how many days old (e.g. specify @2 for files at least 2 days old)
// age can be nil (will match all files)

// directoriesDeep - how deep to search subdirectories ... specify @1 for 'path' level search
// directoriesDeep can be nil (will search all subdirectories)


// e.g. [ATMPPR_Task removeFilesInDirectory:@"/Users/Shared" ofKind:@[@"mov"] minDaysOld:@3];
+(void)removeFilesInDirectory:(NSString *)searchPath ofKind:(NSArray *)fileExtensions minDaysOld:(NSNumber *)age;

//e.g. [ATMPPR_Task removeFilesInDirectory:NSTemporaryDirectory() ofKind:@[@"mov",@"jpg"] daysOld:nil maxDepth:@1];
+(void)removeFilesInDirectory:(NSString *)searchPath ofKind:(NSArray *)fileExtensions daysOld:(NSNumber *)age maxDepth:(NSNumber *)directoriesDeep;

// patterns - case-insensitive NSArray of filename patterns (e.g. @[@"myFile.txt",@"myFile.*",@"*.JPG"]
// patterns can be nil (will match all files)

//e.g. [ATMPPR_Task removeFilesInDirectory:@"/tmp" matchingPatterns:@[@"Teps-video *"] daysOld:@30 maxDepth:nil];
+(void)removeFilesInDirectory:(NSString *)searchPath matchingPatterns:(NSArray *)patterns daysOld:(NSNumber *)age maxDepth:(NSNumber *)directoriesDeep;

// Remove empty subdirectories ...
//e.g. [ATMPPR_Task removeSubdirectoriesAtPath:@"/Users/Shared" matchingPatterns:nil daysOld:@3 maxDepth:@1];
+(void)removeSubdirectoriesAtPath:(NSString *)path matchingPatterns:(NSArray *)patterns daysOld:(NSNumber *)age maxDepth:(NSNumber *)directoriesDeep;



//The generic task class method.
+(NSTask *)launchToolAtPath:(NSString *)launchPath withArguments:(NSArray *)args completionHandler:(void (^)(NSTask * task))completionBlock;

@end
