/*
     File: AVSPDocument.m
 Abstract: The players document class. It sets up the AVPlayer, AVPlayerLayer, manages adjusting the playback rate, enables and disables UI elements as appropriate, sets up a time observer for updating the current time (which the UI's time slider is bound to), and handles adjusting the volume of the AVPlayer.
  Version: 1.0
 
 Disclaimer: IMPORTANT:  This Apple software is supplied to you by Apple
 Inc. ("Apple") in consideration of your agreement to the following
 terms, and your use, installation, modification or redistribution of
 this Apple software constitutes acceptance of these terms.  If you do
 not agree with these terms, please do not use, install, modify or
 redistribute this Apple software.
 
 In consideration of your agreement to abide by the following terms, and
 subject to these terms, Apple grants you a personal, non-exclusive
 license, under Apple's copyrights in this original Apple software (the
 "Apple Software"), to use, reproduce, modify and redistribute the Apple
 Software, with or without modifications, in source and/or binary forms;
 provided that if you redistribute the Apple Software in its entirety and
 without modifications, you must retain this notice and the following
 text and disclaimers in all such redistributions of the Apple Software.
 Neither the name, trademarks, service marks or logos of Apple Inc. may
 be used to endorse or promote products derived from the Apple Software
 without specific prior written permission from Apple.  Except as
 expressly stated in this notice, no other rights or licenses, express or
 implied, are granted by Apple herein, including but not limited to any
 patent rights that may be infringed by your derivative works or by other
 works in which the Apple Software may be incorporated.
 
 The Apple Software is provided by Apple on an "AS IS" basis.  APPLE
 MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
 Copyright (C) 2011 Apple Inc. All Rights Reserved.
 
 */

#import "AVMovieViewController.h"
#import <AVFoundation/AVFoundation.h>

static void *AVSPPlayerItemStatusContext = &AVSPPlayerItemStatusContext;
static void *AVSPPlayerRateContext = &AVSPPlayerRateContext;
static void *AVSPPlayerLayerReadyForDisplay = &AVSPPlayerLayerReadyForDisplay;

@interface AVMovieViewController ()

- (void)setUpPlaybackOfAsset:(AVAsset *)asset withKeys:(NSArray *)keys;
- (void)stopLoadingAnimationAndHandleError:(NSError *)error;

@end

@implementation AVMovieViewController

@synthesize player;
@synthesize playerLayer;
@synthesize loadingSpinner;
@synthesize unplayableLabel;
@synthesize noVideoLabel;
@synthesize playerView;
@synthesize playPauseButton;
@synthesize fastForwardButton;
@synthesize rewindButton;
@synthesize timeSlider;
@synthesize timeObserverToken;
@synthesize fileURL; // 2013-10-16steve



-(void)awakeFromNib
{
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(handleNotification_PlayMovie:) // 2014-10-27 tyler
                                                 name:@"PlayMovieNotification"
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(playerItemDidReachEnd:)
                                                 name:AVPlayerItemDidPlayToEndTimeNotification // 2014-11-04steve
                                               object:nil];

}
- (void)dealloc
{
	[player release];
	[playerLayer release];
    
    
    [[NSNotificationCenter defaultCenter] removeObserver:self ]; // 2014-10-26steve
	
	[super dealloc];
}

- (NSString *)windowNibName
{
	return @"AVSPDocument";
}

-(void)setup
{

	// [[[self playerView] layer] setBackgroundColor:CGColorGetConstantColor(kCGColorBlack)];
	[[[self playerView] layer] setBackgroundColor:CGColorGetConstantColor(kCGColorClear)]; // 2013-10-16steve
	[[self loadingSpinner] startAnimation:self];
	
	// Create the AVPlayer, add rate and status observers
	[self setPlayer:[[[AVPlayer alloc] init] autorelease]];
	[self addObserver:self forKeyPath:@"player.rate" options:NSKeyValueObservingOptionNew context:AVSPPlayerRateContext];
	[self addObserver:self forKeyPath:@"player.currentItem.status" options:NSKeyValueObservingOptionNew context:AVSPPlayerItemStatusContext];
	
	// Create an asset with our URL, asychronously load its tracks, its duration, and whether it's playable or protected.
	// When that loading is complete, configure a player to play the asset.
	AVURLAsset *asset = [AVAsset assetWithURL:self.fileURL];
    NSLog(@"222self.fileURL.absoluteString = %@", self.fileURL.absoluteString);
    NSLog(@"222asset = %@", asset);
    NSLog(@"222asset.url = %@", asset.URL);
    
	NSArray *assetKeysToLoadAndTest = [NSArray arrayWithObjects:@"playable", @"hasProtectedContent", @"tracks", @"duration", nil];
	[asset loadValuesAsynchronouslyForKeys:assetKeysToLoadAndTest completionHandler:^(void) {
		
		// The asset invokes its completion handler on an arbitrary queue when loading is complete.
		// Because we want to access our AVPlayer in our ensuing set-up, we must dispatch our handler to the main queue.
		dispatch_async(dispatch_get_main_queue(), ^(void) {
			
			[self setUpPlaybackOfAsset:asset withKeys:assetKeysToLoadAndTest];
			
		});
		
	}];
    
    
    
    
    BOOL doAutoPlay = [[NSUserDefaults standardUserDefaults] boolForKey:@"prefRunSalesWindowInKioskMode"]; // 2014-11-06steve YES; // 2014-11-04
    if ( doAutoPlay) {
        [self playPauseToggle:nil]; // 2014-11-04steve
    }
    

// 2014-10-27 tyler disabled this because it is in the wrong place. tyler moved it to a place where it will be called only once so that this notification will fire *only once* per each "play button press"
//    [[NSNotificationCenter defaultCenter] addObserver:self
//                                             selector:@selector(handleNotification_PlayMovie:) // 2014-10-26steve
//                                                 name:@"PlayMovieNotification"
//                                               object:nil];

    
}



//- (void)windowControllerDidLoadNib:(NSWindowController *)windowController
//{
//	[super windowControllerDidLoadNib:windowController];
//	[[windowController window] setMovableByWindowBackground:YES];
//	// [[[self playerView] layer] setBackgroundColor:CGColorGetConstantColor(kCGColorBlack)];
//	[[[self playerView] layer] setBackgroundColor:CGColorGetConstantColor(kCGColorClear)]; // 2013-10-16steve
//	[[self loadingSpinner] startAnimation:self];
//	
//	// Create the AVPlayer, add rate and status observers
//	[self setPlayer:[[[AVPlayer alloc] init] autorelease]];
//	[self addObserver:self forKeyPath:@"player.rate" options:NSKeyValueObservingOptionNew context:AVSPPlayerRateContext];
//	[self addObserver:self forKeyPath:@"player.currentItem.status" options:NSKeyValueObservingOptionNew context:AVSPPlayerItemStatusContext];
//	
//	// Create an asset with our URL, asychronously load its tracks, its duration, and whether it's playable or protected.
//	// When that loading is complete, configure a player to play the asset.
//	AVURLAsset *asset = [AVAsset assetWithURL:[self fileURL]];
//	NSArray *assetKeysToLoadAndTest = [NSArray arrayWithObjects:@"playable", @"hasProtectedContent", @"tracks", @"duration", nil];
//	[asset loadValuesAsynchronouslyForKeys:assetKeysToLoadAndTest completionHandler:^(void) {
//		
//		// The asset invokes its completion handler on an arbitrary queue when loading is complete.
//		// Because we want to access our AVPlayer in our ensuing set-up, we must dispatch our handler to the main queue.
//		dispatch_async(dispatch_get_main_queue(), ^(void) {
//			
//			[self setUpPlaybackOfAsset:asset withKeys:assetKeysToLoadAndTest];
//			
//		});
//		
//	}];
//}

- (void)setUpPlaybackOfAsset:(AVAsset *)asset withKeys:(NSArray *)keys
{
	// This method is called when the AVAsset for our URL has completing the loading of the values of the specified array of keys.
	// We set up playback of the asset here.
	
	// First test whether the values of each of the keys we need have been successfully loaded.
	for (NSString *key in keys)
	{
		NSError *error = nil;
		AVURLAsset* avurlasset = nil;
        if ([asset isKindOfClass:[AVURLAsset class]])
            avurlasset = (AVURLAsset*)asset;
        
        NSLog(@"2013-10-16steve key2 = [%@]", key);
        NSLog(@"222asset.url = %@", avurlasset.URL);

		if ([asset statusOfValueForKey:key error:&error] == AVKeyValueStatusFailed)
		{
			[self stopLoadingAnimationAndHandleError:error];
			return;
		}
	}
	
	if (![asset isPlayable] || [asset hasProtectedContent])
	{
		// We can't play this asset. Show the "Unplayable Asset" label.
		[self stopLoadingAnimationAndHandleError:nil];
		[[self unplayableLabel] setHidden:NO];
		return;
	}
	
	// We can play this asset.
	// Set up an AVPlayerLayer according to whether the asset contains video.
	if ([[asset tracksWithMediaType:AVMediaTypeVideo] count] != 0)
	{
        NSLog(@"sublayers = %@", self.playerView.layer.sublayers);
        self.playerView.layer.sublayers = nil; // 2013-10-22steve
        
		// Create an AVPlayerLayer and add it to the player view if there is video, but hide it until it's ready for display
		AVPlayerLayer *newPlayerLayer = [AVPlayerLayer playerLayerWithPlayer:[self player]];
		[newPlayerLayer setFrame:[[[self playerView] layer] bounds]];
		[newPlayerLayer setAutoresizingMask:kCALayerWidthSizable | kCALayerHeightSizable];
		[newPlayerLayer setHidden:YES];
		[[[self playerView] layer] addSublayer:newPlayerLayer];
		[self setPlayerLayer:newPlayerLayer];
		[self addObserver:self forKeyPath:@"playerLayer.readyForDisplay" options:NSKeyValueObservingOptionInitial | NSKeyValueObservingOptionNew context:AVSPPlayerLayerReadyForDisplay];
	}
	else
	{
		// This asset has no video tracks. Show the "No Video" label.
		[self stopLoadingAnimationAndHandleError:nil];
		[[self noVideoLabel] setHidden:NO];
	}
	
	// Create a new AVPlayerItem and make it our player's current item.
	AVPlayerItem *playerItem = [AVPlayerItem playerItemWithAsset:asset];
	[[self player] replaceCurrentItemWithPlayerItem:playerItem];
	
	[self setTimeObserverToken:[[self player] addPeriodicTimeObserverForInterval:CMTimeMake(1, 10) queue:dispatch_get_main_queue() usingBlock:^(CMTime time) {
		[[self timeSlider] setDoubleValue:CMTimeGetSeconds(time)];
	}]];
	
}

- (void)stopLoadingAnimationAndHandleError:(NSError *)error
{
	[[self loadingSpinner] stopAnimation:self];
	[[self loadingSpinner] setHidden:YES];
	if (error)
	{
        NSLog(@"2013-10-16steve - NEED to ERROR HERE = %@", error);
//		[self presentError:error
//			modalForWindow:[self windowForSheet]
//				  delegate:nil
//		didPresentSelector:NULL
//			   contextInfo:nil];
	}
}


- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    NSNumber * changeValue = [change objectForKey:NSKeyValueChangeNewKey];
    if (![changeValue isKindOfClass:NSNumber.class]) {
        changeValue = nil;
    }
    
    if (context == AVSPPlayerItemStatusContext)
    {
        AVPlayerStatus status = [changeValue integerValue];
        BOOL enable = NO;
        switch (status)
        {
            case AVPlayerItemStatusUnknown:
                break;
            case AVPlayerItemStatusReadyToPlay:
                enable = YES;
                break;
            case AVPlayerItemStatusFailed:
                [self stopLoadingAnimationAndHandleError:[[[self player] currentItem] error]];
                break;
        }
        
        [[self playPauseButton] setEnabled:enable];
        [[self fastForwardButton] setEnabled:enable];
        [[self rewindButton] setEnabled:enable];
    }
    else if (context == AVSPPlayerRateContext)
    {
        float rate = [changeValue floatValue];
        if (rate != 1.f)
        {
            // [[self playPauseButton] setTitle:@"Play"];
            [[self playPauseButton] setImage:[NSImage imageNamed:@"PlayON"]]; // 2013-10-22steve
            [[NSNotificationCenter defaultCenter] postNotificationName:@"AVMovieViewNotificaiton"
                                                                object:self
                                                              userInfo:[NSDictionary dictionaryWithObjectsAndKeys:@"SetAsPlayButton",@"command",nil]]; // 2014-11-04steve
            
        }
        else
        {
            // [[self playPauseButton] setTitle:@"Pause"];
            [[self playPauseButton] setImage:[NSImage imageNamed:@"PauseOFF"]]; // 2013-10-22steve
            [[NSNotificationCenter defaultCenter] postNotificationName:@"AVMovieViewNotificaiton"
                                                                object:self
                                                              userInfo:[NSDictionary dictionaryWithObjectsAndKeys:@"SetAsPauseButton",@"command",nil]]; // 2014-11-04steve

        }
    }
    else if (context == AVSPPlayerLayerReadyForDisplay)
    {
        if ([changeValue boolValue] == YES)
        {
            // The AVPlayerLayer is ready for display. Hide the loading spinner and show it.
            [self stopLoadingAnimationAndHandleError:nil];
            [[self playerLayer] setHidden:NO];
        }
    }
    else
    {
        [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
    }
}

//- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
//{
//    NSNumber * changeValue = [change objectForKey:NSKeyValueChangeNewKey]; // 2014-11-04steve - from tyler -to prevent crashing
//    if (![changeValue isKindOfClass:NSNumber.class]) {
//        changeValue = nil;
//    }
//
//    
//	if (context == AVSPPlayerItemStatusContext)
//	{
//		AVPlayerStatus status = [[change objectForKey:NSKeyValueChangeNewKey] integerValue];
//		BOOL enable = NO;
//		switch (status)
//		{
//			case AVPlayerItemStatusUnknown:
//				break;
//			case AVPlayerItemStatusReadyToPlay:
//				enable = YES;
//				break;
//			case AVPlayerItemStatusFailed:
//				[self stopLoadingAnimationAndHandleError:[[[self player] currentItem] error]];
//				break;
//		}
//		
//		[[self playPauseButton] setEnabled:enable];
//		[[self fastForwardButton] setEnabled:enable];
//		[[self rewindButton] setEnabled:enable];
//	}
//	else if (context == AVSPPlayerRateContext)
//	{
//		float rate = [[change objectForKey:NSKeyValueChangeNewKey] floatValue];
//		if (rate != 1.f)
//		{
//			// [[self playPauseButton] setTitle:@"Play"];
//            [[self playPauseButton] setImage:[NSImage imageNamed:@"PlayON"]]; // 2013-10-22steve
//		}
//		else
//		{
//			// [[self playPauseButton] setTitle:@"Pause"];
//            [[self playPauseButton] setImage:[NSImage imageNamed:@"PauseOFF"]]; // 2013-10-22steve
//		}
//	}
//	else if (context == AVSPPlayerLayerReadyForDisplay)
//	{
//		if ([[change objectForKey:NSKeyValueChangeNewKey] boolValue] == YES)
//		{
//			// The AVPlayerLayer is ready for display. Hide the loading spinner and show it.
//			[self stopLoadingAnimationAndHandleError:nil];
//			[[self playerLayer] setHidden:NO];
//		}
//	}
//	else
//	{
//		[super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
//	}
//}

- (void)close
{
    NSLog(@"Inside %@-%@...",self,NSStringFromSelector(_cmd));
    
	[[self player] pause];
	[[self player] removeTimeObserver:[self timeObserverToken]];
	[self setTimeObserverToken:nil];
	[self removeObserver:self forKeyPath:@"player.rate"];
	[self removeObserver:self forKeyPath:@"player.currentItem.status"];
	if ([self playerLayer]) {
		[self removeObserver:self forKeyPath:@"playerLayer.readyForDisplay"];        
    }
    
    self.playerView.layer.sublayers = nil; // 2013-10-22steve - this removes old movie from view

	// [super close];
}

+ (NSSet *)keyPathsForValuesAffectingDuration
{
	return [NSSet setWithObjects:@"player.currentItem", @"player.currentItem.status", nil];
}

- (double)duration
{
	AVPlayerItem *playerItem = [[self player] currentItem];
	
	if ([playerItem status] == AVPlayerItemStatusReadyToPlay)
		return CMTimeGetSeconds([[playerItem asset] duration]);
	else
		return 0.f;
}

- (double)currentTime
{
	return CMTimeGetSeconds([[self player] currentTime]);
}

- (void)setCurrentTime:(double)time
{
	[[self player] seekToTime:CMTimeMakeWithSeconds(time, 1) toleranceBefore:kCMTimeZero toleranceAfter:kCMTimeZero];
}

+ (NSSet *)keyPathsForValuesAffectingVolume
{
	return [NSSet setWithObject:@"player.volume"];
}

- (float)volume
{
	return [[self player] volume];
}

- (void)setVolume:(float)volume
{
	[[self player] setVolume:volume];
}
-(NSScreen *)movieScreen
{
    NSScreen * movieScreen;
    NSArray * screens = [NSScreen screens];
    if (screens.count > 1)
    {
        for (movieScreen in screens) {
            NSRect f = movieScreen.frame;
            if (f.size.width >= 1920 && f.size.height >= 1080) {
                break;
            }
        }

    }
    else
    {
        movieScreen = [screens lastObject];
    }
    return movieScreen;
}

-(void) doAutoPlay // 2014-11-04steve
{
    BOOL handleFullScreen = NO; // 2014-10-28steve
    if ([[self player] rate] != 1.f)
	{
        if (handleFullScreen) { // 2014-10-28steve
            [self.view enterFullScreenMode:[self movieScreen] withOptions:nil];
        }
		
        if ([self currentTime] == [self duration])
			[self setCurrentTime:0.f];
		[[self player] play];
	}

}

- (IBAction)playPauseToggle:(id)sender
{
    BOOL handleFullScreen = NO; // 2014-10-28steve
    
	if ([[self player] rate] != 1.f)
	{
        // 2014-10-28steve
        
        if (handleFullScreen) { // 2014-10-28steve
            [self.view enterFullScreenMode:[self movieScreen] withOptions:nil];
        }
		
        if ([self currentTime] == [self duration])
			[self setCurrentTime:0.f];
		[[self player] play];
	}
	else
	{
        if (handleFullScreen) { // 2014-10-28steve
            [self.view exitFullScreenModeWithOptions:nil];
            [[NSNotificationCenter defaultCenter] postNotificationName:@"MovieExitFullScreenMode" object:self userInfo:nil]; // 2014-10-27steve
        }
        
		[[self player] pause];
	}
}

- (IBAction)fastForward:(id)sender
{
	if ([[self player] rate] < 2.f)
	{
		[[self player] setRate:2.f];
	}
	else
	{
		[[self player] setRate:[[self player] rate] + 2.f];
	}
}

- (IBAction)rewind:(id)sender
{
	if ([[self player] rate] > -2.f)
	{
		[[self player] setRate:-2.f];
	}
	else
	{
		[[self player] setRate:[[self player] rate] - 2.f];
	}
}

- (NSData *)dataOfType:(NSString *)typeName error:(NSError **)outError
{
	if (outError != NULL)
		*outError = [NSError errorWithDomain:NSOSStatusErrorDomain code:unimpErr userInfo:NULL];
	return nil;
}

- (BOOL)readFromData:(NSData *)data ofType:(NSString *)typeName error:(NSError **)outError
{
	if (outError != NULL)
		*outError = [NSError errorWithDomain:NSOSStatusErrorDomain code:unimpErr userInfo:NULL];
	return YES;
}



#pragma mark -

- (void) handleNotification_PlayMovie:(NSNotification*) note // 2014-10-26steve
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	NSLog (@ "Inside %@ => %@...", [self class], NSStringFromSelector(_cmd) );
    
    [self playPauseToggle:nil];
	
	if ( pool )	[pool release];
}


-(void) playerItemDidReachEnd:(NSNotification*)note // 2014-11-04steve
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	NSLog (@ "Inside %@ => %@...", [self class], NSStringFromSelector(_cmd) );
    
    [self.player seekToTime:kCMTimeZero];
    [[self player] play];

	
	if ( pool )	[pool release];

}



@end
