//
//  EditAVMovieWindowController.m
//  Teps-Video
//
//  Created by tyler on 9/26/13.
//
//

#import "EditAVMovieWindowController.h"
#import "TepsExtras.h" // 2014-09-30steve


// 2013-11-08steve - from tyler
const NSString * TepsAV_AssetsArray = @"TepsAVAssetsArray";
const NSString * TepsAV_AssetURL = @"TepsAV_AssetURL";
const NSString * TepsAV_OutputURL = @"TepsAVOutputURL";
const NSString * TepsAV_UseableTimeRange = @"TepsAV_UseableTimeRange";


static NSString* sPrefMasterDvdFolderKey = @"prefMasterDvdFolderFolder"; // 2013-11-11steve
static NSString* sPrefBeginningChapterNameKey = @"prefBeginningChapterName"; // 2013-11-11steve
static NSString* sPrefEndingChapterNameKey = @"prefEndingChapterName"; // 2013-11-11steve



@interface EditAVMovieWindowController ()

@end

@implementation EditAVMovieWindowController




-(void)exportCurrentMovie:(NSDictionary*)ticket
{
    NSString* currentFeatureVideoPath = self.movieRef;
    NSString* name = [[self.movieRef lastPathComponent] stringByDeletingPathExtension];
    
    NSURL * part1URL = [NSURL fileURLWithPath:currentFeatureVideoPath];
    NSURL * part2URL = nil;
    NSURL * outputURL = [NSURL URLWithString:[NSString stringWithFormat:@"file:///tmp/%@_exported.mov",name]];
    
    
    NSError* error = nil;
    
    AVMutableComposition* composition = [AVMutableComposition composition];
    
    AVURLAsset* video1 = nil;
    AVURLAsset* video2 = nil;
    if (part1URL) video1 = [AVURLAsset URLAssetWithURL:part1URL options:nil]; // 2013-12-03 fix leaks
    if (part2URL) video2 = [AVURLAsset URLAssetWithURL:part2URL options:nil]; // 2013-12-03 fix leaks
    
    AVMutableCompositionTrack * composedTrack = [composition addMutableTrackWithMediaType:AVMediaTypeVideo
                                                                         preferredTrackID:kCMPersistentTrackID_Invalid];
    
    // ADD Video1
    if (video1) {
        NSLog(@"attempting to insert video1...");
        [composedTrack insertTimeRange:CMTimeRangeMake(kCMTimeZero, video1.duration)
                               ofTrack:[[video1 tracksWithMediaType:AVMediaTypeVideo] objectAtIndex:0]
                                atTime:kCMTimeZero
                                 error:&error];
        if (error) NSLog(@"error inserting video1 = %@",error);
    }
    
    // ADD Video2
    if (video2) {
        NSLog(@"attempting to insert video2...");
        error = nil;
        [composedTrack insertTimeRange:CMTimeRangeMake(kCMTimeZero, video2.duration)
                               ofTrack:[[video2 tracksWithMediaType:AVMediaTypeVideo] objectAtIndex:0]
                                atTime:video1.duration
                                 error:&error];
        if (error) NSLog(@"error inserting video2 = %@",error);
    }
    
    
    NSString* exportPresetName = AVAssetExportPreset960x540; // AVAssetExportPreset1280x720; // AVAssetExportPreset640x480; // AVAssetExportPreset960x540
    AVAssetExportSession *exportSession = [[[AVAssetExportSession alloc] initWithAsset:composition
                                                                            presetName: exportPresetName] autorelease];
    
    [[NSFileManager defaultManager] removeItemAtURL:outputURL error:nil];
    
    exportSession.outputURL=outputURL;
    exportSession.outputFileType = AVFileTypeQuickTimeMovie; // @"com.apple.quicktime-movie";// AVFileTypeAppleM4V; //
    exportSession.shouldOptimizeForNetworkUse = YES;
    [exportSession exportAsynchronouslyWithCompletionHandler:^(void){
        NSLog(@"exportAsynchronouslyWithCompletionHandler....");
        switch (exportSession.status) {
            case AVAssetExportSessionStatusCompleted:
                NSLog(@"%@ => Export completed to :%@", NSStringFromSelector(_cmd), exportSession.outputURL.path);
                // do something here
                break;
            case AVAssetExportSessionStatusFailed:
                //
                NSLog(@"%@ => Failed:%@", NSStringFromSelector(_cmd), exportSession.error);
                break;
            case AVAssetExportSessionStatusCancelled:
                //
                NSLog(@"%@ => Canceled:%@", NSStringFromSelector(_cmd), exportSession.error);
                break;
            default:
                NSLog(@"%@ => Other:(Status %ld),%@",NSStringFromSelector(_cmd), exportSession.status, exportSession.error);
                break;
        }
    }];
    
    
    
    
}


-(void)processAVOutputTicketSteve:(NSDictionary*)ticket
{
    NSString* currentFeatureVideoPath = self.movieRef;
    
    
    NSString* endingChapterMovieName = [[NSUserDefaults standardUserDefaults] stringForKey:sPrefEndingChapterNameKey];
	NSString* beginningChapterMovieName = [[NSUserDefaults standardUserDefaults] stringForKey:sPrefBeginningChapterNameKey];
	NSString* masterDvdFolderPath = [[NSUserDefaults standardUserDefaults] stringForKey:sPrefMasterDvdFolderKey];
    
    NSLog(@"   endingChapterMovieName = [%@]",endingChapterMovieName);
    NSLog(@"beginningChapterMovieName = [%@]",beginningChapterMovieName);
    NSLog(@"      masterDvdFolderPath = [%@]",masterDvdFolderPath);
    NSLog(@"       self.mAVMovieAsset = [%@]",self.mAVMovieAsset);
    
    
    NSURL * chapter1URL = [NSURL URLWithString:@"file:///Users/Shared/MasterDvdContentFolder/chapter1.mov"];
    NSURL * chapter3URL = [NSURL URLWithString:@"file:///Users/Shared/MasterDvdContentFolder/chapter3.mov"];
    NSURL * featureURL = [NSURL URLWithString:@"file:///Users/Shared/customer.mov"];
    
    NSURL * outputURLFinal = [NSURL URLWithString:@"file:///tmp/MergeMovie.mov"];
    NSURL * outputURLPartA = [NSURL URLWithString:@"file:///tmp/MergeMoviePartA.mov"];
    
    
    NSURL * part1URL = nil;
    NSURL * part2URL = nil;
    NSURL * outputURL = nil;
    
    
    {
        NSString* tempPath = nil;
        
        tempPath = [masterDvdFolderPath stringByAppendingPathComponent:beginningChapterMovieName];
        chapter1URL = [NSURL fileURLWithPath:tempPath];
        
        tempPath = [masterDvdFolderPath stringByAppendingPathComponent:endingChapterMovieName];
        chapter3URL = [NSURL fileURLWithPath:tempPath];
        
        tempPath = currentFeatureVideoPath; // [masterDvdFolderPath stringByAppendingPathComponent:currentFeatureVideoPath];
        featureURL = [NSURL fileURLWithPath:tempPath];
        
    }
    
    NSLog(@"chapter1URL = %@",chapter1URL);
    NSLog(@"chapter3URL = %@",chapter3URL);
    NSLog(@" featureURL = %@",featureURL);
    NSLog(@"  outputURL = %@",outputURL);
    
    
    BOOL doFinalMerge = YES;
    if (doFinalMerge) {
        part1URL = outputURLPartA;
        part2URL = chapter3URL;
        outputURL = outputURLFinal;
        
    } else {
        part1URL = chapter1URL;
        part2URL = featureURL;
        outputURL = outputURLPartA;
        
    }
    
    
    NSError* error = nil;
    
    AVMutableComposition* composition = [AVMutableComposition composition];
    
    AVURLAsset* video1 = [AVURLAsset URLAssetWithURL:part1URL options:nil]; //2013-12-03 fix leaks
    AVURLAsset* video2 = [AVURLAsset URLAssetWithURL:part2URL options:nil]; //2013-12-03 fix leaks
    
    AVMutableCompositionTrack * composedTrack = [composition addMutableTrackWithMediaType:AVMediaTypeVideo
                                                                         preferredTrackID:kCMPersistentTrackID_Invalid];
    
    // ADD Chapter1
    [composedTrack insertTimeRange:CMTimeRangeMake(kCMTimeZero, video1.duration)
                           ofTrack:[[video1 tracksWithMediaType:AVMediaTypeVideo] objectAtIndex:0]
                            atTime:kCMTimeZero
                             error:&error];
    if (error) NSLog(@"error inserting video1 = %@",error);
    
    // ADD FEATURE
    error = nil;
    [composedTrack insertTimeRange:CMTimeRangeMake(kCMTimeZero, video2.duration)
                           ofTrack:[[video2 tracksWithMediaType:AVMediaTypeVideo] objectAtIndex:0]
                            atTime:video1.duration
                             error:&error];
    if (error) NSLog(@"error inserting video2 = %@",error);
    
    
    NSString* exportPresetName = AVAssetExportPreset960x540; // AVAssetExportPreset1280x720; // AVAssetExportPreset640x480
    AVAssetExportSession *exportSession = [[[AVAssetExportSession alloc] initWithAsset:composition
                                                                            presetName: exportPresetName] autorelease];
    
    [[NSFileManager defaultManager] removeItemAtURL:outputURL error:nil];
    
    exportSession.outputURL=outputURL;
    exportSession.outputFileType = @"com.apple.quicktime-movie";
    exportSession.shouldOptimizeForNetworkUse = YES;
    [exportSession exportAsynchronouslyWithCompletionHandler:^(void){
        NSLog(@"exportAsynchronouslyWithCompletionHandler....");
        switch (exportSession.status) {
            case AVAssetExportSessionStatusCompleted:
                NSLog(@"Export completed to :%@",exportSession.outputURL.path);
                // do something here
                break;
            case AVAssetExportSessionStatusFailed:
                //
                NSLog(@"Failed:%@",exportSession.error);
                break;
            case AVAssetExportSessionStatusCancelled:
                //
                NSLog(@"Canceled:%@",exportSession.error);
                break;
            default:
                NSLog(@"Other:(Status %ld),%@",exportSession.status,exportSession.error);
                break;
        }
    }];
    
    
    
    
}
-(CGSize)trackDimensionsForAsset:(AVAsset *)asset
{
    
    AVAssetTrack *videoTrack = nil;
    
    // Check if the asset contains video and audio tracks
    if ([[asset tracksWithMediaType:AVMediaTypeVideo] count] != 0)
    {
        videoTrack = [[asset tracksWithMediaType:AVMediaTypeVideo] objectAtIndex:0];
    }
    
    // Get the format description of the track, to fill in attributes of the video stream that we don't want to change
    CMFormatDescriptionRef formatDescription = NULL;
    NSArray *formatDescriptions = [videoTrack formatDescriptions];
    if ([formatDescriptions count] > 0)
        formatDescription = (CMFormatDescriptionRef)[formatDescriptions objectAtIndex:0];
    
    // Grab track dimensions from format description
    CGSize trackDimensions = {
        .width = 0.0,
        .height = 0.0,
    };
    if (formatDescription)
        trackDimensions = CMVideoFormatDescriptionGetPresentationDimensions(formatDescription, true, false);
    else
        trackDimensions = [videoTrack naturalSize];
    return trackDimensions;
    
    
    // Grab clean aperture, pixel aspect ratio from format description
    NSDictionary *compressionSettings = nil;
    if (formatDescription)
    {
        NSDictionary *cleanAperture = nil;
        NSDictionary *pixelAspectRatio = nil;
        CFDictionaryRef cleanApertureFromCMFormatDescription = CMFormatDescriptionGetExtension(formatDescription, kCMFormatDescriptionExtension_CleanAperture);
        if (cleanApertureFromCMFormatDescription)
        {
            cleanAperture = [NSDictionary dictionaryWithObjectsAndKeys:
                             CFDictionaryGetValue(cleanApertureFromCMFormatDescription, kCMFormatDescriptionKey_CleanApertureWidth), AVVideoCleanApertureWidthKey,
                             CFDictionaryGetValue(cleanApertureFromCMFormatDescription, kCMFormatDescriptionKey_CleanApertureHeight), AVVideoCleanApertureHeightKey,
                             CFDictionaryGetValue(cleanApertureFromCMFormatDescription, kCMFormatDescriptionKey_CleanApertureHorizontalOffset), AVVideoCleanApertureHorizontalOffsetKey,
                             CFDictionaryGetValue(cleanApertureFromCMFormatDescription, kCMFormatDescriptionKey_CleanApertureVerticalOffset), AVVideoCleanApertureVerticalOffsetKey,
                             nil];
        }
        CFDictionaryRef pixelAspectRatioFromCMFormatDescription = CMFormatDescriptionGetExtension(formatDescription, kCMFormatDescriptionExtension_PixelAspectRatio);
        if (pixelAspectRatioFromCMFormatDescription)
        {
            pixelAspectRatio = [NSDictionary dictionaryWithObjectsAndKeys:
                                CFDictionaryGetValue(pixelAspectRatioFromCMFormatDescription, kCMFormatDescriptionKey_PixelAspectRatioHorizontalSpacing), AVVideoPixelAspectRatioHorizontalSpacingKey,
                                CFDictionaryGetValue(pixelAspectRatioFromCMFormatDescription, kCMFormatDescriptionKey_PixelAspectRatioVerticalSpacing), AVVideoPixelAspectRatioVerticalSpacingKey,
                                nil];
        }
        
        if (cleanAperture || pixelAspectRatio)
        {
            NSMutableDictionary *mutableCompressionSettings = [NSMutableDictionary dictionary];
            if (cleanAperture)
                [mutableCompressionSettings setObject:cleanAperture forKey:AVVideoCleanApertureKey];
            if (pixelAspectRatio)
                [mutableCompressionSettings setObject:pixelAspectRatio forKey:AVVideoPixelAspectRatioKey];
            compressionSettings = mutableCompressionSettings;
        }
    }
    return trackDimensions;
    
}
-(void)writerDidFinishWithSuccess:(BOOL)success
{
    if(success)
    {
        //a conversion just completed
        dispatch_semaphore_signal(waitSemaphore);
    }
}
// 2013-11-08steve - from tyler
-(void)processAVOutputTicket:(NSDictionary *)ticket
{
    
    // 2013-11-19steve
    NSTimer* aTimer = [NSTimer timerWithTimeInterval:1.0
                                              target:self
                                            selector:@selector(progressTimer2:)
                                            userInfo:nil
                                             repeats:YES]; // 2013-11-19steve
    
    [[NSRunLoop mainRunLoop] addTimer:aTimer
                              forMode:NSDefaultRunLoopMode]; // 2013-11-19steve
    
    
    // 2013-11-19steve   [[NSTimer scheduledTimerWithTimeInterval:2.5 target:self selector:@selector(progressTimer:) userInfo:exportSession repeats:YES] retain];

    
    
    // 2013-11-12steve DEBUG[self exportCurrentMovie:ticket]; // 2013-11-12steve - TEST exporting of feature video in case it was trimmed
    
    
    
    // [self processAVOutputTicketSteve:ticket]; // 2013-11-12steve
    // return; // 2013-11-12steve
    
    // tyler (for steve)
    // "ticket" is a dict that contains the following pairs:
    
    //NSDictionary * d = @{TepsAV_AssetsArray : @[@{TepsAV_AssetURL : [NSURL URLWithString:@"file:///anAssestURL1"],TepsAV_UseableTimeRange : [NSValue valueWithCMTimeRange:CMTimeRangeMake(kCMTimeZero, kCMTimeZero)]},@{TepsAV_AssetURL : [NSURL URLWithString:@"file:///anAssestURL2"],TepsAV_UseableTimeRange : [NSValue valueWithCMTimeRange:CMTimeRangeMake(kCMTimeZero, kCMTimeZero)]},@{TepsAV_AssetURL : [NSURL URLWithString:@"file:///anAssestURL3"],TepsAV_UseableTimeRange : [NSValue valueWithCMTimeRange:CMTimeRangeMake(kCMTimeZero, kCMTimeZero)]}], TepsAV_OutputURL:[NSURL URLWithString:@"file:///outputFile"]};
    //
    // 1) key=TepsAV_AssetsArray; value is an NSArray of dicts
    // 2) key=TepsAV_OutputURL; value is an NSURL pointing to the output file location (must be a file URL)
    
    NSArray * assets = ticket[TepsAV_AssetsArray];
    NSDictionary * dict;
    AVURLAsset * asset;
    NSURL * url;
    NSString * pathExtension = nil;
    NSUInteger assetCount = assets.count;
    if(3 == assetCount)
    {
        //process lead-in and lead-out to match the feature
        MyAVAssetRW * resizer = [[MyAVAssetRW alloc] init];
        resizer.delegate = self;
        dispatch_queue_t newQ =  NULL;
        
        dict = assets[1];
        url = dict[TepsAV_AssetURL];
        // get the output size that we need to match
        asset = [AVURLAsset URLAssetWithURL:url options:@{AVURLAssetPreferPreciseDurationAndTimingKey:@YES}];
        CGSize size = [self trackDimensionsForAsset:asset];
//        
//        size.height = 540; // 2013-11-18steve
//        size.width = 960; // 2013-11-18steve
//        
        resizer.outputSize = size;
        pathExtension = [NSString stringWithFormat:@"%.0fx%.0f.%@",resizer.outputSize.width,resizer.outputSize.height,[url pathExtension]];
        
        
        // resize the lead-in(asset[0]) and lead-out(asset[2]) to match the feature (asset[1])
        dict = assets[0];
        newQ = dispatch_queue_create(NULL, NULL);
        for (dict in @[assets[0],assets[1],assets[2]])
        {
            url = dict[TepsAV_AssetURL];
            asset = [AVURLAsset URLAssetWithURL:url options:@{AVURLAssetPreferPreciseDurationAndTimingKey:@YES}];
            size = [self trackDimensionsForAsset:asset];
            // 2013-11-18steve if (!CGSizeEqualToSize(resizer.outputSize, size))
            {
                resizer.inputURL = url;
                url = [url URLByDeletingPathExtension];
                url = [url URLByAppendingPathExtension:pathExtension];
                if(![[NSFileManager defaultManager] isReadableFileAtPath:[url path]])
                {
                    resizer.outputURL = url;
                    dispatch_async(newQ, ^{
                        [resizer start:nil];
                    });
                    dispatch_semaphore_wait(waitSemaphore, DISPATCH_TIME_FOREVER);
                }
            }
        }
        dispatch_release(newQ);
        
        [resizer release]; // 2013-12-04steve fix LEAK
        
    }
    
    NSUInteger i = 0;
    const NSString * key;
    CMTime insertionPoint = kCMTimeZero;
    CMTime duration = kCMTimeZero;
    CMTime transitionDuration = CMTimeMake(1, 1);
    CMTimeRange selectedRange;
    NSError * error = nil;
    NSValue * v;
    AVMutableComposition * mutableComposition = [AVMutableComposition composition];
    AVMutableVideoComposition * mutableVideoComposition = nil;
    for (NSDictionary * dict in assets)
    {
        // (required) TepsAV_AssetURL: an NSURL for each asset file
        key = TepsAV_AssetURL;
        if( (url = dict[key]) )
        {
            NSURL *testURL = [[url URLByDeletingPathExtension] URLByAppendingPathExtension:pathExtension];
            
            if([[NSFileManager defaultManager] isReadableFileAtPath:[testURL path]])
            {
                url = testURL;
            }
            
            asset = [AVURLAsset URLAssetWithURL:url options:@{AVURLAssetPreferPreciseDurationAndTimingKey:@YES}];
            //asset = [AVAsset assetWithURL:url];
        }
        else
        {
            NSAssert(NO, @"Dictionary does not include required value for key %@ for asset %@",key,url);
            
        }
        AVAssetTrack *videoTrack = nil;
        AVAssetTrack *audioTrack = nil;
        // Check if the asset contains video and audio tracks
        if ([[asset tracksWithMediaType:AVMediaTypeVideo] count] != 0)
        {
            videoTrack = [[asset tracksWithMediaType:AVMediaTypeVideo] objectAtIndex:0];
        }
        if ([[asset tracksWithMediaType:AVMediaTypeAudio] count] != 0)
        {
            audioTrack = [[asset tracksWithMediaType:AVMediaTypeAudio] objectAtIndex:0];
        }
        
        // (required) TepsAV_UseableTimeRange: NSValue encoding CMTimeRange for the usable portion of the asset
        key = TepsAV_UseableTimeRange;
        if( (v = dict[key]) )
        {
            selectedRange = v.CMTimeRangeValue;
        }
        else
        {
            NSLog(@"Getting asset duration for asset %@",asset);
            selectedRange = CMTimeRangeMake(kCMTimeZero, asset.duration);
            CFStringRef strRef;
            NSLog(@"Got range %@ for asset %@",(strRef = CMTimeRangeCopyDescription(NULL, selectedRange)),asset);
            CFRelease(strRef);
            // fail here
            //NSAssert(NO, @"Dictionary does not include required value for key %@ for asset %@",key,url);
        }
        duration = CMTimeAdd(duration, selectedRange.duration);
        // Insert selection of video and audio tracks from AVAsset
        AVMutableCompositionTrack * vtrack = nil;
        AVMutableCompositionTrack * atrack;
        if(videoTrack)
        {
            vtrack = [mutableComposition mutableTrackCompatibleWithTrack:videoTrack] ?: [mutableComposition addMutableTrackWithMediaType:AVMediaTypeVideo preferredTrackID:kCMPersistentTrackID_Invalid];
            
            [vtrack insertTimeRange:selectedRange ofTrack:videoTrack atTime:insertionPoint error:&error];
        }
        if(audioTrack != nil)
        {
            atrack = [mutableComposition mutableTrackCompatibleWithTrack:audioTrack] ?:[mutableComposition addMutableTrackWithMediaType:AVMediaTypeAudio preferredTrackID:kCMPersistentTrackID_Invalid];
            [atrack insertTimeRange:selectedRange ofTrack:audioTrack atTime:insertionPoint error:&error];
        }
        
        AVMutableVideoCompositionInstruction * segmentInstruction = [AVMutableVideoCompositionInstruction videoCompositionInstruction];
        CGColorRef bgcolor = CGColorCreateGenericRGB(1.0, 0.5, 0.2, 1.0);
        segmentInstruction.backgroundColor = bgcolor;
        CGColorRelease(bgcolor);
        
        segmentInstruction.timeRange = CMTimeRangeMake(insertionPoint, selectedRange.duration);
        AVMutableVideoCompositionLayerInstruction * segmentLayer = [AVMutableVideoCompositionLayerInstruction videoCompositionLayerInstructionWithAssetTrack:videoTrack];
        CMTime rampStart = CMTimeAdd(insertionPoint, selectedRange.duration);
        CMTime rampDuration = kCMTimeZero;
        if (CMTIME_COMPARE_INLINE(selectedRange.duration, >=, transitionDuration))
        {
            rampStart = CMTimeSubtract(rampStart, transitionDuration);
            rampDuration = transitionDuration;
        }
        ////////////////
        //rampStart = insertionPoint;
        //rampDuration = selectedRange.duration;
        ////////////////
        [segmentLayer setOpacityRampFromStartOpacity:1.0f toEndOpacity:0.0f timeRange:CMTimeRangeMake(rampStart, rampDuration)];
        segmentInstruction.layerInstructions = @[segmentLayer];
        if(nil == mutableVideoComposition)
        {
            rampStart = CMTimeMakeWithSeconds(1.0f/videoTrack.nominalFrameRate, 30);
            // on maverix we could use videoCompositionWithPropertiesOfAsset:
            mutableVideoComposition = [AVMutableVideoComposition videoComposition];
            mutableVideoComposition.frameDuration = CMTimeMakeWithSeconds(1.0f/videoTrack.nominalFrameRate, 30); // 30 fps
            mutableVideoComposition.renderSize = mutableComposition.naturalSize;
            mutableVideoComposition.instructions = [NSArray array];
        }
        mutableVideoComposition.instructions = [mutableVideoComposition.instructions arrayByAddingObject:segmentInstruction];
        
        
        insertionPoint = CMTimeAdd(insertionPoint, selectedRange.duration);
        i++;
    }
    
    // (required) TepsAV_UseableTimeRange: NSValue encoding CMTimeRange for the usable portion of the asset
    key = TepsAV_OutputURL;
    NSURL * outputURL = ticket[key];
    NSAssert(outputURL, @"ExportTicket does not include required value for key %@",key);
    
    
    // Create an export session with the composition
    //    AVAssetExportSession *exportSession = [[AVAssetExportSession alloc] initWithAsset:mutableComposition presetName:AVAssetExportPresetPassthrough]; //AVAssetExportPreset1280x720 AVAssetExportPresetPassthrough
    
    
    
    NSLog(@"exportPresetsCompatibleWithAsset = %@", [AVAssetExportSession exportPresetsCompatibleWithAsset:mutableComposition] );
    
    
#ifndef NSAppKitVersionNumber10_8
#define NSAppKitVersionNumber10_8 1187
#endif
    
    NSString * exportPreset;
    if (floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_8)
    {
        /* On a 10.8 or earlier */
        exportPreset = AVAssetExportPreset1280x720;
        exportPreset = AVAssetExportPreset960x540;
    } else {
        /* 10.9 or later system */
        exportPreset = AVAssetExportPresetPassthrough; // this will NOT work correctly on 10.9 - cant playback
        exportPreset = AVAssetExportPreset1920x1080;
        exportPreset = AVAssetExportPreset1280x720;
        exportPreset = AVAssetExportPreset960x540;
    }
    
    exportPreset = AVAssetExportPresetPassthrough; // AVAssetExportPreset1280x720; // tyler set this as a test on 2013-11-18
    
    // Create an export session with the composition
    AVAssetExportSession *exportSession = [[[AVAssetExportSession alloc] initWithAsset:mutableComposition presetName:exportPreset] autorelease]; // 2013-12-04steve LEAK??

    //    exportSession.videoComposition = mutableVideoComposition;
    exportSession.outputURL = outputURL;
    exportSession.outputFileType=AVFileTypeQuickTimeMovie;
    
    [exportSession exportAsynchronouslyWithCompletionHandler:^(void){
        switch (exportSession.status) {
            case AVAssetExportSessionStatusCompleted:
                NSLog(@"%@ - Export FINALLY completed to :%@",[self class],exportSession.outputURL.path);
                
                // do something here
                break;
            case AVAssetExportSessionStatusFailed:
                //
                NSLog(@"Failed:%@",exportSession.error);
                break;
            case AVAssetExportSessionStatusCancelled:
                //
                NSLog(@"Canceled:%@",exportSession.error);
                break;
            default:
                NSLog(@"Other:(Status %ld),%@",exportSession.status,exportSession.error);
                break;
        }
    }];
    
 // we need a progress timer during the conversion of each movie file - not during
// 2013-11-19steve   [[NSTimer scheduledTimerWithTimeInterval:2.5 target:self selector:@selector(progressTimer:) userInfo:exportSession repeats:YES] retain];

    [aTimer invalidate]; // 2013-11-19steve
    
}

-(void)progressTimer2:(NSTimer*)aTimer
{
    NSLog(@"2013-11-19steve %@...", NSStringFromSelector(_cmd));

}

-(void)progressTimer:(NSTimer *)aTimer
{
    NSLog(@"2013-11-19steve %@...", NSStringFromSelector(_cmd));
    
    //check the movie
    if ([aTimer.userInfo isKindOfClass:AVAssetExportSession.class])
    {
        AVAssetExportSession * exportSession = aTimer.userInfo;
        NSLog(@"current export progress:%.2f for movie:%@",exportSession.progress,exportSession.outputURL);
        
        switch (exportSession.status) {
            case AVAssetExportSessionStatusCompleted:
            case AVAssetExportSessionStatusFailed:
            case AVAssetExportSessionStatusCancelled:
                
                // do something usefule here ... update a progress bar or something
                NSLog(@"progressTimer Export completed to :%@",exportSession.outputURL.path);
                [aTimer invalidate];
                [aTimer release];
                [exportSession release];
                // do something here
                break;
                //
            default:
                //NSLog(@"Other:(Status %ld)",exportSession.status);
                break;
        }
    }
}


-(IBAction)mergeVideos:(id)sender
{
    BOOL doMergeVideo = NO;
    NSLog(@"%@-%@ -> doMergeVideo is %@", [self class], NSStringFromSelector(_cmd), doMergeVideo?@"YES":@"NO");
    
    if (doMergeVideo==NO) {
        return;
    }
    
    NSString* currentFeatureVideoPath = self.movieRef;
    
	NSString* endingChapterMovieName = [[NSUserDefaults standardUserDefaults] stringForKey:sPrefEndingChapterNameKey];
	NSString* beginningChapterMovieName = [[NSUserDefaults standardUserDefaults] stringForKey:sPrefBeginningChapterNameKey];
	NSString* masterDvdFolderPath = [[NSUserDefaults standardUserDefaults] stringForKey:sPrefMasterDvdFolderKey];
    
    NSLog(@"   endingChapterMovieName = [%@]",endingChapterMovieName);
    NSLog(@"beginningChapterMovieName = [%@]",beginningChapterMovieName);
    NSLog(@"      masterDvdFolderPath = [%@]",masterDvdFolderPath);
    NSLog(@"       self.mAVMovieAsset = [%@]",self.mAVMovieAsset);
    
    
    NSURL * chapter1URL = [NSURL URLWithString:@"file:///Users/Shared/MasterDvdContentFolder/chapter1.mov"];
    NSURL * chapter3URL = [NSURL URLWithString:@"file:///Users/Shared/MasterDvdContentFolder/chapter3.mov"];
    NSURL * featureURL = [NSURL URLWithString:@"file:///Users/Shared/customer.mov"];
    NSURL * outputURL = [NSURL URLWithString:@"file:///tmp/MergeMovie.mov"];
    
    if ( endingChapterMovieName && beginningChapterMovieName && masterDvdFolderPath && currentFeatureVideoPath) {
        NSString* tempPath = nil;
        
        tempPath = [masterDvdFolderPath stringByAppendingPathComponent:beginningChapterMovieName];
        chapter1URL = [NSURL fileURLWithPath:tempPath];
        
        tempPath = [masterDvdFolderPath stringByAppendingPathComponent:endingChapterMovieName];
        chapter3URL = [NSURL fileURLWithPath:tempPath];
        
        tempPath = currentFeatureVideoPath; // [masterDvdFolderPath stringByAppendingPathComponent:currentFeatureVideoPath];
        featureURL = [NSURL fileURLWithPath:tempPath];
        
            
        tempPath = @"/tmp/MergeMovie.mov";
        outputURL = [NSURL fileURLWithPath:tempPath];
    }
    
    NSLog(@"chapter1URL = %@",chapter1URL);
    NSLog(@"chapter3URL = %@",chapter3URL);
    NSLog(@" featureURL = %@",featureURL);
    NSLog(@"  outputURL = %@",outputURL);
    
    if ( chapter1URL && chapter3URL && featureURL && outputURL ) {
        
        // delete the last temp
        [[NSFileManager defaultManager] removeItemAtURL:outputURL error:NULL];
        
        // setup params for merging
        NSDictionary * d = @{TepsAV_AssetsArray : @[@{TepsAV_AssetURL : chapter1URL},@{TepsAV_AssetURL : featureURL},@{TepsAV_AssetURL : chapter3URL}], TepsAV_OutputURL:outputURL};
        
        dispatch_queue_t newQ = dispatch_queue_create(NULL, NULL);
        dispatch_async(newQ, ^{
            [self processAVOutputTicket:d];
        });
        dispatch_release(newQ);
    }
    
    return;
}

-(IBAction)outputFinal:(id)sender
{
    [self mergeVideos:sender];
    return;
    
    
    NSURL * chapter1URL = [NSURL URLWithString:@"file:///Users/Shared/MasterDvdContentFolder/chapter1.mov"];
    NSURL * chapter3URL = [NSURL URLWithString:@"file:///Users/Shared/MasterDvdContentFolder/chapter3.mov"];
    NSURL * featureURL = [NSURL URLWithString:@"file:///Users/Shared/customer.mov"];
    NSURL * outputURL = [NSURL URLWithString:@"file:///tmp/MergeMovie.mov"];
    [[NSFileManager defaultManager] removeItemAtURL:outputURL error:NULL];
    
    NSDictionary * d = @{TepsAV_AssetsArray : @[@{TepsAV_AssetURL : chapter1URL},@{TepsAV_AssetURL : featureURL},@{TepsAV_AssetURL : chapter3URL}], TepsAV_OutputURL:outputURL};
    [self processAVOutputTicket:d];
    return;
}





#pragma mark -



- (id)initWithWindow:(NSWindow *)window
{
    self = [super initWithWindow:window];
    if (self) {
        // Initialization code here.
        [self addObserver:self forKeyPath:@"movieRef" options:NSKeyValueObservingOptionNew context:NULL];
        //    [NSNotificationCenter.defaultCenter addObserver:self selector:@selector(gotNote:) name:nil object:nil];
        //        [NSNotificationCenter.defaultCenter addObserver:self selector:@selector(movieTimeDidChange:) name:QTMovieTimeDidChangeNotification object:nil];
        //        [NSNotificationCenter.defaultCenter addObserver:self selector:@selector(movieRateDidChange:) name:QTMovieRateDidChangeNotification object:nil];
        
        self.handleAsSheet = NO; // 2013-11-20steve
        
        waitSemaphore = dispatch_semaphore_create(0); // 2013-12-04steve moved up here

    }
    // 2013-12-04steve move up waitSemaphore = dispatch_semaphore_create(0);
    
    return self;
}
//-(void)playbackTimer:(NSTimer *)aTimer
//{
//    //check the movie
//    if ([aTimer.userInfo isKindOfClass:QTMovie.class])
//    {
//        QTMovie * movie = aTimer.userInfo;
//        NSTimeInterval currentTime;
//        QTGetTimeInterval(movie.currentTime,&currentTime);
//        //NSLog(@"current time:%.2f for movie:%@",currentTime,movie);
//        self.mSlideHead.doubleValue = currentTime;
//        if(currentTime >= self.setOut || self.setOut - currentTime < 0.01)
//        {
//            NSLog(@"stopping movie due to outTime:%.2f",self.setOut);
//            [movie stop];
//        }
//    }
//}
//-(void)movieTimeDidChange:(NSNotification *)note
//{
//    // reset the playhead
//    if([note.object isKindOfClass:QTMovie.class])
//    {
//        QTMovie * movie = note.object;
//        NSTimeInterval currentTime;
//        QTGetTimeInterval(movie.currentTime,&currentTime);
//        self.mSlideHead.doubleValue = currentTime;
//    }
//
//}
//-(void)movieRateDidChange:(NSNotification *)note
//{
//    //NSLog(@"%@",note);
//    if([note.name isEqualToString:QTMovieRateDidChangeNotification] && [note.object isKindOfClass:QTMovie.class])
//    {
//        // 2013-10-09steve NSNumber *newRate = note.userInfo[QTMovieRateDidChangeNotificationParameter];
//        NSNumber *newRate = [note.userInfo objectForKey:QTMovieRateDidChangeNotificationParameter];
//        if ([newRate isEqualToNumber:@0])
//        {
//            //kill the playback timer
//            NSLog(@"Killing playbacktimer for movie %@",note.object);
//            [self.playTimer invalidate];
//            self.playTimer = nil;
//        }
//        else
//        {
//            [self.playTimer invalidate];
//            self.playTimer = [NSTimer scheduledTimerWithTimeInterval:0.01 target:self selector:@selector(playbackTimer:) userInfo:note.object repeats:YES];
//        }
//
//    }
//}
-(void)gotNote:(NSNotification *)note
{
    NSString * name = note.name;
    if([name isEqualToString:@"BufferBackingWillDeallocate"]);
    else if([name isEqualToString:@"NSWindowDidUpdateNotification"]);
    else if([name isEqualToString:@"NSApplicationDidUpdateNotification"]);
    else if([name hasPrefix:@"FigImageQueueNotification"]);
    else
    {
        NSLog(@"%@",note);
    }
}
- (void)dealloc
{
    [NSNotificationCenter.defaultCenter removeObserver:self];
    [self removeObserver:self forKeyPath:@"movieRef"];
    [self removeObserver:self forKeyPath:@"player.rate"];
    [self removeObserver:self forKeyPath:@"player.currentItem.status"];
    [self removeObserver:self forKeyPath:@"playerLayer.readyForDisplay"];
    
    self.TOToken = nil;
    self.BOToken = nil;
    
    self.mAVMovieAsset = nil;
    self.playerView = nil;
    self.player = nil;
    self.playerLayer = nil;
    self.movieRef = nil;
    self.mSlideIn = nil;
    self.mSlideOut = nil;
    self.mSlideHead = nil;
    self.revertButton = nil;
    self.tSlideInValue = nil;
    self.tSlideOutValue = nil;
    self.tSelectionLength = nil;
    
    self.loadingSpinner = nil;
    self.playPauseButton = nil;
    self.composition = nil;
    //    self.videoComposition = nil;
    //    self.audioMix = nil;
    self.playTimer = nil;
    
    dispatch_release(waitSemaphore);
    
    [super dealloc];
}
- (void)windowDidLoad
{
    [super windowDidLoad];
    self.window.delegate = self;
    
    // Implement this method to handle any initialization after your window controller's window has been loaded from its nib file.
    [self setup];
    [self loadMovie];
}
- (BOOL)windowShouldClose:(id)sender
{
    return NO;
}
- (BOOL)windowShouldZoom:(NSWindow *)window toFrame:(NSRect)newFrame
{
    return NO;
}

-(void)setSliders
{
    NSTimeInterval duration = self.duration;
    {
        // got the duration
        //now set the selection sliders
        self.mSlideIn.minValue = 0;
        self.mSlideOut.minValue = 0;
        self.mSlideHead.minValue = 0;
        
        // now using binding
        //        self.mSlideIn.maxValue = duration;
        //        self.mSlideOut.maxValue = duration;
        //        self.mSlideHead.maxValue = duration;
        
        self.setIn = 0;
        self.mSlideIn.doubleValue = 0;
        self.tSlideInValue.stringValue = [self timeStringForTimeInterval:self.setIn];
        
        self.setOut = duration;
        self.mSlideOut.doubleValue = duration;
        self.tSlideOutValue.stringValue = [self timeStringForTimeInterval:self.setOut];
        
        [self setMovieSelection];
    }
}
+ (NSSet *)keyPathsForValuesAffectingDuration
{
	return [NSSet setWithObjects:@"player.currentItem", @"player.currentItem.status", nil];
}

- (double)duration
{
	AVPlayerItem *playerItem = [[self player] currentItem];
	
	if ([playerItem status] == AVPlayerItemStatusReadyToPlay)
		return CMTimeGetSeconds([[playerItem asset] duration]);
	else
		return 0.f;
}

- (double)currentTime
{
	return CMTimeGetSeconds([[self player] currentTime]);
}

- (void)setCurrentTime:(double)time
{
	[[self player] seekToTime:CMTimeMakeWithSeconds(time, 100)];
}
-(void)loadMovie
{
    NSURL * assetURL = nil;
    AVURLAsset * asset = nil;
    if([self.movieRef isKindOfClass:NSString.class])
    {
        assetURL = [NSURL fileURLWithPath:self.movieRef];
    }
    else if([self.movieRef isKindOfClass:NSURL.class])
    {
        assetURL = self.movieRef;
    }
    if(assetURL)
    {
        [[NSFileManager defaultManager] removeItemAtPath:[self getTrimmedMoviePath] error:nil]; // 2013-11-03steve
        
        // Create an asset with our URL, asychronously load its tracks, its duration, and whether it's playable or protected.
        // When that loading is complete, configure a player to play the asset.
        asset = [AVAsset assetWithURL:assetURL];
        self.mAVMovieAsset = asset;
        
        NSArray *assetKeysToLoadAndTest = [NSArray arrayWithObjects:@"playable", @"hasProtectedContent", @"tracks", @"duration", nil];
        [asset loadValuesAsynchronouslyForKeys:assetKeysToLoadAndTest completionHandler:^(void) {
            
            // The asset invokes its completion handler on an arbitrary queue when loading is complete.
            // Because we want to access our AVPlayer in our ensuing set-up, we must dispatch our handler to the main queue.
            dispatch_async(dispatch_get_main_queue(), ^(void) {
                
                [self setUpPlaybackOfAsset:asset withKeys:assetKeysToLoadAndTest];
                
            });
            
        }];
    }
}
-(void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    if([keyPath isEqualToString:@"movieRef"])
    {
        NSLog(@"someone set a new movie");
        [self loadMovie];
    }
    else if([keyPath isEqualToString:@"player.currentItem.status"])
	{
		AVPlayerStatus status = [[change objectForKey:NSKeyValueChangeNewKey] integerValue];
		BOOL enable = NO;
		switch (status)
		{
			case AVPlayerItemStatusUnknown:
				break;
			case AVPlayerItemStatusReadyToPlay:
				enable = YES;
                [self setSliders];
                
				break;
			case AVPlayerItemStatusFailed:
				[self stopLoadingAnimationAndHandleError:[[[self player] currentItem] error]];
				break;
		}
		
		[[self playPauseButton] setEnabled:enable];
        //		[[self fastForwardButton] setEnabled:enable];
        //		[[self rewindButton] setEnabled:enable];
	}
    else if([keyPath isEqualToString:@"player.rate"])
	{
		float rate = [[change objectForKey:NSKeyValueChangeNewKey] floatValue];
		if (rate != 1.f)
		{
			// [[self playPauseButton] setTitle:@"Play"];
            [[self playPauseButton] setImage:[NSImage imageNamed:@"PlayON"]]; // 2013-10-22steve
		}
		else
		{
			// [[self playPauseButton] setTitle:@"Pause"];
            [[self playPauseButton] setImage:[NSImage imageNamed:@"PauseOFF"]]; // 2013-10-22steve
		}
	}
    else if([keyPath isEqualToString:@"playerLayer.readyForDisplay"])
	{
		if ([[change objectForKey:NSKeyValueChangeNewKey] boolValue] == YES)
		{
			// The AVPlayerLayer is ready for display. Hide the loading spinner and show it.
			[self stopLoadingAnimationAndHandleError:nil];
			[[self playerLayer] setHidden:NO];
		}
	}
	else
	{
		[super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
	}
}
- (void)exportWillBegin
{
    // Hide play until the export is complete
    [[self playPauseButton] setEnabled:NO];
    self.loadingSpinner.hidden = NO;
    [self.loadingSpinner startAnimation:self];
}
- (void)exportDidEnd
{
    // Update UI after export is completed
    [self.loadingSpinner stopAnimation:self];
    self.loadingSpinner.hidden = YES;
    // Enable play button
    [[self playPauseButton] setEnabled:YES];
    
    NSLog(@"movie file updated!");
    self.revertButton.enabled = NO;
    
    
    
    if (self.handleAsSheet) { // 2013-11-20steve
        
        [NSApp endSheet:self.window returnCode:-3333]; // 2013-11-20steve
        
    } else {

        [self mergeVideos:nil]; // 2013-11-19steve
        
        // 2013-10-08steve
        NSLog(@"self.mydelegate = %@", self.mydelegate);
        if (self.mydelegate && [self.mydelegate respondsToSelector:@selector(didSaveEditedMovie:)])
        {
            [self.mydelegate didSaveEditedMovie:self];
        }
    }
    
    
}

-(NSString*) getTrimmedMoviePath // 2013-11-03steve
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDesktopDirectory, NSUserDomainMask, YES);
    NSString *outputPath = [paths objectAtIndex:0];
    NSFileManager *manager = [NSFileManager defaultManager];
    [manager createDirectoryAtPath:outputPath withIntermediateDirectories:YES attributes:nil error:nil];
    //    outputURL = [outputURL stringByAppendingPathComponent:@"output.mp4"];
    outputPath = [outputPath stringByAppendingPathComponent:@"trimmedVideoOutput.mp4"]; // 2013-11-01steve
    
    return outputPath;
}


- (void)performExportWithAsset:(AVAsset *)asset
{
    [self exportWillBegin];
    
    NSFileManager *manager = [NSFileManager defaultManager];
    
    // 2013-11-03steve - use method above
    //    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDesktopDirectory, NSUserDomainMask, YES);
    //    NSString *outputURL = [paths objectAtIndex:0];
    //    [manager createDirectoryAtPath:outputURL withIntermediateDirectories:YES attributes:nil error:nil];
    ////    outputURL = [outputURL stringByAppendingPathComponent:@"output.mp4"];
    //    outputURL = [outputURL stringByAppendingPathComponent:@"trimmedVideoOutput.mp4"]; // 2013-11-01steve
    
    NSString* outputURL = [self getTrimmedMoviePath]; // 2013-11-03steve
    
    // Remove Existing File
    [manager removeItemAtPath:outputURL error:nil];
    
    //dispatch_queue_t exportQueue = dispatch_queue_create("Export Queue", NULL);
    // Create an export session with the composition
    AVAssetExportSession *exportSession = [[[AVAssetExportSession alloc] initWithAsset:self.composition  presetName:AVAssetExportPresetPassthrough] autorelease];
    
    //    exportSession.videoComposition = self.videoComposition;
    //    exportSession.audioMix = self.audioMix;
    exportSession.outputURL = [NSURL fileURLWithPath:outputURL];
    exportSession.outputFileType=AVFileTypeQuickTimeMovie;
    
    [exportSession exportAsynchronouslyWithCompletionHandler:^(void){
        // NSLog(@"export completed"); // 2013-11-01steve
        switch (exportSession.status) {
            case AVAssetExportSessionStatusCompleted:
                dispatch_async(dispatch_get_main_queue(), ^(void) {
                    [self exportDidEnd];
                });
                break;
            case AVAssetExportSessionStatusFailed:
                //
                NSLog(@"exportSession Failed:%@",exportSession.error);
                break;
            case AVAssetExportSessionStatusCancelled:
                //
                NSLog(@"exportSession Canceled:%@",exportSession.error);
                break;
            default:
                NSLog(@"exportSession Unknown???:%@",exportSession.error); // 2013-11-01steve
                break;
        }
    }];
}

-(IBAction)saveWithoutEdit:(id)sender // 2013-11-03
{

    if (self.handleAsSheet) { // 2013-11-20steve

        [NSApp endSheet:self.window returnCode:-2222]; // 2013-11-20steve

    } else {
        
        [self mergeVideos:nil]; // 2013-11-19steve
        
        NSLog(@"self.mydelegate = %@", self.mydelegate);
        if (self.mydelegate && [self.mydelegate respondsToSelector:@selector(didSaveWithoutEditing:)])
        {
            [self.mydelegate didSaveWithoutEditing:self];
        }
        
    }
    
}


-(IBAction)cropAndSave:(id)sender
{
    
    if (self.composition==nil) { // 2013-11-03steve
        
        BOOL sliderDidChange = NO;
        {
            NSTimeInterval timeIn = self.mSlideIn.doubleValue;
            CMTime startTime = CMTimeMakeWithSeconds(timeIn, 100);
            NSTimeInterval selectionDuration = MAX(self.mSlideOut.doubleValue - timeIn, 0);
            
            NSLog(@"self.originalAssetDuration = %5.2f", self.originalAssetDuration );
            NSLog(@"         selectionDuration = %5.2f", selectionDuration );
            
            
            if ( self.originalAssetDuration != selectionDuration) {
                NSLog(@"User Changed Settings");
                sliderDidChange = YES;
                
                NSAlert * alert = [NSAlert alertWithMessageText:@"You Changed the Sliders?" defaultButton:@"Cancel" alternateButton:@"Purchase Original Movie" otherButton:@"Trim and Purchase" informativeTextWithFormat:@"Do Want to Purchase Original Movie or Trimmed Movie"];
                NSImage * icon = [NSImage imageNamed:NSImageNameCaution];
                if(icon)
                {
                    [alert setIcon:icon];
                }
                [alert beginSheetModalForWindow:[self window] modalDelegate:self didEndSelector:@selector(alertDidEnd2:returnCode:contextInfo:) contextInfo:NULL];
                return;
            }
            
            
        }
        
        // 2013-11-03steve this doesnt tell us if user DID MOVE the Sliders but Did NOT Click on Trim button
        
        [self saveWithoutEdit:nil];
        
        // [self trim:nil]; // 2013-11-01steve
        
        return;
        
    }
    
    
    //export the movie
    [self performExportWithAsset:nil];
    
}
-(IBAction)cancel:(id)sender
{
    if (self.handleAsSheet) { // 2013-11-20steve
        
        [NSApp endSheet:self.window returnCode:NSRunAbortedResponse];
        
    } else {
        // 2013-10-08steve
        NSLog(@"self.mydelegate = %@", self.mydelegate);
        if (self.mydelegate && [self.mydelegate respondsToSelector:@selector(didCancelEditedMovie:)])
        {
            [self.mydelegate didCancelEditedMovie:self];
        }
    }
}

-(CMTimeRange)setMovieSelection
{
    //get the movie duration and set the in selection to this percent of duration
    NSTimeInterval timeIn = self.mSlideIn.doubleValue;
    CMTime startTime = CMTimeMakeWithSeconds(timeIn, 100);
    NSTimeInterval selectionDuration = MAX(self.mSlideOut.doubleValue - timeIn, 0);
    CMTime duration = CMTimeMakeWithSeconds(selectionDuration, 100);
    CMTimeRange selection = CMTimeRangeMake(startTime, duration);
    self.tSelectionLength.stringValue = [self timeStringForTimeInterval:selectionDuration];
    return selection;
}
-(NSString *)timeStringForTimeInterval:(NSTimeInterval)aTime
{
    NSString * s = @"0.00";
    NSTimeInterval secs;
    unsigned hundreths = modf(aTime, &secs)*100;
    unsigned minutes = secs/60;
    unsigned seconds = secs - minutes*60;
    if(minutes)
    {
        s = [NSString stringWithFormat:@"%u:%.2u.%.2u",minutes,seconds,hundreths];
    }
    else if (seconds)
    {
        s = [NSString stringWithFormat:@"%u.%.2u",seconds,hundreths];
    }
    else
    {
        s = [NSString stringWithFormat:@"0.%.2u",hundreths];
    }
    return s;
}
-(IBAction)setInSelection:(id)sender
{
    self.setIn = self.mSlideIn.doubleValue;
    //    self.tSlideInValue.doubleValue = self.setIn;
    self.tSlideInValue.stringValue = [self timeStringForTimeInterval:self.setIn];
    [self setMovieSelection];
    // move play head to slideIn
    CMTime seekTime = CMTimeMakeWithSeconds(self.setIn,100);
    CMTime tol = CMTimeMake(2, 100);
    [self.player seekToTime:seekTime toleranceBefore:tol toleranceAfter:tol];
    //    [self.mQTMovie setCurrentTime:QTMakeTimeWithTimeInterval(self.mSlideIn.doubleValue)];
}
-(IBAction)setOutSelection:(id)sender
{
    Float64 outTime = self.mSlideOut.doubleValue;
    self.setOut = outTime;
    //    self.tSlideOutValue.doubleValue = self.setOut;
    self.tSlideOutValue.stringValue = [self timeStringForTimeInterval:outTime];
    
    [self setMovieSelection];
    CMTime seekTime = CMTimeMakeWithSeconds(self.setOut,100);
    CMTime tol = CMTimeMake(2, 100);
    [self.player seekToTime:seekTime toleranceBefore:tol toleranceAfter:tol];
    
    //    [self.mQTMovie gotoEnd];
    //    [self.mQTMovie gotoPreviousSelectionPoint];
    //    [self.mQTMovie setCurrentTime:QTMakeTimeWithTimeInterval(self.mSlideOut.doubleValue)];
    self.BOToken = [self.player addBoundaryTimeObserverForTimes:@[[NSValue valueWithCMTime:CMTimeMakeWithSeconds(outTime,100)]] queue:dispatch_get_main_queue() usingBlock:^{
		// just stop playback
        [self.player pause];
        [self.player seekToTime:CMTimeMakeWithSeconds(outTime,100) toleranceBefore:kCMTimeZero toleranceAfter:kCMTimeZero];
	}];
    
}
-(IBAction)setPlayHead:(id)sender
{
    //[self.mQTMovie setCurrentTime:QTMakeTimeWithTimeInterval(self.mSlideHead.doubleValue)];
}
-(IBAction)playSelection:(id)sender
{
    if(self.player.rate == 0.0f)
    {
        //play current selection
        // check if the slideOut was set and play from SlideIn
        Float64 currentTime = CMTimeGetSeconds(self.player.currentTime);
        double tDiff = currentTime - self.setOut;
        NSLog(@"tDiff = %f",tDiff);
        if(0 < tDiff || -0.05 < tDiff)
        {
            // move play head to slideIn
            CMTime seekTime = CMTimeMakeWithSeconds(self.setIn,100);
            [self.player seekToTime:seekTime toleranceBefore:kCMTimeZero toleranceAfter:kCMTimeZero completionHandler:^(BOOL finished) {
                if(finished)
                {
                    [self.player play];
                }
            }];
        }
        else
        {
            [self.player play];
        }
    }
    else
    {
        [self.player pause];
    }
}
-(IBAction)revertToOriginal:(id)sender
{
    // reload the file
    //get confirmation before discarding changes
    NSAlert * alert = [NSAlert alertWithMessageText:@"Revert to Original?" defaultButton:@"Revert" alternateButton:@"Cancel" otherButton:nil informativeTextWithFormat:@"All changes to this point will be lost."];
    NSImage * icon = [NSImage imageNamed:NSImageNameCaution];
    if(icon)
    {
        [alert setIcon:icon];
    }
    [alert beginSheetModalForWindow:[self window] modalDelegate:self didEndSelector:@selector(alertDidEnd:returnCode:contextInfo:) contextInfo:NULL];
}
- (void)alertDidEnd:(NSAlert *)alert returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo
{
    NSLog(@"%@-%@...", self,NSStringFromSelector(_cmd)); // 2013-11-20steve

    if(NSAlertDefaultReturn == returnCode)
    {
        //revert to original movie
        NSLog(@"Reverting to original");
        //        [self loadMovie];
        self.BOToken = nil;
        AVPlayerItem *playerItem = [AVPlayerItem playerItemWithAsset:self.mAVMovieAsset];
        [[self player] replaceCurrentItemWithPlayerItem:playerItem];
        self.revertButton.enabled = NO;
        
        self.composition = nil; // 2013-11-03steve - is this okay to do???
        
    }
}

// 2013-11-03steve
- (void)alertDidEnd2:(NSAlert *)alert returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo
{
    
    switch (returnCode) {
        case NSAlertDefaultReturn:
            NSLog(@"Cancel");
            break;
        case NSAlertOtherReturn:
            NSLog(@"Trim and Purchase Movie");
            
            [self performTrimWithAsset:self.mAVMovieAsset]; // 2013-11-11steve perform trim first
            
            //export the movie
            [self performExportWithAsset:nil];
            break;
        case NSAlertAlternateReturn:
            NSLog(@"Purchase ORIGINAL Movie");
            [self saveWithoutEdit:nil];
            break;
            
        default:
            break;
    }
    
}


- (void)setTOToken:(id)timeObserverToken
{
    if(self.timeObserverToken != nil)
    {
        [[self player] removeTimeObserver:self.timeObserverToken];
    }
    self.timeObserverToken = timeObserverToken;
    // here we need to set the actual token
}
- (void)setBOToken:(id)boundaryObserverToken
{
    if(self.boundaryObserverToken != nil)
    {
        [[self player] removeTimeObserver:self.boundaryObserverToken];
    }
    self.boundaryObserverToken = boundaryObserverToken;
    // here we need to set the actual token
}
- (void)reloadPlayerView
{
    // This method is called every time a tool has been applied to a composition
    // It reloads the player view with the updated composition
    // Create a new AVPlayerItem and make it our player's current item.
    // Pause the player till the playerview is reloaded with the new playerItem
    if(self.composition)
    {
        [[self player] pause];
        //    self.videoComposition.animationTool = nil;
        AVPlayerItem *playerItem = [AVPlayerItem playerItemWithAsset:self.composition];
        //    if ([[self noVideoLabel] isHidden]) {
        //        playerItem.videoComposition = self.videoComposition;
        //        playerItem.audioMix = self.audioMix;
        //    }
        [[self player] replaceCurrentItemWithPlayerItem:playerItem];
        
        //    // Resize window to adjust for rotation
        //	if(newSize.width != 0){
        //        NSRect oldFrame = [[self window] frame];
        //        oldFrame.size.width = newSize.width;
        //        oldFrame.size.height = newSize.height;
        //        [[self window] setFrame:oldFrame display:YES animate:NO];
        //    }
        
        self.TOToken = [self.player addPeriodicTimeObserverForInterval:CMTimeMake(1, 10) queue:dispatch_get_main_queue() usingBlock:^(CMTime time) {
            [[self mSlideHead] setDoubleValue:CMTimeGetSeconds(time)];
            
            //        UInt64 currentTimeSec = time.value / time.timescale;
            //        UInt64 minutes = currentTimeSec / 60;
            //        UInt64 seconds = currentTimeSec % 60;
            //        NSString *playbackTimeLabel = [NSString stringWithFormat:
            //                                       @"%02lld:%02lld", minutes, seconds];
            //        [[self timeElapsed] setStringValue:playbackTimeLabel];
        }];
        
    }
}

- (void)performTrimWithAsset:(AVAsset*)asset
{
    AVAssetTrack *videoTrack = nil;
    AVAssetTrack *audioTrack = nil;
    // Check if the asset contains video and audio tracks
    if ([[asset tracksWithMediaType:AVMediaTypeVideo] count] != 0) {
        videoTrack = [[asset tracksWithMediaType:AVMediaTypeVideo] objectAtIndex:0];
    }
    if ([[asset tracksWithMediaType:AVMediaTypeAudio] count] != 0) {
        audioTrack = [[asset tracksWithMediaType:AVMediaTypeAudio] objectAtIndex:0];
    }
    
    CMTime insertionPoint = kCMTimeZero;
    NSError * error = nil;
    
    CMTimeRange selectedRange = [self setMovieSelection];
    // Create a new composition
    if(!self.composition)
    {
        self.composition = [AVMutableComposition composition];
    }
    AVMutableComposition * mutableComposition = self.composition;
    // Insert selection of video and audio tracks from AVAsset
    if(videoTrack != nil) {
        AVMutableCompositionTrack *vtrack = [mutableComposition addMutableTrackWithMediaType:AVMediaTypeVideo preferredTrackID:kCMPersistentTrackID_Invalid];
        [vtrack insertTimeRange:selectedRange ofTrack:videoTrack atTime:insertionPoint error:&error];
    }
    if(audioTrack != nil) {
        AVMutableCompositionTrack *atrack = [mutableComposition addMutableTrackWithMediaType:AVMediaTypeAudio preferredTrackID:kCMPersistentTrackID_Invalid];
        [atrack insertTimeRange:selectedRange ofTrack:audioTrack atTime:insertionPoint error:&error];
    }
    [self reloadPlayerView];
}

-(IBAction)trim:(id)sender
{
    [self performTrimWithAsset:self.mAVMovieAsset];
    //    [self setSliders];
    self.revertButton.enabled = YES;
}

/************************************************************************************/
-(void)setup
{
    
	[[[self playerView] layer] setBackgroundColor:CGColorGetConstantColor(kCGColorClear)]; // 2013-10-16steve
	[self.loadingSpinner startAnimation:self];
    self.loadingSpinner.hidden = NO;
	
	// Create the AVPlayer, add rate and status observers
	self.player = [[[AVPlayer alloc] init] autorelease];
	[self addObserver:self forKeyPath:@"player.rate" options:NSKeyValueObservingOptionNew context:NULL];
	[self addObserver:self forKeyPath:@"player.currentItem.status" options:NSKeyValueObservingOptionNew context:NULL];
	
    
}



//- (void)windowControllerDidLoadNib:(NSWindowController *)windowController
//{
//	[super windowControllerDidLoadNib:windowController];
//	[[windowController window] setMovableByWindowBackground:YES];
//	// [[[self playerView] layer] setBackgroundColor:CGColorGetConstantColor(kCGColorBlack)];
//	[[[self playerView] layer] setBackgroundColor:CGColorGetConstantColor(kCGColorClear)]; // 2013-10-16steve
//	[[self loadingSpinner] startAnimation:self];
//
//	// Create the AVPlayer, add rate and status observers
//	[self setPlayer:[[[AVPlayer alloc] init] autorelease]];
//	[self addObserver:self forKeyPath:@"player.rate" options:NSKeyValueObservingOptionNew context:AVSPPlayerRateContext];
//	[self addObserver:self forKeyPath:@"player.currentItem.status" options:NSKeyValueObservingOptionNew context:AVSPPlayerItemStatusContext];
//
//	// Create an asset with our URL, asychronously load its tracks, its duration, and whether it's playable or protected.
//	// When that loading is complete, configure a player to play the asset.
//	AVURLAsset *asset = [AVAsset assetWithURL:[self fileURL]];
//	NSArray *assetKeysToLoadAndTest = [NSArray arrayWithObjects:@"playable", @"hasProtectedContent", @"tracks", @"duration", nil];
//	[asset loadValuesAsynchronouslyForKeys:assetKeysToLoadAndTest completionHandler:^(void) {
//
//		// The asset invokes its completion handler on an arbitrary queue when loading is complete.
//		// Because we want to access our AVPlayer in our ensuing set-up, we must dispatch our handler to the main queue.
//		dispatch_async(dispatch_get_main_queue(), ^(void) {
//
//			[self setUpPlaybackOfAsset:asset withKeys:assetKeysToLoadAndTest];
//
//		});
//
//	}];
//}

- (void)setUpPlaybackOfAsset:(AVAsset *)asset withKeys:(NSArray *)keys
{
	// This method is called when the AVAsset for our URL has completing the loading of the values of the specified array of keys.
	// We set up playback of the asset here.
	
	// First test whether the values of each of the keys we need have been successfully loaded.
	for (NSString *key in keys)
	{
		NSError *error = nil;
		AVURLAsset* avurlasset = nil;
        if ([asset isKindOfClass:[AVURLAsset class]])
        {
            avurlasset = (AVURLAsset*)asset;
            NSLog(@"222asset.url = %@", avurlasset.URL);
        }
        
        NSLog(@"2013-10-16steve key,value = [%@,%@]", key, [asset valueForKey:key]);
        
		if ([asset statusOfValueForKey:key error:&error] == AVKeyValueStatusFailed)
		{
			[self stopLoadingAnimationAndHandleError:error];
			return;
		}
	}
	
	if (![asset isPlayable] || [asset hasProtectedContent])
	{
		// We can't play this asset. Show the "Unplayable Asset" label.
		[self stopLoadingAnimationAndHandleError:nil];
        //		[[self unplayableLabel] setHidden:NO];
		return;
	}
	
	// We can play this asset.
	// Set up an AVPlayerLayer according to whether the asset contains video.
	if ([[asset tracksWithMediaType:AVMediaTypeVideo] count] != 0)
	{
        AVAssetImageGenerator * gen = [AVAssetImageGenerator assetImageGeneratorWithAsset:self.mAVMovieAsset];
        [gen retain];
        NSLog(@"now calling generateCGImagesAsynchronouslyForTimes:completionHandler: with generator %@",gen);
        [gen generateCGImagesAsynchronouslyForTimes:@[[NSValue valueWithCMTime:kCMTimeZero]] completionHandler:^(CMTime requestedTime, CGImageRef image, CMTime actualTime, AVAssetImageGeneratorResult result, NSError *error) {
            if(AVAssetImageGeneratorSucceeded == result && image)
            {
                [gen release];
                CGFloat width = CGImageGetWidth(image);
                CGFloat height = CGImageGetHeight(image);
                CGFloat aspectRatio = width/height;
                //resize the window
                CGSize viewSize = CGSizeMake(self.playerView.bounds.size.width,self.playerView.bounds.size.height ); // 2013-11-01steve make release target build  self.playerView.bounds.size;
                CGFloat deltaH = ceil(viewSize.width/aspectRatio) - viewSize.height;
                NSLog(@"window should resize by %f",deltaH);
                if(deltaH)
                {
                    // resize the window by deltaH
                    NSRect wFrame = self.window.frame;
                    wFrame.size.height += deltaH;
                    wFrame.origin.y -= deltaH;
                    dispatch_async(dispatch_get_main_queue(), ^(void) {
                        NSLog(@"now resize window by %f",deltaH);
                        [self.window setFrame:wFrame display:YES animate:YES];
                    });
                }
            }
            else
            {
                NSLog(@"Error generating asset image %@",error);
            }
        }];
        //        AVAssetTrack * videoTrack = [[asset tracksWithMediaType:AVMediaTypeVideo] objectAtIndex:0];
        //        CGSize natSize = videoTrack.naturalSize;
        //        CGAffineTransform transform = videoTrack.preferredTransform;
        //        CGSize presentationSize = CGSizeApplyAffineTransform(natSize, transform);
        //        CGFloat aspectRatio = presentationSize.width/presentationSize.height;
        
        
        NSLog(@"sublayers = %@", self.playerView.layer.sublayers);
        self.playerView.layer.sublayers = nil; // 2013-10-22steve
        
		// Create an AVPlayerLayer and add it to the player view if there is video, but hide it until it's ready for display
		AVPlayerLayer *newPlayerLayer = [AVPlayerLayer playerLayerWithPlayer:[self player]];
		[newPlayerLayer setFrame:[[[self playerView] layer] bounds]];
		[newPlayerLayer setAutoresizingMask:kCALayerWidthSizable | kCALayerHeightSizable];
		[newPlayerLayer setHidden:YES];
		[[[self playerView] layer] addSublayer:newPlayerLayer];
		[self setPlayerLayer:newPlayerLayer];
        self.composition = nil;
        //        self.videoComposition = nil;
        //        self.audioMix = nil;
        //        newSize.width = [[notification object] newWindowWidth];
        //        newSize.height = [[notification object] newWindowHeight];
        //        dispatch_async(dispatch_get_main_queue(), ^(void) {
        //            [self reloadPlayerView];
        //        });
        
		[self addObserver:self forKeyPath:@"playerLayer.readyForDisplay" options:NSKeyValueObservingOptionInitial | NSKeyValueObservingOptionNew context:NULL];
	}
	else
	{
		// This asset has no video tracks. Show the "No Video" label.
		[self stopLoadingAnimationAndHandleError:nil];
        //		[[self noVideoLabel] setHidden:NO];
	}
	
	// Create a new AVPlayerItem and make it our player's current item.
	AVPlayerItem *playerItem = [AVPlayerItem playerItemWithAsset:asset];
	[[self player] replaceCurrentItemWithPlayerItem:playerItem];
	
	[self setTimeObserverToken:[[self player] addPeriodicTimeObserverForInterval:CMTimeMake(1, 10) queue:dispatch_get_main_queue() usingBlock:^(CMTime time) {
		[[self mSlideHead] setDoubleValue:CMTimeGetSeconds(time)];
	}]];
    
    
    
    [self setSliders];
    self.revertButton.enabled = NO;
    
    
    {
        // 2013-11-03steve
        AVPlayerItem *playerItem = [[self player] currentItem];
        {
            NSTimeInterval originalduration = CMTimeGetSeconds([[playerItem asset] duration]);
            self.originalAssetDuration = originalduration;
            NSLog(@" setting self.originalAssetDuration = %5.2f", self.originalAssetDuration ); // 2013-11-03steve
        }
        
    }
	
}

- (void)stopLoadingAnimationAndHandleError:(NSError *)error
{
	[[self loadingSpinner] stopAnimation:self];
	[[self loadingSpinner] setHidden:YES];
	if (error)
	{
        NSLog(@"2013-10-16steve - NEED to ERROR HERE = %@", error);
        //		[self presentError:error
        //			modalForWindow:[self windowForSheet]
        //				  delegate:nil
        //		didPresentSelector:NULL
        //			   contextInfo:nil];
	}
}
/************************************************************************************/

@end
