//
//  MyRecorderController.m
//  MyRecorder

/*
 
 Disclaimer: IMPORTANT:  This Apple software is supplied to you by 
 Apple Inc. ("Apple") in consideration of your agreement to the
 following terms, and your use, installation, modification or
 redistribution of this Apple software constitutes acceptance of these
 terms.  If you do not agree with these terms, please do not use,
 install, modify or redistribute this Apple software.
 
 In consideration of your agreement to abide by the following terms, and
 subject to these terms, Apple grants you a personal, non-exclusive
 license, under Apple's copyrights in this original Apple software (the
 "Apple Software"), to use, reproduce, modify and redistribute the Apple
 Software, with or without modifications, in source and/or binary forms;
 provided that if you redistribute the Apple Software in its entirety and
 without modifications, you must retain this notice and the following
 text and disclaimers in all such redistributions of the Apple Software. 
 Neither the name, trademarks, service marks or logos of Apple Inc. 
 may be used to endorse or promote products derived from the Apple
 Software without specific prior written permission from Apple.  Except
 as expressly stated in this notice, no other rights or licenses, express
 or implied, are granted by Apple herein, including but not limited to
 any patent rights that may be infringed by your derivative works or by
 other works in which the Apple Software may be incorporated.
 
 The Apple Software is provided by Apple on an "AS IS" basis.  APPLE
 MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
 Copyright (C) 2007-2008 Apple Inc. All Rights Reserved.  
 
 */

#import "MyRecorderController.h"
#import "NSTextField+TepsAdditions.h"
#import "MyAliasKit.h"
#import "TepsExtras.h"
#import "Preferences.h"
#import <AVFoundation/AVFoundation.h> // 2014-06-26steve
#import "ATMPPR_Task.h" // 2014-10-15steve


extern NSString* sPrefStartupMode; // 2014-07-31steve

NSString * sTepsVideoAppNotification = @"TepsVideoAppNotification"; // 2014-07-14steve
NSString * sTepsXNotification = @"tepsXNotification"; // 2014-07-14steve
NSString * sTepsPreviewerNotification = @"TepsPreviewerNotification"; // 2014-07-14steve
NSString * sDefaultMovieExtension = @"mp4"; // 2014-07-28steve


static NSString* sLocalWorkingFolderName = @"WorkingFolderForCamera";

NSString* cSantaMovieStart = @"/santaStart.mov";
NSString* cSantaMovieEnd = @"/santaEnd.mov";
NSString* cPrefRenameString = @"prefRenameStringKey";
NSString* cTempRecordedMovieBaseName = @"myRecordedMovie";
// NSString* cPrefRenameMovieAllowed = @"prefSaveMovieWithCustomName"; // 2013-10-04steve
NSString* cPrefErrorMessage = @"prefErrorMessage"; // 2013-10-04steve

static NSString* sNewCustomerIdNotification = @"NewCustomerId"; // 2014-11-05steve


@implementation MyRecorderController

@synthesize mRecordedMoviePath;
@synthesize mMergedMovieItem;
@synthesize mMovieToPlayPath;
@synthesize mTempRenameString;
@synthesize mCurWorkingPath;
@synthesize mRecordingTimer;
@synthesize fSelectedIndex; // 2013-10-10steve
@synthesize fSelectedOptionValue; // 2013-10-10steve
@synthesize mVideoStatusWindowController; // 2014-07-13steve
@synthesize fCurrentTepsCustomerID; // 2014-07-15steve
@synthesize fCurrentTepsDailyFolderPath; // 2014-07-25steve
@synthesize fWarningTimer1; // 2014-07-22steve
// @synthesize mVideoAlertWindowController2; // 2014-08-09steve

#pragma mark -



-(void) removeOldMoviesFromWorkingDirectory:(id)sender // 2014-10-08steve
{
    NSAutoreleasePool* p = [[NSAutoreleasePool alloc] init];
    
    
    NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
    
    NSString* myBundlePath = [[NSBundle mainBundle] bundlePath];
    NSLog (@ "Inside %@ => %@... myBundlePath returned = %@", self, NSStringFromSelector(_cmd), myBundlePath );
    myBundlePath = [myBundlePath stringByDeletingLastPathComponent];
    NSString* curWorkingPath = [myBundlePath stringByAppendingPathComponent:sLocalWorkingFolderName];
    NSLog (@ "Inside %@ => %@... getWorkingDirectoryPath returned = %@", self, NSStringFromSelector(_cmd), curWorkingPath );
    

    NSNumber* daysOld = @1;
    
    // first remove all old movie files
    [ATMPPR_Task removeFilesInDirectory:curWorkingPath
                                 ofKind:@[@"mov",@"mp4"]
                                daysOld:daysOld
                               maxDepth:nil];

    // remove hidden items - otherwise we cant delete "empty" folders
    [ATMPPR_Task removeFilesInDirectory:curWorkingPath
                       matchingPatterns:@[@".*"]
                                daysOld:@0  // in case someone touched folder and .ds
                               maxDepth:nil];
    
    // delete old empty folders
    [ATMPPR_Task removeSubdirectoriesAtPath:curWorkingPath
                           matchingPatterns:nil
                                    daysOld:daysOld
                                   maxDepth:nil];

    
    
    [p release];
}








#pragma mark -




// 2010-11-10steve
-(BOOL) updateWorkingDirectory
{
	NSString* curWorkingPath = getWorkingDirectoryPath(sLocalWorkingFolderName);
	
	NSLog ( @"2010-11-11steve curWorkingPath is [%@]", curWorkingPath );
	BOOL didCreate = createDirectoryIfMissing (curWorkingPath);
	if ( didCreate )
	{
		self.mCurWorkingPath = curWorkingPath;
	}
	else 
	{
		NSLog ( @"2010-11-11steve ERROR - unable to create current working directory???" );
	}
	
	return didCreate;
	
}


// 2010-11-10steve
-(BOOL) updateMasterOriginalDirectory
{
	// should validate the alias first
		// -- missing code here
	
	/*
	// get the basePath
	NSString* basePath = [[NSUserDefaults standardUserDefaults] objectForKey:sPrefOriginalVideosFolder];
	
	// make sure basePath exists
	NSString* dailyPath = basePath;	// 2010-11-11steve
	if ( gDoDailySubFolder ) // 2010-11-11steve
	{
		dailyPath = [basePath stringByAppendingPathComponent:getTodaysDateAsString()];
	}
	*/
	NSString* dailyPath = getDailyFolderPathForMasterOriginalVideos(); // 2010-11-11steve
	
	NSLog ( @"2010-11-11steve updateMasterOriginalDirectory dailyPath is [%@]", dailyPath );
	BOOL didCreate = createDirectoryIfMissing (dailyPath);
	if ( didCreate )
	{

	}
	else 
	{
		NSLog ( @"2010-11-11steve updateMasterOriginalDirectory ERROR - unable to create current working directory???" );
	}
	
	return didCreate;
	
}


-(void) showSystemReady
{
	
	// if ( [mCaptureSession isRunning] )
	   
	[ibTextField1 ibUpdateNSTextFieldWithString:[self getSystemReadyString]     // 2014-10-07steve @"System Ready..."
										 doWait:NO 
									  withColor:[NSColor greenColor]];
    
    if (self.mVideoStatusWindowController)
        [self.mVideoStatusWindowController setTextField1:[self getSystemReadyString]     // 2014-10-07steve @"System Ready..."
                                                  doWait:NO
                                               withColor:[NSColor greenColor]]; // 2014-07-13steve

	
}


- (void)awakeFromNib
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );

    // 2014-08-09steve - add antoher window for alert without app coming to the front
//    self.mVideoAlertWindowController2 = [[[StatusWindowController alloc] initWithWindowNibName:@"StatusWindowController3"]autorelease];// 2014-08-9steve
//    self.mVideoAlertWindowController2.fOwner =  nil; // 2014-08-09steve
//    //[self.mVideoAlertWindowController2.window orderOut:nil];// 2014-08-09steve
//    NSScreen* mainScreen = [NSScreen mainScreen];
//    if (mainScreen) {
//        NSRect mainrect = [mainScreen frame];
//        NSRect winrect = self.mVideoAlertWindowController2.window.frame;
//        NSRect centerRect = NSMakeRect(0, 0, 0, 0);
//        centerRect.origin.x = (mainrect.size.width - winrect.size.width)/2;
//        centerRect.origin.y = (mainrect.size.height - winrect.size.height)/2;
//        centerRect.size = winrect.size;
//        [self.mVideoAlertWindowController2.window setFrame:centerRect display:YES];
//    }
//     [self.mVideoAlertWindowController2 showWindow:nil];
    
    if (ibAlertPanel) {
        NSScreen* mainScreen = [NSScreen mainScreen];
        if (mainScreen) {
            NSRect mainrect = [mainScreen frame];
            NSRect winrect = ibAlertPanel.frame;
            NSRect centerRect = NSMakeRect(0, 0, 0, 0);
            centerRect.origin.x = (mainrect.size.width - winrect.size.width)/2;
            centerRect.origin.y = (mainrect.size.height - winrect.size.height)/2;
            centerRect.size = winrect.size;
            [ibAlertPanel setFrame:centerRect display:YES];
        }
    }
    
    
    self.mVideoStatusWindowController = [[[StatusWindowController alloc] initWithWindowNibName:@"StatusWindowController"] autorelease]; // 2014-07-13steve
    self.mVideoStatusWindowController.fOwner =  self;
    [self.mVideoStatusWindowController showWindow:nil]; // 2014-07-13steve
    
    
    if (self.mVideoStatusWindowController)
        [self.mVideoStatusWindowController setTextField2:[NSString stringWithFormat:@"CustomerID=[..UNKNOWN..]"]
                                                  doWait:NO
                                               withColor:[NSColor magentaColor]]; // 2014-07-13steve

    
    
    BOOL doWarnWithTimer = NO; // test alert
    if (doWarnWithTimer) {
        NSTimer* aTimer = [NSTimer timerWithTimeInterval:5.0
                                                  target:self
                                                selector:@selector(warningTimerTrippered:)
                                                userInfo:nil
                                                 repeats:NO];
        [[NSRunLoop mainRunLoop] addTimer:aTimer
                                  forMode:NSDefaultRunLoopMode];
    }

    
	[self setWindowFrameAutosaveName:@"CameraStationWindow"];
	_mIsRecording = NO;
	[self showWindow:nil];

	
	[ibTextField1 ibUpdateNSTextFieldWithString:@"Initializing..."
										 doWait:YES 
									  withColor:[NSColor redColor]];
    
    if (self.mVideoStatusWindowController)
        [self.mVideoStatusWindowController setTextField1:@"Initializing..." doWait:YES withColor:[NSColor redColor]]; // 2014-07-13steve

	
	
	// 2010-11-11steve create the local working directory
	[self updateWorkingDirectory];
	[self updateMasterOriginalDirectory];

	
	BOOL allValid = [self validateAllFolders];
	[_myNSTabView selectTabViewItemAtIndex:0];
	[self showTabForError:allValid];
	
	
	_mDateFormatter = [[NSDateFormatter alloc] init];
	[_mDateFormatter setFormatterBehavior:NSDateFormatterBehavior10_4];
	[_mDateFormatter setDateFormat:@"yyyyMMdd_HHmmss"];
	

	
// Create the capture session
	mCaptureSession = [[QTCaptureSession alloc] init];
    
// Connect inputs and outputs to the session	
	BOOL success = NO;
	NSError *error = nil;
	
    [[mCameraArrayController content] removeAllObjects]; // 2014-06-10steve
    [[mMicrophoneArrayController content] removeAllObjects]; // 2014-06-26steve

    
	
// 2010-10-13steve
	//NSArray* myQTCaptureDeviceArray = [QTCaptureDevice inputDevices]; // 2010-11-01
	NSMutableArray* myQTCaptureDeviceArray = [QTCaptureDevice inputDevicesWithMediaType:QTMediaTypeMuxed]; // 2010-11-01
	NSLog (@ "2010-10-23steve List of QTCaptureDevice: %@", myQTCaptureDeviceArray );
	long count = [myQTCaptureDeviceArray count];
	long i =0;
	long j = 0;
	NSArray* descriptionArray = nil;
	for ( i=0;i<count; i++ )
	{
		NSLog ( @"2010-10-23steve _______________________________________" );
		NSLog (@ "2010-10-23steve myQTCaptureDeviceArray[%i] = %@", i, [[myQTCaptureDeviceArray objectAtIndex:i] deviceAttributes] );
		descriptionArray = [[myQTCaptureDeviceArray objectAtIndex:i] formatDescriptions];
		
//		NSLog (@ "2010-10-23steve descriptionArray = %@", descriptionArray );
//		NSLog (@ "2010-10-23steve descriptionArray = %@", descriptionArray );

        [mCameraArrayController addObject:[NSMutableDictionary dictionaryWithObjectsAndKeys:[[myQTCaptureDeviceArray objectAtIndex:i] localizedDisplayName], @"descriptionstring", @"QTMediaTypeMuxed",@"mediatype", nil ]]; // 2014-06-10steve

        
		if ( descriptionArray && [descriptionArray count] )
		{
			for ( j=0; j<[descriptionArray count]; j++ )
			{
				NSLog (@ "	2010-10-23steve formatDescription[%i] = %@", j, [[descriptionArray objectAtIndex:j] localizedFormatSummary]  );

			}
		}
	}
    
    // 2014-06-10steve ========================================================================
    // 2014-06-10steve - WE WANT TO GET ALL the DEVICES in ADVANCE NOW
    NSArray* videoArray = [QTCaptureDevice  inputDevicesWithMediaType:QTMediaTypeVideo];
    NSLog(@"videoArray of QTMediaTypeVideo = %@", videoArray);
    if (videoArray) {
        [myQTCaptureDeviceArray addObjectsFromArray:videoArray];
    }
    count = [myQTCaptureDeviceArray count];
    
    for ( short i=0; i<count; i++ ) {
        QTCaptureDevice* videoDevice = [myQTCaptureDeviceArray objectAtIndex:i];
        NSLog(@"2014-06-10steve myQTCaptureDeviceArray[%i] class = %@", i, [videoDevice class]);
        NSLog(@"2014-06-10steve myQTCaptureDeviceArray[%i] localizedDisplayName = %@", i, [videoDevice localizedDisplayName]);
        [mCameraArrayController addObject:[NSDictionary dictionaryWithObjectsAndKeys:[[myQTCaptureDeviceArray objectAtIndex:i] localizedDisplayName], @"descriptionstring", @"QTMediaTypeVideo",@"mediatype", nil ]]; // 2014-06-10steve
        
    }

    
    NSArray* microphoneArray = [QTCaptureDevice  inputDevicesWithMediaType:QTMediaTypeSound];
    for (short i=0; i<[microphoneArray count]; i++) {
        QTCaptureDevice* adevice = [microphoneArray objectAtIndex:i];
        NSLog(@"2014-06-10steve myQTCaptureDeviceArray[%i] class = %@", i, [adevice class]);
        NSLog(@"2014-06-10steve myQTCaptureDeviceArray[%i] localizedDisplayName = %@", i, [adevice localizedDisplayName]);
        [mMicrophoneArrayController addObject:[NSDictionary dictionaryWithObjectsAndKeys:[adevice localizedDisplayName], @"descriptionstring", @"QTMediaTypeSound",@"mediatype", nil ]]; // 2014-06-10steve
        
    }
    BOOL doSortArray = NO; // 2014-06-26steve
    if ( doSortArray) {
        NSArray* contents = [NSMutableArray arrayWithArray:[mMicrophoneArrayController content]];
        NSLog(@"contents of mMicrophoneArrayController BEFORE = %@", contents);
        NSSortDescriptor* sort = [[NSSortDescriptor alloc]
                                  initWithKey:@"descriptionstring" ascending:NO];
        contents = [contents sortedArrayUsingDescriptors:[NSArray arrayWithObjects:sort,nil]];
        [sort autorelease];
        NSLog(@"contents of mMicrophoneArrayController AFTER = %@", contents);
        //[mMicrophoneArrayController setContent:contents];
    }

    
    
    // 2014-06-10steve ========================================================================
    
    
    
	NSLog ( @"2010-10-23steve =================================" );
	NSLog ( @"2010-10-23steve =================================" );
    
    
	if ( count==0 )
	{
		NSLog ( @"2010-11-06steve - MISSING Video Feed... Please Check Your Connection!!!!!" );
		bool doAlert = false;
		if (doAlert)
		{
			NSAlert* alert = [[NSAlert alloc] init];
			[alert setMessageText:@"MISSING Live Video Feed!"];
			[alert setInformativeText:@"Please Check Your Cable Connection..."];
			[alert addButtonWithTitle:@"OK"];
			
			[[NSApplication sharedApplication] activateIgnoringOtherApps: YES];
			[alert runModal];
			
			[alert release];
			alert = nil;
		}
		
	}
	
	
	QTCaptureDevice *videoDevice = nil;
	bool doFindDVFirst = false; // 2014-06-10steve true;
	if ( doFindDVFirst )
	{
		// Find a video device  - find external first
		NSLog (@ "2010-11-01steve First Try defaultInputDeviceWithMediaType:QTMediaTypeMuxed" );
		error = nil;
		videoDevice = [QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeMuxed];
		success = [videoDevice open:&error];
		if ( error)
		{
			NSLog ( @"2010-11-08steve QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeMuxed Found Error: %@", error );
		}
		
		// If a video input device can't be found or opened, try to find and open a muxed input device
		if (!success) {
			NSLog (@ "2010-11-01steve Second - TRY  defaultInputDeviceWithMediaType:QTMediaTypeVideo" );
            
            NSArray* myQTCaptureDeviceArray = [QTCaptureDevice inputDevicesWithMediaType:QTMediaTypeVideo]; // 2010-11-01
            NSLog(@"2013-08-27 QTCaptureDevice inputDevicesWithMediaType:QTMediaTypeVideo = %@", myQTCaptureDeviceArray);
            count = [myQTCaptureDeviceArray count];
            for ( short i=0; i<count; i++ ) {
                videoDevice = [myQTCaptureDeviceArray objectAtIndex:i];
                NSLog(@"2014-06-10steve myQTCaptureDeviceArray[%i] class = %@", i, [videoDevice class]);
                NSLog(@"2014-06-10steve myQTCaptureDeviceArray[%i] localizedDisplayName = %@", i, [videoDevice localizedDisplayName]);
                
                [mCameraArrayController addObject:[NSDictionary dictionaryWithObjectsAndKeys:[[myQTCaptureDeviceArray objectAtIndex:i] localizedDisplayName], @"descriptionstring", @"QTMediaTypeVideo",@"mediatype", nil ]]; // 2014-06-10steve
                
            }
            videoDevice = nil;
            
            
            
            short totalCameras = [myQTCaptureDeviceArray count];
            if ( totalCameras>1) {
                videoDevice = [myQTCaptureDeviceArray objectAtIndex:totalCameras-1];
                
                // 2014-06-10steve - hmm - if there are more than one - what to do??
                videoDevice = [myQTCaptureDeviceArray objectAtIndex:0];

                
            } else {
                videoDevice = [QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeVideo];
            }
            
			error = nil;
			success = [videoDevice open:&error];
			if ( error)
			{
				NSLog ( @"2010-11-08steve QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeVideo Found Error: %@", error );
			} else {
                NSLog(@"2013-08-27 did find QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeVideo = [%@]", videoDevice);
            }
			
		}
		
		
		
	}
	else
	{
        BOOL doNewMethod = YES; // 2014-06-10steve use preferences this time
        
        
        if (doNewMethod) {
            
            NSString* userSelectedCamera = [[NSUserDefaults standardUserDefaults] stringForKey:@"prefDesiredCamera"];
            
            videoDevice= nil; // 2014-06-26steve
            if ( userSelectedCamera) {
                short count = [myQTCaptureDeviceArray count];
                for ( short i=0; i<count; i++) {
                    
                    videoDevice = [myQTCaptureDeviceArray objectAtIndex:i];
                    NSString* localname = [videoDevice localizedDisplayName];
                    if (localname && [localname caseInsensitiveCompare:userSelectedCamera]==NSOrderedSame) {
                        break;
                    } else {
                        videoDevice = nil;
                    }
                    
                }
                
//                if (videoDevice) {
//                    error = nil;
//                    success = [videoDevice open:&error];
//                    if ( error)
//                    {
//                        NSLog ( @"2010-11-08steve QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeVideo Found Error: %@", error );
//                    } else {
//                        NSLog(@"2013-08-27 did find QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeVideo = [%@]", videoDevice);
//                    }
//                }
                
            }
            //else
            if (videoDevice==nil) { // 2014-06-26steve
                NSLog(@"User NEVER selected a Camera Before...");
                videoDevice = [myQTCaptureDeviceArray objectAtIndex:i]; // 2014-06-26steve
                NSString* description = [videoDevice localizedDisplayName];
                [[NSUserDefaults standardUserDefaults] setObject:description forKey:@"prefDesiredCamera"];

            }
            
            if (videoDevice) {
                error = nil;
                success = [videoDevice open:&error];
                if ( error)
                {
                    NSLog ( @"2010-11-08steve QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeVideo Found Error: %@", error );
                } else {
                    NSLog(@"2013-08-27 did find QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeVideo = [%@]", videoDevice);
                }
            }

            
            
            
        } else {
            // Find a video device
            videoDevice = [QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeVideo];
            success = [videoDevice open:&error];
            
            
            // If a video input device can't be found or opened, try to find and open a muxed input device
            if (!success) {
                NSLog (@ "2010-11-01steve need to try defaultInputDeviceWithMediaType:QTMediaTypeMuxed" );
                videoDevice = [QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeMuxed];
                success = [videoDevice open:&error];
            }
        }
        
	}
	
	
    
    if (!success) {
        videoDevice = nil;
        // Handle error
        
    }
    
    if (videoDevice)
	{
//Add the video device to the session as a device input
		
		mCaptureVideoDeviceInput = [[QTCaptureDeviceInput alloc] initWithDevice:videoDevice];
		success = [mCaptureSession addInput:mCaptureVideoDeviceInput error:&error];
		if (!success) {
			// Handle error
		}
        
// If the video device doesn't also supply audio, add an audio device input to the session
        
        if (![videoDevice hasMediaType:QTMediaTypeSound] && ![videoDevice hasMediaType:QTMediaTypeMuxed]) {
            
            BOOL doNewMethod = YES; // 2014-06-26steve
            
            QTCaptureDevice *audioDevice = nil; // 2014-06-26steve

            if (doNewMethod) { // 2014-06-26steve
                
                NSString* userSelectedInput = [[NSUserDefaults standardUserDefaults] stringForKey:@"prefDesiredMicrophone"];
                
                if ( userSelectedInput) {
                    short count = [microphoneArray count];
                    for ( short i=0; i<count; i++) {
                        
                        audioDevice = [microphoneArray objectAtIndex:i];
                        NSString* localname = [audioDevice localizedDisplayName];
                        if (localname && [localname caseInsensitiveCompare:userSelectedInput]==NSOrderedSame) {
                            break;
                        } else {
                            audioDevice = nil;
                        }
                        
                    }
                    
                } else {
                    NSLog(@"User NEVER selected a Microphone Before...");
                    NSLog(@"Forcing to First Item in Microphone List...");
                    if ([microphoneArray count]) {
                        audioDevice = [microphoneArray objectAtIndex:0];
                        NSString* description = [audioDevice localizedDisplayName];
                        [[NSUserDefaults standardUserDefaults] setObject:description forKey:@"prefDesiredMicrophone"];
                    }
                }

                
            } else {
                // QTCaptureDevice *audioDevice = [QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeSound];
                audioDevice = [QTCaptureDevice defaultInputDeviceWithMediaType:QTMediaTypeSound];
            }
            
            success = [audioDevice open:&error];
            
            NSLog(@"DefaultSoundINput = %@", audioDevice); // 2014-06-26steve
            
            if (!success) {
                audioDevice = nil;
                NSLog(@"2014-06-26steve - UNABLE to setup audioDevice...");
                // Handle error
            }
            
            if (audioDevice) {
                mCaptureAudioDeviceInput = [[QTCaptureDeviceInput alloc] initWithDevice:audioDevice];
                
                success = [mCaptureSession addInput:mCaptureAudioDeviceInput error:&error];
                if (!success) {
                    // Handle error
                }
            }

            
            
            
        } else {
            
            NSLog(@"2014-06-26steve - did fall into here........");
        
        }
        
// Create the movie file output and add it to the session
        
        mCaptureMovieFileOutput = [[QTCaptureMovieFileOutput alloc] init];
        success = [mCaptureSession addOutput:mCaptureMovieFileOutput error:&error];
        if (!success) {
            // Handle error
        }
        
        [mCaptureMovieFileOutput setDelegate:self];


// Set the compression for the audio/video that is recorded to the hard disk.

	NSEnumerator *connectionEnumerator = [[mCaptureMovieFileOutput connections] objectEnumerator];
	QTCaptureConnection *connection;
	
	// iterate over each output connection for the capture session and specify the desired compression
	while ((connection = [connectionEnumerator nextObject])) {
		NSString *mediaType = [connection mediaType];
        
        NSArray* list = [QTCompressionOptions compressionOptionsIdentifiersForMediaType:mediaType];
        NSLog(@"2013-10-10steve compressionOptionsIdentifiersForMediaType[%@] = %@", mediaType,list);

		QTCompressionOptions *compressionOptions = nil;
		// specify the video compression options
		// (note: a list of other valid compression types can be found in the QTCompressionOptions.h interface file)
		if ([mediaType isEqualToString:QTMediaTypeVideo]) {
			// use H.264
			compressionOptions = [QTCompressionOptions compressionOptionsWithIdentifier:@"QTCompressionOptionsSD480SizeH264Video"];
			//compressionOptions = [QTCompressionOptions compressionOptionsWithIdentifier:@"QTCompressionOptionsLosslessAnimationVideo"];
			//compressionOptions = [QTCompressionOptions compressionOptionsWithIdentifier:@"QTCompressionOptionsSD480SizeMPEG4Video"];
			// compressionOptions = [QTCompressionOptions compressionOptionsWithIdentifier:@"QTCompressionOptionsJPEGVideo"];
            compressionOptions = [QTCompressionOptions compressionOptionsWithIdentifier:@"QTCompressionOptions240SizeH264Video"];
            compressionOptions = [QTCompressionOptions compressionOptionsWithIdentifier:@"QTCompressionOptions120SizeH264Video"];
			//compressionOptions = [QTCompressionOptions compressionOptionsWithIdentifier:@"QTCompressionOptionsLosslessAnimationVideo"]; // HIGH - takes a long time to save out
            compressionOptions = [QTCompressionOptions compressionOptionsWithIdentifier:@"QTCompressionOptionsHD720SizeH264Video"]; // medium 16:9
			compressionOptions = [QTCompressionOptions compressionOptionsWithIdentifier:@"QTCompressionOptionsSD480SizeH264Video"];
            
            NSInteger prefIndex = [[NSUserDefaults standardUserDefaults] integerForKey:@"prefSelectedVideoCompressionIndex"];
            id prefArray = [[NSUserDefaults standardUserDefaults] objectForKey:@"prefVideoCompressionOptions"];
            NSString* prefOption = [[prefArray objectAtIndex:prefIndex] objectForKey:@"compressionoption"];
            NSLog(@"last saved compression option = [%@]", prefOption);
            if ( prefOption) {
                compressionOptions = [QTCompressionOptions compressionOptionsWithIdentifier:prefOption];
            }
            

		// specify the audio compression options
		} else if ([mediaType isEqualToString:QTMediaTypeSound]) {
			// use AAC Audio
			compressionOptions = [QTCompressionOptions compressionOptionsWithIdentifier:@"QTCompressionOptionsVoiceQualityAACAudio"];
		}
		
		// set the compression options for the movie file output
		[mCaptureMovieFileOutput setCompressionOptions:compressionOptions forConnection:connection];
	} 

// Associate the capture view in the UI with the session
        
        [mCaptureView setCaptureSession:mCaptureSession];
        
        [mCaptureSession startRunning];
        
	}
	

	[self performSelector:@selector(showSystemReady) withObject:nil afterDelay:2.0];
	
    
	
//	// 2010-10-25steve
//	NSNotificationCenter* nc = [NSNotificationCenter defaultCenter];
//	[nc addObserver:self
//		   selector:@selector(handleNSThreadWillExitNotification:)
//			   name:NSThreadWillExitNotification
//			 object:nil];
	
	
	
	
	mDoMergeVideo = false; // 2010-10-29
	
	

	// 2013-10-10steve
    bool doTestMore = true; // 2013-10-10steve
	if ( doTestMore && mOptionsArrayController )
	{
        [[mOptionsArrayController content] removeAllObjects];
        
        id videoCompressionList = [[NSUserDefaults standardUserDefaults] objectForKey:@"prefVideoCompressionOptions"];
        if (videoCompressionList) {
            
            if ( [videoCompressionList isKindOfClass:[NSArray class]] ) {
                [mOptionsArrayController setContent:videoCompressionList];
                
                // 2013-10-14steve
                if (ibPopupButton) {

                    NSArray* systemCompressionlist = [QTCompressionOptions compressionOptionsIdentifiersForMediaType:QTMediaTypeVideo];
                    NSLog(@"systemCompressionlist = %@", systemCompressionlist); // 2014-06-26steve
                    
                    
                    NSInteger numberOfItems = [videoCompressionList count];
                    if ( numberOfItems) {
                        for (NSInteger idx=0; idx<numberOfItems; idx++) {
                            id obj = [videoCompressionList objectAtIndex:idx];
                            NSLog(@"obj = %@", obj);
                            NSString* compressionoption = [obj objectForKey:@"compressionoption"];
                            BOOL doesExist = NO;
                            if (compressionoption) {
                                // make sure it is avail in current list for this os system
                                for (id listitem in systemCompressionlist) {
                                    NSLog(@"listitem = [%@]", listitem);
                                    if ( [compressionoption caseInsensitiveCompare:listitem]==NSOrderedSame ) {
                                        doesExist = YES;
                                        break;
                                    }
                                }
                            }
                                
                            {
                                NSMenuItem* menuitem = [ibPopupButton itemAtIndex:idx];
                                if (menuitem) {
                                    [menuitem setEnabled:doesExist];
                                }
                            }
                            
                        }
                    }
                }

            }
            
            
        } else {
            NSString* mediaType = QTMediaTypeVideo; // @"QTMediaTypeMuxed"
            NSArray* list = [QTCompressionOptions compressionOptionsIdentifiersForMediaType:mediaType];
            
            NSInteger index = 0;
            for ( id obj in list) {
                
                NSLog(@"adding obj[%li] = [%@]", index,obj);
                
                [mOptionsArrayController addObject:@{@"compressionoption":obj, @"descriptionstring":[NSString stringWithFormat:@"%li. %@",index,obj ]}];
                
                ++index;
            }

        }
        
        NSLog(@"mCameraArrayController = %@", mCameraArrayController); // 2014-06-10steve
        NSLog(@"mCameraArrayController arrangedObjects = %@", [mCameraArrayController arrangedObjects]); // 2014-06-10steve

        NSLog(@"mMicrophoneArrayController = %@", mMicrophoneArrayController); // 2014-06-26steve
        NSLog(@"mMicrophoneArrayController arrangedObjects = %@", [mMicrophoneArrayController arrangedObjects]); // 2014-06-26steve

		
        NSLog(@"mOptionsArrayController = %@", mOptionsArrayController);
        NSLog(@"mOptionsArrayController arrangedObjects = %@", [mOptionsArrayController arrangedObjects]);
        
        //self.fSelectedOptionValue = @"16:9 720p";

	}
	
	
//    // 2014-07-13steve
//    if (self.mVideoStatusWindowController) {
//        [self.mVideoStatusWindowController setCaptureView:mCaptureSession]; // 2014-07-13steve
//    }


    [[NSDistributedNotificationCenter defaultCenter] addObserver:self
                                                        selector:@selector(handleTepsVideoNotification:)
                                                            name:sTepsVideoAppNotification
                                                          object:nil
                                              suspensionBehavior:NSNotificationSuspensionBehaviorDeliverImmediately ];
    
    [[NSDistributedNotificationCenter defaultCenter] addObserver:self
                                                        selector:@selector(handleTepsXNotification:)
                                                            name:sTepsXNotification
                                                          object:nil
                                              suspensionBehavior:NSNotificationSuspensionBehaviorDeliverImmediately ];
    
    [[NSDistributedNotificationCenter defaultCenter] addObserver:self
                                                        selector:@selector(handleTepsXNotification:)
                                                            name:sTepsPreviewerNotification
                                                          object:nil
                                              suspensionBehavior:NSNotificationSuspensionBehaviorDeliverImmediately ];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(handleScannedBarcode:) // 2014-11-04steve
                                                 name:sNewCustomerIdNotification
                                               object:nil];

    
    
    [self sendRecordingStatusNotification]; // 2014-08-09steve
 
    
    [_myNSTabView setDelegate:self]; // 2014-09-04steve
    
    
    
    // 2014-10-15steve
    [self removeOldMoviesFromWorkingDirectory:nil];
//    [NSThread detachNewThreadSelector:@selector(removeOldMoviesFromWorkingDirectory:) toTarget:self withObject:nil]; // 2014-10-15steve
    

    
    
}


-(void) windowDidLoad
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	
}

// Handle window closing notifications for your device input
- (void)windowWillClose:(NSNotification *)notification
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );

	[mCaptureSession stopRunning];
    
    if ([[mCaptureVideoDeviceInput device] isOpen])
        [[mCaptureVideoDeviceInput device] close];
    
    if ([[mCaptureAudioDeviceInput device] isOpen])
        [[mCaptureAudioDeviceInput device] close];
    
}


- (void)keyDown:(NSEvent *)event
{
	NSLog (@ "Inside %@ => %@... keyDownkeyDownkeyDownkeyDownkeyDownkeyDown", self, NSStringFromSelector(_cmd) );

	// real time method
//	if ([[NSApp currentEvent] modifierFlags] & NSCommandKeyMask)
//    {
//        NSLog(@"CommandKey Down!");
//    }
	
	NSResponder* fr = [[self window] firstResponder];
	NSLog (@ "Inside %@ => %@... firstResponder=[%@]", self, NSStringFromSelector(_cmd),fr );

	
	// we need to buffer the keyboard strokes here with some sort of timing
	NSString *characters = [event characters];
	NSUInteger modifiers = [event modifierFlags];
	
	bool hasModifiers = (modifiers & (NSControlKeyMask|NSAlternateKeyMask|NSCommandKeyMask|NSFunctionKeyMask ) );
	
	NSLog (@ "%@->%@...[%@]", [self class], NSStringFromSelector(_cmd), characters );
	
	if ( hasModifiers )
	{
		NSLog ( @"[%@] has MODIFIERS[0x%x] attached...", characters, modifiers );
	}
	
	if ( [characters caseInsensitiveCompare:@" "]==NSOrderedSame )
	{
        
        NSTabViewItem * selectedTabViewItem = [_myNSTabView selectedTabViewItem]; // 2013-10-23steve only allow recording if QTMovieView is visible
        id curIdentifier = [selectedTabViewItem identifier];
        if ( curIdentifier && [curIdentifier isKindOfClass:[NSString class]]) {
            if ([curIdentifier caseInsensitiveCompare:@"1"]==NSOrderedSame )
                [self toggleRecording:nil];
        }
	}
	else
	{
		// if we enable the the text view - then we dont really need to do anything
		[super keyDown:event];
	}
	
}




// Handle deallocation of memory for your capture objects
- (void)dealloc
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );

    
    self.fCurrentTepsCustomerID = nil; // 2014-07-25steve
    self.fCurrentTepsDailyFolderPath = nil; // 2014-07-25steve

    [self.fWarningTimer1 invalidate]; // 2014-07-22steve
    self.fWarningTimer1 = nil; // 2014-07-22steve
    
    
    self.mVideoStatusWindowController = nil; // 2014-07-13steve
    
	[_mDateFormatter release];
	
	[mCaptureSession release];
	NSLog (@ "Inside %@ => %@...1", self, NSStringFromSelector(_cmd) );

	[mCaptureVideoDeviceInput release];
	NSLog (@ "Inside %@ => %@...2", self, NSStringFromSelector(_cmd) );

	[mCaptureAudioDeviceInput release];
	NSLog (@ "Inside %@ => %@...3", self, NSStringFromSelector(_cmd) );

	[mCaptureMovieFileOutput release];
	NSLog (@ "Inside %@ => %@...4", self, NSStringFromSelector(_cmd) );
    
    [[NSDistributedNotificationCenter defaultCenter] removeObserver:self]; // 2014-07-15steve


    
    
    
	[super dealloc];
}

#pragma mark-

// 2009-08-15steve
//- (void) ibUpdateNSTextField:(id)theField withString:(NSString*)theText doWait:(BOOL) doWait isError:(BOOL)isError
//{
//	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
//	if ( theField )
//	{
//		{
//			NSColor* useColor = [NSColor blackColor];
//			if (isError)
//			{
//				useColor = [NSColor redColor];
//			}
//			
//			[theField performSelectorOnMainThread:@selector(setTextColor:)
//									   withObject:useColor 
//									waitUntilDone:doWait];
//			
//			[theField performSelectorOnMainThread:@selector(setStringValue:)
//									   withObject:theText 
//									waitUntilDone:doWait];
//			
//			[theField performSelectorOnMainThread:@selector(display)
//									   withObject:nil 
//									waitUntilDone:doWait];
//		}
//		
//	}
//	else
//	{
//		NSLog (@ "Inside %@ => %@... NIL NSTextField....", self, NSStringFromSelector(_cmd) );
//	}
//	if ( pool) [pool release];
//}


-(void) stopRecordingTicks
{
	NSLog (@ "Inside %@ => %@ ...", self, NSStringFromSelector(_cmd) );

	[self.mRecordingTimer invalidate];
	
	self.mRecordingTimer = nil;
	
	
}

- (void)timerFireMethod:(NSTimer*)theTimer
{
	// NSLog (@ "Inside %@ => %@ ...", self, NSStringFromSelector(_cmd) );
	
	bool doEcho = false;
	
	id userInfo = [theTimer userInfo];
	if ( userInfo )
	{
		NSDate *startDate = [userInfo objectForKey:@"startDateKey"];
		NSTimeInterval lapsedTime = [[NSDate date] timeIntervalSinceDate:startDate];
		short hour = lapsedTime / 3600.0;
		short min = lapsedTime / 60.0;
		//short temp1 = (float)lapsedTime / (float)100.0;
		//float sec = lapsedTime - (float) temp1;
		float sec = fmod(lapsedTime, 60.0);

		
		//NSString* infoString = [NSString stringWithFormat:@"%02i:%02i:%02i", hour,min,(short)sec];
		//NSString* infoString = [NSString stringWithFormat:@"%02i:%02i:%04.1f", hour,min,(float)sec];
		NSString* infoString = [NSString stringWithFormat:@"%02i:%02i:%04.1f", hour,min,(float)sec];

		if ( doEcho )
		{
			NSLog ( @"++++++++++++++++++++++++++++++++++++++++++++" );
			NSLog ( @"2010-11-02steve lapsedTime is %2.3f", lapsedTime );
			NSLog ( @"2010-11-02steve the lapsed time is: %@", infoString );
			NSLog ( @"++++++++++++++++++++++++++++++++++++++++++++" );
		}
		
		[ibTextField1 ibUpdateNSTextFieldWithString:infoString 
											 doWait:NO 
											isError:YES]; // we want red color
		
        if (self.mVideoStatusWindowController)
            [self.mVideoStatusWindowController setTextField1:infoString doWait:NO isError:YES]; // 2014-07-13steve
        
        
	}
	
}

-(void) setupRecordingTicks
{
	NSLog (@ "Inside %@ => %@ ...", self, NSStringFromSelector(_cmd) );
	
	NSDictionary* userInfo = [NSDictionary dictionaryWithObjectsAndKeys:[NSDate date],@"startDateKey",nil];
	

	if (self.mRecordingTimer )
	{
		[self.mRecordingTimer invalidate];
		self.mRecordingTimer = nil;
		
	}
	float timeIntervalInSeconds = 0.1;
	self.mRecordingTimer = [NSTimer timerWithTimeInterval:timeIntervalInSeconds
											 target:self 
										   selector:@selector(timerFireMethod:) 
										   userInfo:userInfo
											repeats:YES];
	
	[[NSRunLoop mainRunLoop] addTimer:self.mRecordingTimer forMode:NSRunLoopCommonModes];
	
}


#pragma mark -


-(NSString*) getSystemReadyString // 2014-10-07steve
{
    NSString* string = [[NSUserDefaults standardUserDefaults] stringForKey:@"prefSystemReadyMessage"];
    string = [string stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
    if (string==nil || [string length]==0) {
        string = @"System Ready...";
    }
    return [NSString stringWithFormat:string];
}

-(void) bringToFront:(id)sender // 2014-07-13steve
{
    [[NSRunningApplication currentApplication] activateWithOptions:(NSApplicationActivateAllWindows | NSApplicationActivateIgnoringOtherApps)];

    // [self.window makeKeyAndOrderFront:nil];

}


-(IBAction) sendToBack:(id)sender // 2014-09-25steve
{

    [[NSNotificationCenter defaultCenter] postNotificationName:@"TepsNotificationSendToBack"
                                                        object:nil
                                                      userInfo:nil];

}



#pragma mark -

-(IBAction)handleVideoCompressionPopup:(id)sender
{
    NSInteger selectedIndex = [[NSUserDefaults standardUserDefaults] integerForKey:@"prefSelectedVideoCompressionIndex"];
	NSLog (@ "Inside %@ => %@... prefSelectedVideoCompressionIndex = %li", self, NSStringFromSelector(_cmd), selectedIndex );

    id selectedOption = nil;
    id videoCompressionList = [[NSUserDefaults standardUserDefaults] objectForKey:@"prefVideoCompressionOptions"];
    if (videoCompressionList) {
        NSInteger count = [videoCompressionList count];
        if ( selectedIndex < count) {
            selectedOption = [videoCompressionList objectAtIndex:selectedIndex];
        }
    }
    
    
    NSArray* connections = [mCaptureMovieFileOutput connections];
    NSLog (@ "Inside %@ => %@... mCaptureMovieFileOutput connections = [%@]", self, NSStringFromSelector(_cmd), connections );
    for (id obj in connections) {
        
        if ([obj isKindOfClass:[QTCaptureConnection class]]) {
            QTCompressionOptions *compressionOptions = nil;
            QTCaptureConnection * qtConn = (QTCaptureConnection*) obj;
            NSString* mediaType = [qtConn mediaType];
            if ( [mediaType caseInsensitiveCompare:QTMediaTypeVideo]==NSOrderedSame ) {
                
                NSString* prefOption = [selectedOption objectForKey:@"compressionoption"];
                NSLog(@"user selected compression option = [%@]", prefOption);
                if ( prefOption) {
                    compressionOptions = [QTCompressionOptions compressionOptionsWithIdentifier:prefOption];
                }

                if (compressionOptions) {
                    [mCaptureSession stopRunning];
                    [mCaptureMovieFileOutput setCompressionOptions:compressionOptions forConnection:qtConn];
                    [mCaptureSession startRunning];
                }

            }
        }
    }
    

}


-(NSString*) createMovieThumbnail:(NSString*) sourceMoviePath
{
    
    NSAutoreleasePool* p = [[NSAutoreleasePool alloc] init];
    
    NSString* newPathName = [sourceMoviePath stringByAppendingPathExtension:@"jpg"];
    NSURL* newOutputURL = [NSURL fileURLWithPath:sourceMoviePath];
    
    AVURLAsset *asset1 = [[AVURLAsset alloc] initWithURL:newOutputURL options:nil];
    AVAssetImageGenerator *generate1 = [[AVAssetImageGenerator alloc] initWithAsset:asset1];
    generate1.appliesPreferredTrackTransform = YES;
    NSError *err = NULL;
    CMTime time = CMTimeMake(1, 2); // at 0.5sec
    CGImageRef oneRef = [generate1 copyCGImageAtTime:time actualTime:NULL error:&err];
    
    
    NSImage* overlayImage = [NSImage imageNamed:@"PlayButton"];
    CGSize overlaySize = NSSizeToCGSize([overlayImage size]); // 2014-11-04steve

    
    CGSize finalSize = CGSizeMake(CGImageGetWidth(oneRef), CGImageGetHeight(oneRef));
    NSImage *singleFrameImage = [[NSImage alloc] initWithCGImage:oneRef size:NSSizeFromCGSize(finalSize)]; // 2014-11-04steve
    
    
    NSImage *comboImage = [[NSImage alloc] initWithSize:NSSizeFromCGSize(finalSize)]; // 2014-11-04steve
    [comboImage lockFocus];
    [singleFrameImage drawInRect:NSMakeRect(0, 0, finalSize.width, finalSize.height)
                        fromRect:NSZeroRect
                       operation:NSCompositeSourceOver
                        fraction:1.0];
    [overlayImage drawInRect:NSMakeRect((finalSize.width-overlaySize.width)/2.0,(finalSize.height-overlaySize.height)/2.0,overlaySize.width,overlaySize.height)
                    fromRect:NSZeroRect
                   operation:NSCompositeSourceOver
                    fraction:1.0];
    [comboImage unlockFocus];
    
    
    NSImage* sourceImage = comboImage;
    
    NSString* tiffPath = [[newPathName stringByDeletingPathExtension] stringByAppendingPathExtension:@"tiff"];
    NSData* tiffdata = [sourceImage TIFFRepresentation];
    // [tiffdata writeToFile:tiffPath atomically:YES];
    
    NSBitmapImageRep *imageRep = [NSBitmapImageRep imageRepWithData:tiffdata];
    NSNumber *compressionFactor = [NSNumber numberWithFloat:0.9];
    NSDictionary *imageProps = [NSDictionary dictionaryWithObject:compressionFactor
                                                           forKey:NSImageCompressionFactor];
    NSData* jpgData = [imageRep representationUsingType:NSJPEGFileType properties:imageProps];
    [jpgData writeToFile:newPathName atomically:YES];
    
    
    if (err) {
        // here is where we should clean up
        NSLog(@"copyCGImageAtTime returned error = %@",err);
        
        // we should delete the movie file too - just in case
        
        // give user UIAlertView
    }
    else {
        
        
    }
    
    
    // 2014-03-21steve - move down here
    [comboImage release];
    [singleFrameImage release];
    CGImageRelease(oneRef);
    [generate1 release]; // 2014-03-17steve
    
    
    
    newPathName = [newPathName retain];
    [p release];
    
    return [newPathName autorelease]; // 2014-07-25steve - path to thumbnail
    
}

-(NSString*) createMovieThumbnail:(NSString*) sourceMoviePath toDestItem:(NSString*) pathToDestinationItem // 2014-07-25steve
{
    
    NSAutoreleasePool* p = [[NSAutoreleasePool alloc] init];
    
    NSString* newPathName = [pathToDestinationItem stringByAppendingPathExtension:@"jpg"];

    NSURL* sourceOutputURL = [NSURL fileURLWithPath:sourceMoviePath];
    AVURLAsset *asset1 = [[AVURLAsset alloc] initWithURL:sourceOutputURL options:nil];
    AVAssetImageGenerator *generate1 = [[AVAssetImageGenerator alloc] initWithAsset:asset1];
    generate1.appliesPreferredTrackTransform = YES;
    NSError *err = NULL;
    CMTime time = CMTimeMake(1, 2); // at 0.5sec
    CGImageRef oneRef = [generate1 copyCGImageAtTime:time actualTime:NULL error:&err];
    
    
    NSImage* overlayImage = [NSImage imageNamed:@"PlayButton"];
    CGSize overlaySize = NSSizeToCGSize( [overlayImage size]); // 2014-11-04steve
    
    CGSize finalSize = CGSizeMake(CGImageGetWidth(oneRef), CGImageGetHeight(oneRef));
    NSImage *singleFrameImage = [[NSImage alloc] initWithCGImage:oneRef size:NSSizeFromCGSize(finalSize)]; // 2014-11-04steve

    
    NSImage *comboImage = [[NSImage alloc] initWithSize:NSSizeFromCGSize(finalSize)]; // 2014-11-04steve
    [comboImage lockFocus];
    [singleFrameImage drawInRect:NSMakeRect(0, 0, finalSize.width, finalSize.height)
                        fromRect:NSZeroRect
                       operation:NSCompositeSourceOver
                        fraction:1.0];
    [overlayImage drawInRect:NSMakeRect((finalSize.width-overlaySize.width)/2.0,(finalSize.height-overlaySize.height)/2.0,overlaySize.width,overlaySize.height)
                    fromRect:NSZeroRect
                   operation:NSCompositeSourceOver
                    fraction:1.0];
    [comboImage unlockFocus];

    
    NSImage* sourceImage = comboImage;

    // NSString* tiffPath = [[newPathName stringByDeletingPathExtension] stringByAppendingPathExtension:@"tiff"];
    NSData* tiffdata = [sourceImage TIFFRepresentation];
    // [tiffdata writeToFile:tiffPath atomically:YES];

    NSBitmapImageRep *imageRep = [NSBitmapImageRep imageRepWithData:tiffdata];
    NSNumber *compressionFactor = [NSNumber numberWithFloat:0.9];
    NSDictionary *imageProps = [NSDictionary dictionaryWithObject:compressionFactor
                                                           forKey:NSImageCompressionFactor];
    NSData* jpgData = [imageRep representationUsingType:NSJPEGFileType properties:imageProps];
    [jpgData writeToFile:newPathName atomically:YES];
    
    
    if (err) {
        // here is where we should clean up
        NSLog(@"copyCGImageAtTime returned error = %@",err);
        
        // we should delete the movie file too - just in case
        
        // give user UIAlertView
    }
    else {
        
        
    }
    
    
    // 2014-03-21steve - move down here
    [comboImage release];
    [singleFrameImage release];
    CGImageRelease(oneRef);
    [generate1 release]; // 2014-03-17steve
    
    
    
    newPathName = [newPathName retain];
    [p release];
    
    return [newPathName autorelease]; // 2014-07-25steve - path to thumbnail
    
}


-(NSString*) nameOfCurrentTepsCustomerMovie // 2014-10-12steve
{
    NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );

    NSString* pathToCurrentTepsCustomerMovie = nil;
    
    BOOL interfaceWithTeps = [[NSUserDefaults standardUserDefaults] boolForKey:@"prefInterfaceWithTeps"];

    if ( interfaceWithTeps && self.fCurrentTepsCustomerID) {
        // for TODDL = we need to b - basename_whatever.mp4"
        NSString* tepsNameOfItem = [NSString stringWithFormat:@"%@_001",self.fCurrentTepsCustomerID]; // 2014-07-28steve
        
        
        NSString* destPathWithoutExt = [self.fCurrentTepsDailyFolderPath stringByAppendingPathComponent:tepsNameOfItem?tepsNameOfItem:@""];
        
        
        NSLog(@"2014-10-12steve destPathWithoutExt = %@",destPathWithoutExt);
        
        pathToCurrentTepsCustomerMovie = [NSString stringWithFormat:@"%@",destPathWithoutExt];

    }
    
    
    return pathToCurrentTepsCustomerMovie;
}

-(BOOL) checkIfTepsCustomerMovieFileExistsForExt:(NSString*)ext // 2014-10-12steve
{
    NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
    BOOL exists = NO;
    
    NSString* path = [self nameOfCurrentTepsCustomerMovie];
    if ( path) {
        exists = [[NSFileManager defaultManager] fileExistsAtPath:[path stringByAppendingPathExtension:ext] ];
    }
    
    return exists;
}


-(BOOL) saveMovieToServer
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );

    // 2014-07-25steve
    NSString * tepsNameOfItem = [[self.mRecordedMoviePath lastPathComponent] stringByDeletingPathExtension];
    BOOL interfaceWithTeps = [[NSUserDefaults standardUserDefaults] boolForKey:@"prefInterfaceWithTeps"];
    if (interfaceWithTeps) {
        NSLog(@"Interfacing with Teps - need to get valid teps customer id = [%@]", self.fCurrentTepsCustomerID);
        NSLog(@"Interfacing with Teps - need to get valid teps customer id = [%@]", self.fCurrentTepsCustomerID);

        if ( self.fCurrentTepsCustomerID) {
            tepsNameOfItem = [NSString stringWithFormat:@"%@",self.fCurrentTepsCustomerID];
        } else {
            NSLog(@"Should NOT be here - no valid current ID from Teps....???");
        }
        if ( self.fCurrentTepsDailyFolderPath==nil) {
            NSLog(@"Should NOT be here - no valid Teps Daily Folder Path....???");
        }

    }
    
    
	// we need to copy this movie file to the MasterOriginalServer
	NSString* srcPath = self.mRecordedMoviePath;
	NSString* serverPath = getDailyFolderPathForMasterOriginalVideos();	// 2010-11-11steve [[NSUserDefaults standardUserDefaults] stringForKey:sPrefOriginalVideosFolder];
    
    NSLog(@"   srcPath = %@",srcPath);
    NSLog(@"serverPath = %@",serverPath);
    
	NSString* destPath = [serverPath stringByAppendingPathComponent:[srcPath lastPathComponent]];
	NSError* error = nil;
    
    BOOL didCopy = NO; // 2013-10-23steve
    BOOL srcExists = [[NSFileManager defaultManager] fileExistsAtPath:srcPath]; // 2013-10-23steve
    if (srcExists) { // 2013-10-23steve
        bool autoSaveWithoutPrompting = [[NSUserDefaults standardUserDefaults] boolForKey:sPrefAutoSaveWithoutPrompting];
        if ( !autoSaveWithoutPrompting) {
            // then we wanted to rename it
            NSString* tempname = [[NSUserDefaults standardUserDefaults] objectForKey:cPrefRenameString ];
            
            if (tempname==nil || [tempname length]==0) { // 2013-10-23steve
                tempname = [self.mRecordedMoviePath lastPathComponent];
            }
            tempname = [tempname stringByDeletingPathExtension];

            NSString* tempDestPath = [serverPath stringByAppendingPathComponent:tempname];
            
            NSLog(@"tempDestPath = %@",tempDestPath); // 2013-10-23steve DEBUG
            
            NSString* ext = sDefaultMovieExtension; // 2014-07-28steve @"mov";
            if ([srcPath pathExtension])
                ext = [srcPath pathExtension];
            tempDestPath = [tempDestPath stringByAppendingPathExtension:[srcPath pathExtension]];
            
            BOOL exists = [[NSFileManager defaultManager] fileExistsAtPath:tempDestPath];
            if (exists) {
                
                // WE NEED to PROMPT WAY Before HERE
                
                BOOL didDelete = [[NSFileManager defaultManager] removeItemAtPath:tempDestPath error:&error];
                if (error) {
                    NSLog(@"removeItemAtPath returned error = %@", error);
                }
            }
            if (error==nil) { // 2013-10-18steve was able to delete above file then use new renamed name
                destPath = [NSString stringWithFormat:@"%@",tempDestPath];
            }
            
        }
        
        if (interfaceWithTeps) {
            NSString* origDestPath = [NSString stringWithFormat:@"%@",destPath];
            destPath = [destPath stringByDeletingLastPathComponent];
            NSString* ext = sDefaultMovieExtension; // 2014-07-28steve @"mov";
            if ([srcPath pathExtension])
                ext = [srcPath pathExtension];

            // for TODDL = we need to b - basename_whatever.mp4"
            tepsNameOfItem = [NSString stringWithFormat:@"%@_001",tepsNameOfItem]; // 2014-07-28steve
            
            destPath = [[self.fCurrentTepsDailyFolderPath stringByAppendingPathComponent:tepsNameOfItem] stringByAppendingPathExtension:ext];
            
            NSLog(@"New Name to Save Movie For Teps is: %@",destPath);
        }
        
        NSLog (@ "Inside %@ => %@...Will Copy Movie to: %@", self, NSStringFromSelector(_cmd),destPath ); // 2013-10-18steve
        
        BOOL doDeleteIfExists = YES; // 2014-07-28steve
        if (doDeleteIfExists) {
            BOOL exists = [[NSFileManager defaultManager] fileExistsAtPath:destPath];
            if (exists) {
                NSError* error = nil;
                [[NSFileManager defaultManager] removeItemAtPath:destPath error:&error];
                if (error) {
                    NSLog (@"%@-%@ - problem with removeItemAtPath = [%@]",destPath);
                }
            }
            
        }
        
        
        didCopy = [[NSFileManager defaultManager] copyItemAtPath:srcPath
                                                               toPath:destPath
                                                                error:&error];
        
        if (error)
        {
            NSLog (@"2010-11-09steve Error in copying file.. srcPath[%@] to destPath=[%@]", srcPath, serverPath );
            NSLog (@"2010-11-09steve error = %@", error );
        } else {
            // 2014-07-25steve - WARNING !!!!!!!!!  MAY NEED to create function to create thumbnail from LOCAL path first!!!!!!!
            NSString* thumbnailPath = @"NO-Thumbanil-Created";
            if (interfaceWithTeps) {    // 2014-07-25steve
                thumbnailPath = [self createMovieThumbnail:srcPath toDestItem:destPath];
            }
            else {  // 2014-06-26steve - create the movie thumbnail
                thumbnailPath = [self createMovieThumbnail:srcPath];
            }
            
            NSLog(@"2014-07-25steve - created movie thumbanil at [%@]",thumbnailPath);
            
            BOOL interfaceWithTeps = [[NSUserDefaults standardUserDefaults] boolForKey:@"prefInterfaceWithTeps"]; // 2014-08-08steve
            if (interfaceWithTeps) {
                
                NSDictionary* param2 = [NSDictionary dictionaryWithObjectsAndKeys: @"Teps-Video.app",@"senderApp",
                                        @"statusMovieThumbnailCreated",@"action",
                                        nil];
                NSNotification* note = [NSNotification notificationWithName:sTepsXNotification // 2014-08-08steve
                                                                     object:nil	// must be NSString ONLY
                                                                   userInfo:param2];
                
                [[NSDistributedNotificationCenter defaultCenter] performSelectorOnMainThread:@selector(postNotification:)
                                                                                  withObject:note
                                                                               waitUntilDone:NO];
            }
            
        }

    } else {
        NSLog (@ "Inside %@ => %@... srcPath does NOT exist...", self, NSStringFromSelector(_cmd) );

    }
    
    
	
	return didCopy;
}


-(IBAction) closeSheet:(id)sender
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );

    
    NSInteger senderTag = [sender tag];
	NSInteger returnCode = NSRunContinuesResponse;
	if ( senderTag==99 )
	{
		// user did NOT want to save
		returnCode = NSRunAbortedResponse;
	} else {
        // bool doRename = YES; // 2013-10-18steve - if we are in this method - we must have prompted user to enter a new name
        bool doRename = [[NSUserDefaults standardUserDefaults] boolForKey:@"prefSaveMovieWithCustomName"]; // 2013-10-23steve
        if ( doRename )
        {
            self.mTempRenameString = [[NSUserDefaults standardUserDefaults] objectForKey:cPrefRenameString ];
            NSLog (@ "Inside %@ => %@...self.mTempRenameString = [%@]", self, NSStringFromSelector(_cmd),self.mTempRenameString);

            
            // 2013-10-07steve - make sure string is valid - no empty string allowed
            self.mTempRenameString = [self.mTempRenameString stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
            if ([self.mTempRenameString length]==0)
                self.mTempRenameString = nil;
            
            // 2013-10-18steve - update the pref file - in case we need to use it later on
            if (self.mTempRenameString)
                [[NSUserDefaults standardUserDefaults] setObject:self.mTempRenameString forKey:cPrefRenameString ];
            else
                [[NSUserDefaults standardUserDefaults] setObject:@"" forKey:cPrefRenameString ];

            NSLog(@"self.mRecordedMoviePath is %@", self.mRecordedMoviePath);
            
            BOOL doOverwriteFile = YES; // 2013-10-18steve
            
            if ( self.mTempRenameString )
            {
                NSString *ext = [self.mRecordedMoviePath pathExtension]; // 2013-10-18steve
                NSString *basePath = getWorkingDirectoryPath(sLocalWorkingFolderName);
                NSString *tempString = [basePath stringByAppendingPathComponent:[mTempRenameString stringByDeletingPathExtension]];
                
                tempString = [NSString stringWithFormat:@"%@_%@", tempString,[self.mRecordedMoviePath lastPathComponent]];
                
                NSLog(@"new renamed file is %@", tempString);

                NSFileManager* mgr = [NSFileManager defaultManager];
                NSError* myErr = nil;
                [mgr moveItemAtPath:self.mRecordedMoviePath toPath:tempString error:&myErr];
                NSLog (@ "2010-11-01steve myErr = %@", myErr );
                
                self.mRecordedMoviePath = [NSString stringWithString:tempString];
                
            } else {
                // 2013-10-04steve - no blank entry allowed
                [[NSUserDefaults standardUserDefaults] setObject:@"Invalid Name..." forKey:cPrefErrorMessage];
                NSBeep();
                return; // DO NOT Continue if u dont get correct string
            }
        }
        else
        {
            // if the rename textfield is not visible - then we need to prefill with default name
            NSLog(@"self.mRecordedMoviePath is %@", self.mRecordedMoviePath);
            // make sure it has been created -
            // there are cases where recording started but camera was not setup
            // so no output will get created
            BOOL exists = [[NSFileManager defaultManager] fileExistsAtPath:self.mRecordedMoviePath];
            if ( exists) {
                [[NSUserDefaults standardUserDefaults] setObject:[self.mRecordedMoviePath lastPathComponent] forKey:cPrefRenameString ]; // 2013-10-23steve - in case we dont show the rename textfield
            } else {
                [[NSUserDefaults standardUserDefaults] setObject:@"" forKey:cPrefRenameString ]; // 2013-10-23steve

            }
        }
        
    }

    [ibTextField1 ibUpdateNSTextFieldWithString:[self getSystemReadyString] //  2014-10-07steve @"Ready..."
                                         doWait:NO
                                      withColor:[NSColor greenColor]];
    
    
    [NSApp endSheet:[sender window] returnCode:returnCode ];


}

-(IBAction) closePromptSheet:(id)sender // 2014-10-17steve
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
    
    
    NSInteger senderTag = [sender tag];
	NSInteger returnCode = NSRunContinuesResponse;
	if ( senderTag==99 )
	{
		// user did NOT want to save
		returnCode = NSRunAbortedResponse;
	} else {
        // user wants to record
        
        
    }
    
    [NSApp endSheet:[sender window] returnCode:returnCode ];
    
}


- (void)didEndSheet:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo
{
	NSLog (@ "Inside %@ => %@... with returnCode=[%i] and ContextInfo=[%@]", self, NSStringFromSelector(_cmd), returnCode, contextInfo );

	[sheet orderOut:self];

	if ( returnCode==NSRunContinuesResponse )
	{
		BOOL doOpenQTPlayer = [[NSUserDefaults standardUserDefaults] boolForKey:sPrefOpenQTPlayerAfterSavedMovie];
		if ( doOpenQTPlayer )
		{
			// NSLog (@"2010-11-01steve - try to open the recorded movie after didEndSheet movie = %@",self.mRecordedMoviePath );
			[[NSWorkspace sharedWorkspace] openURL:[NSURL fileURLWithPath:self.mRecordedMoviePath]];
		}

		
		bool doNewMethod = true;
		if ( doNewMethod )
		{
			[self saveMovieToServer]; // 2010-11-11steve

			[ibTextField1 ibUpdateNSTextFieldWithString:@"Ready Again..."	// 2010-11-15steve
												 doWait:NO 
											  withColor:[NSColor greenColor]];
			
			
		}
		else
		{
			// we need to copy this movie file to the MasterOriginalServer
			NSString* srcPath = self.mRecordedMoviePath;
			NSString* serverPath = getDailyFolderPathForMasterOriginalVideos();	// 2010-11-11steve [[NSUserDefaults standardUserDefaults] stringForKey:sPrefOriginalVideosFolder];
			NSString* destPath = [serverPath stringByAppendingPathComponent:[srcPath lastPathComponent]];
			NSError* error = nil;
			[[NSFileManager defaultManager] copyItemAtPath:srcPath 
																   toPath:destPath
																	error:&error];
			
			if (error)
			{
				NSLog (@"2010-11-09steve Error in copying file.. srcPath[%@] to destPath=[%@]", srcPath, serverPath );
				NSLog (@"2010-11-09steve error = %@", error );
			}
		}
		
		
	}
	

}

-(void) preStartRecording2:(NSString*) childname // 2014-11-05steve
{
    NSLog (@ "Inside %@ => %@.....", self, NSStringFromSelector(_cmd) );

    if ([childname length]) {
        self.fCurrentTepsCustomerID = [NSString stringWithFormat:@"%@",childname]; // 2014-07-15steve
        
        self.fCurrentTepsDailyFolderPath =  getDailyFolderPathForMasterOriginalVideos(); // always use this folder if in this mode
        
        if (self.mVideoStatusWindowController)
            [self.mVideoStatusWindowController setTextField2:[NSString stringWithFormat:@"CustomerID=[%@]",self.fCurrentTepsCustomerID]
                                                      doWait:NO
                                                   withColor:[NSColor magentaColor]]; // 2014-07-13steve
        
        
        [self startRecordingPart2:nil]; // 2014-10-25steve

    }
}


// 2014-10-17steve
- (void)didEndPromptSheet:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo
{
	NSLog (@ "Inside %@ => %@... with returnCode=[%li] and ContextInfo=[%@]", self, NSStringFromSelector(_cmd), (long)returnCode, contextInfo );
	
    
    if ( returnCode==NSRunContinuesResponse ) {
        NSLog (@ "Inside %@ => %@... Got NSRunContinuesResponse.....", self, NSStringFromSelector(_cmd) );
        
        NSString* childname = [[NSUserDefaults standardUserDefaults] stringForKey:cPrefRenameString]; // 2014-10-25steve
        childname = [childname stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
        if ([childname length]) {
            
            [self preStartRecording2:childname]; // 2014-11-05steve - go thru one function
//            self.fCurrentTepsCustomerID = [NSString stringWithFormat:@"%@",childname]; // 2014-07-15steve
//            
//            self.fCurrentTepsDailyFolderPath =  getDailyFolderPathForMasterOriginalVideos(); // always use this folder if in this mode
//
//            if (self.mVideoStatusWindowController)
//                [self.mVideoStatusWindowController setTextField2:[NSString stringWithFormat:@"CustomerID=[%@]",self.fCurrentTepsCustomerID]
//                                                          doWait:NO
//                                                       withColor:[NSColor magentaColor]]; // 2014-07-13steve
//
//            
//            [self startRecordingPart2:nil]; // 2014-10-25steve
        }
        else {
            [[NSUserDefaults standardUserDefaults] setObject:@"Please Enter A Valid Name..." forKey:cPrefErrorMessage];
            NSBeep();
            return; // do not close the window as bad entry
        }

    } else {
        _mIsRecording = NO; // 2014-11-05steve - user cancelled or something
    }

    [[NSUserDefaults standardUserDefaults] setObject:@"" forKey:cPrefErrorMessage]; // 2014-10-25steve

    [sheet orderOut:self];

}



-(IBAction) doClear:(id)sender
{
	[[NSUserDefaults standardUserDefaults] setObject:@"" forKey:cPrefRenameString ];
	[[NSUserDefaults standardUserDefaults] setObject:@"" forKey:cPrefErrorMessage ]; // 2013-10-04steve
}

//-(IBAction) doRename:(id)sender
//{
//	[self closeSheet:sender];
//	
//}



// doAction2 - this gives user option to rename the movie file
- (IBAction) doAction2:(id)sender
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	
	
	// 2010-11-01steve
	// [[NSUserDefaults standardUserDefaults] setObject:[NSString stringWithString:@""] forKey:cPrefRenameString ];
    [self doClear:nil]; // 2013-10-04steve
    
    
	[NSApp beginSheet:mSaveAsPanel
	   modalForWindow:mMainWindow
		modalDelegate:self 
	   didEndSelector:@selector(didEndSheet:returnCode:contextInfo:) 
		  contextInfo:nil];
	

	
}


// 2010-10-22steve
- (IBAction) doAction1:(id)sender
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );

	// [self doTestQTKit];
	
	NSLog ( @"2010-10-25steve mRecordedMoviePath = [%@]", self.mRecordedMoviePath );
	
	// [self testMergeMovies:self.mRecordedMoviePath];
	
	// 2010-10-25steve
	NSNotificationCenter* nc = [NSNotificationCenter defaultCenter];
	[nc addObserver:self
		   selector:@selector(handleNSThreadWillExitNotification:)
			   name:NSThreadWillExitNotification
			 object:nil];
	
	[mProgressIndicator startAnimation:nil];
	
	[self performSelectorOnMainThread:@selector(testMergeMovies:) withObject:self.mRecordedMoviePath waitUntilDone:NO]; // 2010-10-25steve		
	

}


-(void) doSendToBackLater // 2014-10-07steve
{
    NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );

	[self performSelector:@selector(sendToBack:) withObject:nil afterDelay:0.5];

}

// Add these start and stop recording actions, and specify the output destination for your recorded media. The output is a QuickTime movie.

- (IBAction)toggleRecording:(id)sender;
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	if ( _mIsRecording )
	{
		_mIsRecording = NO;
		[self stopRecording:nil];
	}
	else
	{
		_mIsRecording = YES;
		[self startRecording:nil];
	}
    
    BOOL forSTB = [[NSUserDefaults standardUserDefaults] boolForKey:@"prefForSTB"]; // 2014-10-25steve
    if ( forSTB) { // 2014-10-25steve
        NSInteger tag = [sender tag]; // 2014-10-07steve
        if (tag==88) {
            
            [self performSelectorOnMainThread:@selector(doSendToBackLater)
                                   withObject:nil
                                waitUntilDone:NO]; // 2014-10-07steve
            
            
        }
    }
	
    
}


-(BOOL) promptToOverwriteExistingMovie // 2014-10-17steve
{
    
    BOOL okToOverwrite = NO;
    
    BOOL customerMovieFileExists = [self checkIfTepsCustomerMovieFileExistsForExt:sDefaultMovieExtension]; // 2014-10-12steve
    if ( customerMovieFileExists) {
        
        [[NSRunningApplication currentApplication] activateWithOptions:(NSApplicationActivateAllWindows | NSApplicationActivateIgnoringOtherApps)];
        
        // cannot record - no valid customerID set
        NSBeep();
        
        NSString* informativestring = [NSString stringWithFormat:@"\nAre you SURE you want to overwrite it?\n\n"];
        NSString* messageText = [NSString stringWithFormat:@"WARNING! A video file already exists for this customer [%@]",self.fCurrentTepsCustomerID];
        NSAlert * alert = [NSAlert alertWithMessageText:messageText
                                          defaultButton:@"Cancel Recording"
                                        alternateButton:nil
                                            otherButton:@"YES - Record Over"
                              informativeTextWithFormat:@"%@",informativestring];
        NSInteger response = [alert runModal];
        if ( response == NSAlertOtherReturn) {
            okToOverwrite = YES;
        } else {
            okToOverwrite = NO;
        }
        
    } else {
        okToOverwrite = YES;
    }

    return okToOverwrite;
}

-(IBAction)startRecordingPart3:(id)sender // 2014-10-25steve
{
    NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );

    
    
    
    // 2014-07-25steve
    BOOL interfaceWithTeps = [[NSUserDefaults standardUserDefaults] boolForKey:@"prefInterfaceWithTeps"];
    if (interfaceWithTeps) {
        
        BOOL recordAny = [self promptToOverwriteExistingMovie]; // 2014-10-17steve - re-do
        if ( recordAny==NO) {
            return;
        }
        
        
        if (self.fCurrentTepsCustomerID==nil )
        {
            
            _mIsRecording = NO; // 2014-07-25steve
            
            [[NSRunningApplication currentApplication] activateWithOptions:(NSApplicationActivateAllWindows | NSApplicationActivateIgnoringOtherApps)];
            
            // cannot record - no valid customerID set
            NSBeep();
            
            NSString* informativestring = [NSString stringWithFormat:@"There is NO Valid Customer ID From Teps System...\n[%@]",self.fCurrentTepsCustomerID];
            NSAlert * alert = [NSAlert alertWithMessageText:@"!!!!!  CANNOT Start Recording  !!!!!!!"
                                              defaultButton:@"OK"
                                            alternateButton:nil
                                                otherButton:nil
                                  informativeTextWithFormat:@"%@",informativestring];
            [alert runModal];
            
            
            return;
        } else {
            [self.fWarningTimer1 invalidate]; // 2014-07-25steve stop the timer
            self.fWarningTimer1 = nil;
            
        }
        
        BOOL dothis = NO; // 2014-07-25steve
        if ( dothis) {
            // send notification to teps...
            NSDictionary* param2 = [NSDictionary dictionaryWithObjectsAndKeys: @"Teps-Video.app",@"senderApp",@"requestImageID",@"action",nil];
            NSNotification* note = [NSNotification notificationWithName:sTepsVideoAppNotification
                                                                 object:nil	// must be NSString ONLY
                                                               userInfo:param2];
            
            [[NSDistributedNotificationCenter defaultCenter] performSelectorOnMainThread:@selector(postNotification:)
                                                                              withObject:note
                                                                           waitUntilDone:NO];
            
        }
    }
    
    
	
	// 2010-11-11steve self.mRecordedMoviePath = [NSString stringWithFormat:@"/%@_%@.mov",cTempRecordedMovieBaseName, [_mDateFormatter stringFromDate:[NSDate date]]]; // 2010-10-25steve
    
	self.mRecordedMoviePath = [self.mCurWorkingPath stringByAppendingPathComponent:[NSString stringWithFormat:@"%@_%@.%@",cTempRecordedMovieBaseName, [_mDateFormatter stringFromDate:[NSDate date]],sDefaultMovieExtension ]]; // 2014-07-28steve // 2010-11-11steve
    
    
    
	[mCaptureMovieFileOutput recordToOutputFileURL:[NSURL fileURLWithPath:self.mRecordedMoviePath]];
    
	[self setupRecordingTicks]; // 2010-11-02steve
    
    
    
    [self sendRecordingStatusNotification]; // 2014-08-08steve
    
    _mIsRecording = YES; // 2014-11-05steve

}

-(IBAction)startRecordingPart2:(id)sender // 2014-11-05steve
{
    NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
    NSTimeInterval delay = [[NSUserDefaults standardUserDefaults] floatForKey:@"prefDelayRecordingInSeconds"]; // 2014-11-05steve
    
    
    [self performSelector:@selector(startRecordingPart3:) withObject:sender afterDelay:delay];

}

- (IBAction)startRecording:(id)sender
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
    
    _mIsRecording = YES; // 2014-07-25steve

    BOOL promptForCustomerIDFirst = [[NSUserDefaults standardUserDefaults] boolForKey:@"prefPromptForIDBeforeRecording"]; // 2014-11-06steve  forKey:<#(NSString *)#>  YES; // 2014-10-17steve for the grove
    if (promptForCustomerIDFirst) { // 2014-10-17steve
        
        [[NSUserDefaults standardUserDefaults] setObject:@"" forKey:cPrefErrorMessage]; // 2014-10-25steve clear messages if any

        // need to show dialog to get customer ID
        
        [NSApp beginSheet:ibPromptForChildNamePanel // 2014-10-17steve
           modalForWindow:mMainWindow
            modalDelegate:self
           didEndSelector:@selector(didEndPromptSheet:returnCode:contextInfo:)
              contextInfo:nil];
        
        return;
    }
    
    [self startRecordingPart2:sender]; // 2014-10-25steve
    
    /* 2014-10-25steve - call startRecordingPart2
     
    // 2014-07-25steve
    BOOL interfaceWithTeps = [[NSUserDefaults standardUserDefaults] boolForKey:@"prefInterfaceWithTeps"];
    if (interfaceWithTeps) {
        
        BOOL recordAny = [self promptToOverwriteExistingMovie]; // 2014-10-17steve - re-do
        if ( recordAny==NO) {
            return;
        }
        

        if (self.fCurrentTepsCustomerID==nil )
        {
            
            _mIsRecording = NO; // 2014-07-25steve

            [[NSRunningApplication currentApplication] activateWithOptions:(NSApplicationActivateAllWindows | NSApplicationActivateIgnoringOtherApps)];

            // cannot record - no valid customerID set
            NSBeep();
            
            NSString* informativestring = [NSString stringWithFormat:@"There is NO Valid Customer ID From Teps System...\n[%@]",self.fCurrentTepsCustomerID];
            NSAlert * alert = [NSAlert alertWithMessageText:@"!!!!!  CANNOT Start Recording  !!!!!!!"
                                              defaultButton:@"OK"
                                            alternateButton:nil
                                                otherButton:nil
                                  informativeTextWithFormat:@"%@",informativestring];
            [alert runModal];
            

            return;
        } else {
            [self.fWarningTimer1 invalidate]; // 2014-07-25steve stop the timer
            self.fWarningTimer1 = nil;

        }
        
        BOOL dothis = NO; // 2014-07-25steve
        if ( dothis) {
            // send notification to teps...
            NSDictionary* param2 = [NSDictionary dictionaryWithObjectsAndKeys: @"Teps-Video.app",@"senderApp",@"requestImageID",@"action",nil];
            NSNotification* note = [NSNotification notificationWithName:sTepsVideoAppNotification
                                                                 object:nil	// must be NSString ONLY
                                                               userInfo:param2];
            
            [[NSDistributedNotificationCenter defaultCenter] performSelectorOnMainThread:@selector(postNotification:)
                                                                              withObject:note
                                                                           waitUntilDone:NO];
            
        }
    }
    
    
	
	// 2010-11-11steve self.mRecordedMoviePath = [NSString stringWithFormat:@"/%@_%@.mov",cTempRecordedMovieBaseName, [_mDateFormatter stringFromDate:[NSDate date]]]; // 2010-10-25steve

	self.mRecordedMoviePath = [self.mCurWorkingPath stringByAppendingPathComponent:[NSString stringWithFormat:@"%@_%@.%@",cTempRecordedMovieBaseName, [_mDateFormatter stringFromDate:[NSDate date]],sDefaultMovieExtension ]]; // 2014-07-28steve // 2010-11-11steve

	[mCaptureMovieFileOutput recordToOutputFileURL:[NSURL fileURLWithPath:self.mRecordedMoviePath]];

	[self setupRecordingTicks]; // 2010-11-02steve
    
    

    [self sendRecordingStatusNotification]; // 2014-08-08steve

     */
}

- (IBAction)stopRecording:(id)sender
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
    
    _mIsRecording = NO; // 2014-07-25steve

	[mCaptureMovieFileOutput recordToOutputFileURL:nil];

	[self stopRecordingTicks]; // 2010-11-02steve
    
    
    [self sendRecordingStatusNotification]; // 2014-08-08steve
	
}
- (void)beginEditingMovie:(id)movie
{
    NSString * srcPath = self.mRecordedMoviePath;
    //open a QTMovieView with this movie open for editing
    if ([[NSApp delegate] respondsToSelector:@selector(beginEditingMovie:)] )
    {
        [[NSApp delegate] beginEditingMovie:srcPath];
    }
}
// Do something with your QuickTime movie at the path you've specified at /Users/Shared/My Recorded Movie.mov"
- (void)captureOutput:(QTCaptureFileOutput *)captureOutput didFinishRecordingToOutputFileAtURL:(NSURL *)outputFileURL forConnections:(NSArray *)connections dueToError:(NSError *)error
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );

	if ( mDoMergeVideo )
	{
		[self doAction1:nil];
	}
	else
	{
		bool autoSaveWithoutPrompting = [[NSUserDefaults standardUserDefaults] boolForKey:sPrefAutoSaveWithoutPrompting];
		if ( autoSaveWithoutPrompting )
		{
			BOOL success = [self saveMovieToServer]; // 2010-11-11steve
			[ibTextField1 ibUpdateNSTextFieldWithString:[self getSystemReadyString] // 2014-10-07steve @"Ready..."
												 doWait:NO 
												withColor:[NSColor greenColor]];
            if (self.mVideoStatusWindowController)
                [self.mVideoStatusWindowController setTextField1:@"OFF - NOT Recording..." doWait:NO withColor:[NSColor redColor]]; // 2014-07-13steve
            
            if(success)
            {
                // 2013-10-04steve [self beginEditingMovie:nil];
            }
			
		}
		else
		{
            // prompt to save movie to local folder
			[self doAction2:nil];
		}

	}
}



#pragma mark -
+ (QTMovie *)appendMovies:(NSArray *)inMoviesArray destFilePath:(NSString *)filePath
{
	QTMovie			*outMovie = NULL; 
	NSDictionary	*dict = nil;
	NSNumber		*isEditable = nil;
	BOOL			success = NO;
	NSEnumerator	*movieObjEnumerator1 = nil, *movieObjEnumerator2 = nil;
	NSSize			savedMovieSizes = {0,0};
	
	if (!inMoviesArray)
		goto bail;
	
	if (!filePath) 
		goto bail;
	
	// create our destination movie which we'll use to
	// write all our movie clips
	outMovie = [QTMovie movie];
	if (!outMovie)
		goto bail;
	
	// in order to add segments to our newly created movie
	// we must make sure the "QTMovieEditableAttribute" 
	// attribute is set
	isEditable = [NSNumber numberWithBool:YES];
	[outMovie setAttribute:isEditable forKey:QTMovieEditableAttribute];
	
	movieObjEnumerator1 = [inMoviesArray objectEnumerator];
	QTMovie *aQTMovie;
	
	// first we'll iterate over every movie clip to find the
	// one with the biggest natural size -- we'll then use
	// this size for our new movie
	while (aQTMovie = [movieObjEnumerator1 nextObject]) 
	{
		NSValue *naturalSize = [aQTMovie attributeForKey:QTMovieNaturalSizeAttribute];
		NSSize theSize = [naturalSize sizeValue];
		
		// save the biggest dimensions we find 
		// for height/width to use for our destination
		// movie
		if (theSize.width > savedMovieSizes.width)
		{
			savedMovieSizes.width = theSize.width;
		}
		
		if (theSize.height > savedMovieSizes.height)
		{
			savedMovieSizes.height = theSize.height;
		}
	}
	
	// we'll use the biggest height/width values found for
	// our destination movie
	[outMovie setAttribute:[NSValue valueWithSize:savedMovieSizes] forKey:QTMovieNaturalSizeAttribute];
	
	// now iterate each clip and add it to our destination
	// movie
	movieObjEnumerator2 = [inMoviesArray objectEnumerator];
	while (aQTMovie = [movieObjEnumerator2 nextObject]) 
	{
		NSNumber *timeScale = [aQTMovie attributeForKey:QTMovieTimeScaleAttribute];
		
		// make a QTTime corresponding to time 0 for our movie
		QTTime timeZero = QTMakeTime(0,[timeScale longValue]);
		
		// make a QTTimeRange for the entire source movie 
		// (time 0 to movie end)
		QTTimeRange timeRange = QTMakeTimeRange(timeZero, [aQTMovie duration]);
		
		// insert all src movie tracks from this clip into 
		// our destination movie - specify a time range of
		// the entire src. movie clip
		[outMovie insertSegmentOfMovie:aQTMovie timeRange:timeRange 
								atTime:[outMovie duration]];
	}
	
	// create a dict. with the movie flatten attribute (QTMovieFlatten)
	// which we'll use to flatten the movie to a file below
	dict = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] 
									   forKey:QTMovieFlatten];
	if (dict)
	{
		// create a new movie file and flatten the movie to the file
		
		// passing the QTMovieFlatten attribute here means the movie
		// will be flattened
		
		NSError* error = nil;
		success = [outMovie writeToFile:filePath withAttributes:dict error:&error];
		NSLog (@ "2010-10-20steve [%@] writeToFile returned error: %@", [outMovie attributeForKey:QTMovieFileNameAttribute], error );
		
		
	}
	
bail:
	return outMovie;
}


-(void) testMergeMovies2
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	
	NSError* errorPtr = nil;
	NSString* movieBegPath = @"/movieBeg.mov";
	NSString* movieEndPath = @"/movieEnd.mov";
	QTMovie* movie1 = [QTMovie movieWithFile:movieBegPath error:&errorPtr];
	NSLog ( @"2010-10-20steve movie1 = %@", movie1 );
	QTMovie* movie2= [QTMovie movieWithFile:movieEndPath error:&errorPtr];
	NSLog ( @"2010-10-20steve movie2 = %@", movie2 );
	NSArray* myArray = [NSArray arrayWithObjects:movie1,movie2,nil];
	NSLog ( @"2010-10-20steve myArray = %@", myArray );
	
	
	NSDateFormatter *f = [[[NSDateFormatter alloc] init] autorelease];
	[f setFormatterBehavior:NSDateFormatterBehavior10_4];
	[f setDateFormat:@"yyyy_MM_dd_HH_mm_ss"];
	NSString* outputMoviePath = [NSString stringWithFormat:@"/finalMovie_%@.mov",[f stringFromDate:[NSDate date]]];
	
	{
		NSFileManager* mgr = [NSFileManager defaultManager];
		BOOL isDirectory = NO;
		BOOL exists = [mgr fileExistsAtPath:outputMoviePath isDirectory:&isDirectory];
		if ( exists && !isDirectory )
		{
			[mgr removeItemAtPath:outputMoviePath error:&errorPtr];
			if ( errorPtr)
				NSLog ( @"2010-10-20steve removeItemAtPath returned: %@", errorPtr );
		}				
		
	}
	
	
	
	QTMovie* finalMove = [MyRecorderController appendMovies:myArray destFilePath:outputMoviePath];
	
	NSLog ( @"2010-10-20steve Done with appendMovies....." );
	
}


-(void) testMergeMovies1
{
	
	NSLog ( @"2010-10-21steve 1111111111111111111111111111111111111111111111111111111111111111111111111111111111" );
	
	//bool doSaveInThread = false;
	BOOL doFlatten = false; // 2010-101-25steve
	
	
	
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	
	NSDateFormatter *f = [[[NSDateFormatter alloc] init] autorelease];
	[f setFormatterBehavior:NSDateFormatterBehavior10_4];
	[f setDateFormat:@"yyyy_MM_dd_HH_mm_ss"];
	NSString* outputMoviePath = [NSString stringWithFormat:@"/myMerged_%@.mov",[f stringFromDate:[NSDate date]]];
	
	
	NSError* errorPtr = nil;
	
	
	NSLog (@ "Inside %@ => %@...Instantiating newMovie...", self, NSStringFromSelector(_cmd) );
	QTMovie* newMovie= [QTMovie movie];		// [[QTMovie alloc] initToWritableFile:outputMoviePath error:&errorPtr];
	if ( errorPtr )
	{
		NSLog ( @"2010-10-20steve newMovie = %@", newMovie );
		NSLog ( @"2010-10-20steve [initToWritableFile:newMovie] returned error: %@", errorPtr );
		return;
	}
	[newMovie setAttribute:[NSNumber numberWithBool:YES] forKey:QTMovieEditableAttribute];
	NSLog ( @"2010-10-20steve %@ has attributes: %@",[newMovie attributeForKey:QTMovieFileNameAttribute], [newMovie movieAttributes] );
	
	
	NSString* movieBegPath = @"/movieBeg.mov";
	NSString* movieEndPath = @"/movieEnd.mov";
	errorPtr = nil;
	QTMovie* movie1 = [QTMovie movieWithFile:movieBegPath error:&errorPtr];
	if ( errorPtr )
	{
		NSLog ( @"2010-10-20steve movie1 = %@", movie1 );
		NSLog ( @"2010-10-20steve [movieWithFile:movieBegPath] returned error: %@", errorPtr );
		
	}
	if ( movie1 )
	{
		[movie1 setAttribute:[NSNumber numberWithBool:YES] forKey:QTMovieEditableAttribute];
		
	}
	QTMovie* movie2 = [QTMovie movieWithFile:movieEndPath error:&errorPtr];
	if ( errorPtr )
	{
		NSLog ( @"2010-10-20steve movie2 = %@", movie2 );
		NSLog ( @"2010-10-20steve [movieWithFile:movieEndPath] returned error: %@", errorPtr );
	}
	
	NSLog ( @"2010-10-21steve 2222222222222222222222222222222222222222222222222222222222222222222222222222222222" );
	
	if (!( movie1 && movie2 ))
	{
		return;
	}
	
	
	NSLog ( @"2010-10-21steve 3333333333333333333333333333333333333333333333333333333333333333333333333333333333" );
	
	QTTime timeZero = QTMakeTime(0,600);
	QTTimeRange selection1 = QTMakeTimeRange(timeZero, [movie1 duration]);
	QTTimeRange selection2 = QTMakeTimeRange(timeZero, [movie2 duration]);
	
	[movie1 setSelection:selection1];
	[newMovie appendSelectionFromMovie:movie1];
	[newMovie updateMovieFile];
	
	[movie2 setSelection:selection2];
	[newMovie appendSelectionFromMovie:movie2];
	[newMovie updateMovieFile];
	
//	if ( doSaveInThread==false )
//	{
//		[mProgressIndicator stopAnimation:nil];
//		NSLog ( @"2010-10-21steve pppppppppppppp      %@ Done - NOT doing in thread      pppppppppppppp", NSStringFromSelector(_cmd) );
//		return;
//	}
	
	NSDictionary* dict = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:doFlatten]  // 2010-10-25steve
													 forKey:QTMovieFlatten];
	
	{
		NSFileManager* mgr = [NSFileManager defaultManager];
		BOOL isDirectory = NO;
		BOOL exists = [mgr fileExistsAtPath:outputMoviePath isDirectory:&isDirectory];
		if ( exists && !isDirectory )
		{
			[mgr removeItemAtPath:outputMoviePath error:&errorPtr];
			if ( errorPtr)
				NSLog ( @"2010-10-20steve ");
			
		}
		
		bool doAsThread = true;
		if ( doAsThread )
		{
			NSLog ( @"2010-10-21steve yyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyy" );
			NSDictionary* param = [NSDictionary dictionaryWithObjectsAndKeys:newMovie,@"sourceMovie",outputMoviePath,@"outputMoviePath", dict,@"attributes",  nil];
			NSThread* myThread = [[NSThread alloc] initWithTarget:self selector:@selector(myThread1:) object:param];

			NSMutableDictionary* threadDictionary = [myThread threadDictionary];
			[threadDictionary setObject:@"something to specific about this thread" forKey:@"key1"];
			[threadDictionary setObject:@"something to specific about this thread222" forKey:@"key2"];
			
			[myThread setName:@"myThread1"];
			
			[myThread start];
			
		}
		else
		{
			NSLog ( @"2010-10-21steve zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz" );
			errorPtr = nil;
			[newMovie writeToFile:outputMoviePath withAttributes:dict error:&errorPtr];
			if( errorPtr )
			{
				NSLog ( @"2010-10-20steve newMovie writeToFile has error: %@", errorPtr );
			}
		}
		
		
	}
	
	
	NSLog ( @"2010-10-21steve pppppppppppppp      %@ DONE      pppppppppppppp", NSStringFromSelector(_cmd) );

}


-(void) testMergeMovies:(NSString*) middleMoviePath
{
	
	NSLog ( @"2010-10-21steve 1111111111111111111111111111111111111111111111111111111111111111111111111111111111" );
	
	NSDate* startDate = [NSDate date]; // 2010-11-01steve
	
	//bool doSaveInThread = false;
	BOOL doFlatten = true; // 2010-101-25steve
	
	[mProgressIndicator performSelectorOnMainThread:@selector(startAnimation:) withObject:nil waitUntilDone:NO];
	
	
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	
	NSDateFormatter *f = [[[NSDateFormatter alloc] init] autorelease];
	[f setFormatterBehavior:NSDateFormatterBehavior10_4];
	[f setDateFormat:@"yyyy_MM_dd_HH_mm_ss"];
	NSString* outputMoviePath = [NSString stringWithFormat:@"/myMerged_%@.mov",[f stringFromDate:[NSDate date]]];
	
	
	NSError* errorPtr = nil;
	
	
	NSLog (@ "Inside %@ => %@...Instantiating newMovie...", self, NSStringFromSelector(_cmd) );
	QTMovie* newMovie= [QTMovie movie];		// [[QTMovie alloc] initToWritableFile:outputMoviePath error:&errorPtr];
	if ( errorPtr )
	{
		NSLog ( @"2010-10-20steve newMovie = %@", newMovie );
		NSLog ( @"2010-10-20steve [initToWritableFile:newMovie] returned error: %@", errorPtr );
		return;
	}
	[newMovie setAttribute:[NSNumber numberWithBool:YES] forKey:QTMovieEditableAttribute];
	NSLog ( @"2010-10-20steve %@ has attributes: %@",[newMovie attributeForKey:QTMovieFileNameAttribute], [newMovie movieAttributes] );
	
	
	NSString* movieBegPath = cSantaMovieStart;	// 2010-11-01steve @"/movieBeg.mov";
	NSString* movieEndPath = cSantaMovieEnd;	// 2010-11-01steve @"/movieEnd.mov";
	errorPtr = nil;
	QTMovie* movie1 = [QTMovie movieWithFile:movieBegPath error:&errorPtr];
	if ( errorPtr )
	{
		NSLog ( @"2010-10-20steve movie1 = %@", movie1 );
		NSLog ( @"2010-10-20steve [movieWithFile:movieBegPath] returned error: %@", errorPtr );
		
	}
	if ( movie1 )
	{
		[movie1 setAttribute:[NSNumber numberWithBool:YES] forKey:QTMovieEditableAttribute];
	}

	errorPtr = nil;
	QTMovie* movie2 = [QTMovie movieWithFile:movieEndPath error:&errorPtr];
	if ( errorPtr )
	{
		NSLog ( @"2010-10-20steve movie2 = %@", movie2 );
		NSLog ( @"2010-10-20steve [movieWithFile:movieEndPath] returned error: %@", errorPtr );
	}
	
	errorPtr = nil;
	QTMovie* movieMiddle = [QTMovie movieWithFile:middleMoviePath error:&errorPtr ];
	if ( errorPtr )
	{
		NSLog ( @"2010-10-25steve movieMiddle = %@", movieMiddle );
		NSLog ( @"2010-10-25steve [movieWithFile:middleMoviePath] returned error: %@", errorPtr );
	}
	
	
	NSLog ( @"2010-10-21steve 2222222222222222222222222222222222222222222222222222222222222222222222222222222222" );
	
	if (!( movie1 && movie2 ))
	{
		return;
	}
	
	
	NSLog ( @"2010-10-21steve 3333333333333333333333333333333333333333333333333333333333333333333333333333333333" );
	
	QTTime timeZero = QTMakeTime(0,600);
	QTTimeRange selection1 = QTMakeTimeRange(timeZero, [movie1 duration]);
	QTTimeRange selection2 = QTMakeTimeRange(timeZero, [movie2 duration]);
	QTTimeRange selectionM = QTMakeTimeRange(timeZero, [movieMiddle duration]);
	
	[movie1 setSelection:selection1];
	[newMovie appendSelectionFromMovie:movie1];
	[newMovie updateMovieFile];
	
	if ( movieMiddle ) // 2010-10-25steve
	{
		NSLog ( @"2010-10-25steve - TRYING to Merge Middle Movie..." );
		[movieMiddle setSelection:selectionM];
		[newMovie appendSelectionFromMovie:movieMiddle];
		[newMovie updateMovieFile];
	}
	else
	{
		NSLog ( @"2010-10-25steve there is NO Middle MOvie to Merge????" );
	}
	
	
	[movie2 setSelection:selection2];
	[newMovie appendSelectionFromMovie:movie2];
	[newMovie updateMovieFile];
	
	//	if ( doSaveInThread==false )
	//	{
	//		[mProgressIndicator stopAnimation:nil];
	//		NSLog ( @"2010-10-21steve pppppppppppppp      %@ Done - NOT doing in thread      pppppppppppppp", NSStringFromSelector(_cmd) );
	//		return;
	//	}
	
	NSDictionary* dict = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:doFlatten]  // 2010-10-25steve
													 forKey:QTMovieFlatten];
	
	{
		NSFileManager* mgr = [NSFileManager defaultManager];
		BOOL isDirectory = NO;
		BOOL exists = [mgr fileExistsAtPath:outputMoviePath isDirectory:&isDirectory];
		if ( exists && !isDirectory )
		{
			[mgr removeItemAtPath:outputMoviePath error:&errorPtr];
			if ( errorPtr)
				NSLog ( @"2010-10-20steve ");
			
		}
		
		self.mMergedMovieItem = [NSString stringWithString:outputMoviePath]; // 2010-10-25steve
		
		bool doAsThread = true;
		if ( doAsThread )
		{
			NSLog ( @"2010-10-21steve yyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyy" );
			NSDictionary* param = [NSDictionary dictionaryWithObjectsAndKeys:newMovie,@"sourceMovie",outputMoviePath,@"outputMoviePath", dict,@"attributes",  nil];
			NSThread* myThread = [[NSThread alloc] initWithTarget:self selector:@selector(myThread1:) object:param];
			
			NSMutableDictionary* threadDictionary = [myThread threadDictionary];
			[threadDictionary setObject:startDate forKey:@"startDateKey"]; // 2010-11-01
			[threadDictionary setObject:@"something to specific about this thread" forKey:@"key1"];
			[threadDictionary setObject:@"something to specific about this thread222" forKey:@"key2"];
			[threadDictionary setObject:myThread forKey:@"senderThread"]; // 2010-10-025steve
			
			[myThread setName:@"myThread1"];
			
			[myThread start];
			
		}
		else
		{
			NSLog ( @"2010-10-21steve zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz" );
			errorPtr = nil;
			[newMovie writeToFile:outputMoviePath withAttributes:dict error:&errorPtr];
			if( errorPtr )
			{
				NSLog ( @"2010-10-20steve newMovie writeToFile has error: %@", errorPtr );
			}
		}
		
		
	}
	
	
	NSLog ( @"2010-10-21steve pppppppppppppp      %@ DONE      pppppppppppppp", NSStringFromSelector(_cmd) );
	
}



-(void) myThread2:(id)sender
{
	NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
	NSLog ( @"2010-10-21steve pppppppppppppp      %@ STARTED          pppppppppppppp", NSStringFromSelector(_cmd) );
	NSLog ( @"2010-10-21steve pppppppppppppp      %@ STARTED          pppppppppppppp", NSStringFromSelector(_cmd) );
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	
	
	NSString* outputMoviePath = [sender objectForKey:@"outputMoviePath"];
	QTMovie*  newMovie = [sender objectForKey:@"sourceMovie"];
	NSDictionary*  attributes = [sender objectForKey:@"attributes"];
	
	NSError* errorPtr = nil;
	[newMovie writeToFile:outputMoviePath withAttributes:attributes error:&errorPtr];
	

	NSLog ( @"2010-10-21steve pppppppppppppp      %@ ENDED          pppppppppppppp", NSStringFromSelector(_cmd) );
	NSLog ( @"2010-10-21steve pppppppppppppp      %@ ENDED          pppppppppppppp", NSStringFromSelector(_cmd) );
	
	NSLog (@ "Inside %@ => %@... DONE...", self, NSStringFromSelector(_cmd) );
	[pool release];
	//	NSLog (@ "Inside %@ => %@... DONE pool released...", self, NSStringFromSelector(_cmd) );
}

-(void) myThread1:(id)sender
{
	NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
	NSLog ( @"2010-10-21steve pppppppppppppp      %@ STARTED          pppppppppppppp", NSStringFromSelector(_cmd) );
	NSLog ( @"2010-10-21steve pppppppppppppp      %@ STARTED          pppppppppppppp", NSStringFromSelector(_cmd) );
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	
	
	NSString* outputMoviePath = [sender objectForKey:@"outputMoviePath"];
	QTMovie*  newMovie = [sender objectForKey:@"sourceMovie"];
	NSDictionary*  attributes = [sender objectForKey:@"attributes"];
	//BOOL doSaveAsReference = [[sender objectForKey:@"doSaveAsReference"] boolValue]; // 2010-10-25steve
	
	NSError* errorPtr = nil;
	[newMovie writeToFile:outputMoviePath withAttributes:attributes error:&errorPtr];
	
	
	NSLog ( @"2010-10-21steve pppppppppppppp      %@ ENDED          pppppppppppppp", NSStringFromSelector(_cmd) );
	NSLog ( @"2010-10-21steve pppppppppppppp      %@ ENDED          pppppppppppppp", NSStringFromSelector(_cmd) );
	
	NSLog (@ "Inside %@ => %@... DONE...", self, NSStringFromSelector(_cmd) );
	[pool release];
	//	NSLog (@ "Inside %@ => %@... DONE pool released...", self, NSStringFromSelector(_cmd) );
}

- (void)handleNSThreadWillExitNotification:(NSNotification *)notification
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	NSLog (@ "Inside %@ => %@...notification: %@", self, NSStringFromSelector(_cmd), notification );
	
	NSLog ( @"2010-10-21steve pppppppppppppp      %@      pppppppppppppp", NSStringFromSelector(_cmd) );
	NSLog ( @"2010-10-21steve pppppppppppppp      %@      pppppppppppppp", NSStringFromSelector(_cmd) );
	NSLog ( @"2010-10-21steve thread dictionary: %@", [[notification object] threadDictionary] );
	NSLog ( @"2010-10-21steve thread name: %@", [[notification object] name] );

	
	[mProgressIndicator performSelectorOnMainThread:@selector(stopAnimation:) withObject:nil waitUntilDone:NO];


	// 2010-10-25steve
	NSNotificationCenter* nc = [NSNotificationCenter defaultCenter];
	[nc removeObserver:self
				  name:NSThreadWillExitNotification
				object:nil];
	
	if ( [[notification object] name] ) // 2010-10-29steve make sure it is NOT nil
		[[NSWorkspace sharedWorkspace] openURL:[NSURL fileURLWithPath:self.mMergedMovieItem]];
	
	//NSNumber * isQTCallBack = [[[notification object] threadDictionary] objectForKey:@"com.apple.qtkit.__QTCaptureMovieFileOutputIsCallbackThread"];
	
	id startDate = [[[notification object] threadDictionary] objectForKey:@"startDateKey"];
	if ( startDate )
	{
		NSTimeInterval lapseTime = [[NSDate date] timeIntervalSinceDate:startDate];
		NSDateFormatter *f = [[[NSDateFormatter alloc] init] autorelease];
		[f setFormatterBehavior:NSDateFormatterBehavior10_4];
		[f setDateFormat:@"yyyy_MM_dd_HH_mm_ss.SSS"];
		
		NSLog (@"2010-11-02steve the startDate is: %@", [f stringFromDate:startDate] );
		NSLog (@"2010-11-02steve Operation Took: %2.3f", lapseTime );

	}
	
	
}

-(void) doTestQTKit
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	
	
	
	[self testMergeMovies1];
	//	[self testMergeMovies2];
	
	
	// [NSApp terminate:self];
	
	NSLog ( @"2010-10-21steve pppppppppppppp      doTestQTKit  doTestQTKit  doTestQTKit      pppppppppppppp" );
	NSLog ( @"2010-10-21steve pppppppppppppp      doTestQTKit  doTestQTKit  doTestQTKit      pppppppppppppp" );
	
	
}


#pragma mark -

- (void)openPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo: (void*)contextInfo
{
	NSLog (@ "Inside %@ => %@... with returnCode=[%i] and ContextInfo=[%@]", self, NSStringFromSelector(_cmd), returnCode, contextInfo );
	
	
	if (returnCode == NSOKButton)
	{
		NSLog (@"openPanelDidEnd:returnCode:contextInfo returned NSOKButton" );
		
		NSString* itemName = [[sheet filenames] lastObject];
		
		if ( contextInfo )
		{
			
			id parameters = (id) contextInfo;
			if ( [parameters isKindOfClass:[NSDictionary class]] )
			{
				// great 
				
				NSString* folderPathKey = [parameters objectForKey:sPrefFolderNameKey];
				NSString* folderPathIsValidKey = [parameters objectForKey:sPrefFolderNameIsValidKey];
				NSString* folderAliasDataKey = [parameters objectForKey:sPrefFolderAliasDataKey];
				
				
				if ( itemName )
				{
					[[NSUserDefaults standardUserDefaults] setObject:itemName forKey:folderPathKey];
					[[NSUserDefaults standardUserDefaults] setBool:YES forKey:folderPathIsValidKey ];
					[MyAliasKit savePath:itemName forKey:folderAliasDataKey];
				}
			}
			
			else
			{
				NSLog ( @"2010-11-04steve - we dont know what to do????" );
				
			}
			
			
			
			
			
		}
		
		else
		{
			NSLog ( @"openPanelDidEnd passed NIL for contextInfo??????" );
		}
	}
	else if (returnCode == NSCancelButton)
	{
		NSLog (@"openPanelDidEnd:returnCode:contextInfo returned NSCancelButton" );
	}
	else
	{
		NSLog (@"openPanelDidEnd:returnCode:contextInfo returned ?????" );
	}
	
	
	
}

- (IBAction) doNSOpenPanelWithParameters:(NSDictionary*)parameters
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	NSLog (@ "Inside %@ => %@... parameters = %@", self, NSStringFromSelector(_cmd), parameters );
	
	NSString* prefFolderNameKey =[parameters objectForKey:sPrefFolderNameKey];
	NSString* panelDelegate = [parameters objectForKey:sPanelDelegateKey];
	NSString* panelMessage = [parameters objectForKey:sPanelMessageKey];
	NSString* panelPrompt = [parameters objectForKey:sPanelPromptKey];
	id panelSender = [parameters objectForKey:sPanelSenderKey];
	
	
	NSWindow* parentWindow = [panelSender window];
	NSOpenPanel* op = [NSOpenPanel openPanel];
	
	[op setCanChooseDirectories:YES];
	[op setCanChooseFiles:NO];
	[op setAllowsMultipleSelection:NO];
	[op setCanCreateDirectories:YES];
	
	[op setMessage:[NSString stringWithString:panelMessage]];
	[op setPrompt:[NSString stringWithString:panelPrompt]];
	
	
	NSString* initialDirectory = [[NSUserDefaults standardUserDefaults] stringForKey:prefFolderNameKey];
	
	bool doAsSheet = true;	// 08-20-07 put back
	if ( doAsSheet )
	{
		[op setDelegate:panelDelegate];
		
		[op beginSheetForDirectory:initialDirectory
							  file:nil
							 types:nil
					modalForWindow:parentWindow
					 modalDelegate:panelDelegate
					didEndSelector:@selector(openPanelDidEnd:returnCode:contextInfo:)
					   contextInfo:[parameters retain]];
		
	}
	
	
}

#pragma mark -

- (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem // 2014-09-04steve
{

    NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
    if([[tabViewItem identifier] caseInsensitiveCompare:@"2"]==NSOrderedSame ) {
        [self.window makeFirstResponder:nil]; // 2014-09-04steve

    }
    
}



#pragma mark -


-(IBAction) ibClickedOnResetDateToDefault:(id)sender
{
    setToFactoryDateFormatString (); // 2014-09-04
}


-(IBAction) ibClickedOnChangeOriginalVideosFolder:(id)sender
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	
	NSDictionary* parameters = [NSDictionary dictionaryWithObjectsAndKeys:self,sPanelDelegateKey, 
								@"Please Select the MASTER ORIGINAL Videos Folder", sPanelMessageKey, 
								@"Select", sPanelPromptKey,
								sender, sPanelSenderKey,
								sPrefOriginalVideosFolder, sPrefFolderNameKey,
								sPrefOriginalVideosFolderAliasData, sPrefFolderAliasDataKey,
								sPrefOriginalVideosFolderIsValid, sPrefFolderNameIsValidKey,
								nil];
	
	[self doNSOpenPanelWithParameters:parameters];
	
}

-(BOOL) showTabForError:(BOOL) isValid
{
	if ( isValid )
	{
		
	}
	else
	{
		[_myNSTabView selectTabViewItemAtIndex:1];
	}
	return YES;
}

/* 2010-11-12steve rewritten as c
-(BOOL) validateFolderPrefWithParam:(NSDictionary*)parameters
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	
	
	BOOL isValid = NO;
	
	NSString* folderPathKey = [parameters objectForKey:sPrefFolderNameKey];
	NSString* folderPathIsValidKey = [parameters objectForKey:sPrefFolderNameIsValidKey];
	NSString* folderAliasDataKey = [parameters objectForKey:sPrefFolderAliasDataKey];
	
	
	NSString* path1 = [[NSUserDefaults standardUserDefaults] objectForKey:folderPathKey];
	id aliasData1 = [[NSUserDefaults standardUserDefaults] objectForKey:folderAliasDataKey];
	
	if ( path1 )
	{
		BOOL isDirectory = NO;
		BOOL exists = [[NSFileManager defaultManager] fileExistsAtPath:path1 isDirectory:&isDirectory];
		if ( exists && isDirectory )
		{
			// great
			isValid = YES;
		}
		else
		{
			NSString* resolvedPath = nil;
			BOOL didResolve = [MyAliasKit resolveAliasFromData:aliasData1 resolvedPath:&resolvedPath];
			if ( didResolve==false )
			{
				[[NSUserDefaults standardUserDefaults] setBool:NO forKey:folderPathIsValidKey];
				// need to openPref Tab
			}
			else
			{
				[[NSUserDefaults standardUserDefaults] setBool:YES forKey:folderPathIsValidKey];
				[[NSUserDefaults standardUserDefaults] setObject:resolvedPath forKey:folderPathKey];
				isValid = NO;
			}
		}
		
	}
	
	return isValid;
	
}
*/

-(BOOL) validateAllFolders
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );
	
	
	NSDictionary* parameters = nil;
	
	parameters = [NSDictionary dictionaryWithObjectsAndKeys:
				  sPrefOriginalVideosFolder, sPrefFolderNameKey,
				  sPrefOriginalVideosFolderAliasData, sPrefFolderAliasDataKey,
				  sPrefOriginalVideosFolderIsValid, sPrefFolderNameIsValidKey,
				  nil];
	BOOL validate1 = validateFolderPrefWithParam (parameters ); // 2010-11-12steve [self validateFolderPrefWithParam:parameters];
	
	NSLog ( @"2010-11-03steve validate1 = %@", validate1?@"YES":@"NO" );
	
	BOOL allValid = (validate1 );
	if ( allValid )
	{
		// great - all found
	}
	else
	{
		// need to open in Pref Tab
		NSRunAlertPanel (	@"Alert",
												 @"Please Point To Valid Folders...",
												 @"OK",	// default
												 nil,	// alternate
												 nil,		// NSAlertOtherReturn
												 @"param1",
												 @"param2"
												 );
		
	}
	
	return allValid;
}


#pragma mark -

//- (void) prepTextField:(NSTextField *)field withString:(NSString *)string
//{
//#define kMaxFontSize 144.0f;
//#define kMinFontSize 6.0f
//    float fontSize = kMaxFontSize;
//    NSFont curfont = field.font;
//
//    while (([string widthForHeight:[field frame].size.height font:[NSFont systemFontOfSize:fontSize]] > [field frame].size.width) && (fontSize > kMinFontSize))
//    {
//        fontSize--;
//    }
//    [field setFont:[NSFont systemFontOfSize:fontSize]];
//
//    [field setStringValue:string];
//    
//}

- (void)windowDidResize:(NSNotification *)notification
{
    NSLog (@ "Inside %@ => %@... notification = %@", self, NSStringFromSelector(_cmd),notification );

}



#pragma mark -

-(IBAction) doButton4:(id)sender // 2014-08-09steve
{
    NSLog(@"%@-%@...", self, NSStringFromSelector(_cmd));
    
    [ibAlertPanel orderOut:nil];
}

-(IBAction) doButton5:(id)sender // 2014-08-09steve
{
    NSLog(@"%@-%@...", self, NSStringFromSelector(_cmd));
    
    [ibAlertPanel orderOut:nil];
    
    [self toggleRecording:nil];
    
}

-(void) setAlertTextField2:(NSString*)infoString doWait:(BOOL)doWait isError:(BOOL)isError // 2014-08-09steve
{
    [ibAlertTextField2 ibUpdateNSTextFieldWithString:infoString
                                              doWait:doWait
                                             isError:isError]; // we want red color
    
}



// need to make sure to INVALIDATE timer if user did start recording
-(void)warningTimerTrippered:(NSTimer*)aTimer
{
    NSLog(@"2014-07-14steve %@... REMIND USER TO TAKE VIDEO....", NSStringFromSelector(_cmd));
    
    
    
    // 2014-08-09teve [self bringToFront:nil];
    
    
    NSDictionary* param = [aTimer userInfo];

    [aTimer invalidate]; // 2014-07-22steve
    if ( aTimer==self.fWarningTimer1) {
        self.fWarningTimer1 = nil; // 2014-07-23steve
    }
    
    
    // 2014-08-09steve
    NSString* currentid = @"<unknown current id>";
    id temp = [param objectForKey:@"currentid"];
    if (temp)
        currentid = [NSString stringWithFormat:@"%@",temp];

    
    bool doAlert = NO; // 2014-08-09steve moved up
    if (doAlert)
    {
        [self bringToFront:nil]; // 2014-08-09steve
        
//        NSString* currentid = @"<unknown current id>";
//        id temp = [param objectForKey:@"currentid"];
//        if (temp)
//            currentid = [NSString stringWithFormat:@"%@",temp];
        
        NSAlert * alert = [NSAlert alertWithMessageText:@"!!!!!  REMINDER !!!!!!"
                                          defaultButton:@"Start Recording"
                                        alternateButton:@"Close"
                                            otherButton:nil
                              informativeTextWithFormat:[NSString stringWithFormat:@"Dont Forget To Start Movie Recording\nFor Customer ID = [%@]",currentid]];
        
        [[NSApplication sharedApplication] activateIgnoringOtherApps: YES];

        NSInteger returnValue = [alert runModal];
        
        switch (returnValue) {
            case NSAlertDefaultReturn:
                // start the video recording
                [self startRecording:nil];
                break;
                
            default:
                break;
        }
        
        
    }

    else { // 2014-08-09steve
        
        BOOL bringAppToFront =YES; // 2014-09-25steve - peter wants to bring app to front instead of this warning
        if ( bringAppToFront) {
            
            [self bringToFront:nil];
            
        } else {
            
            if (ibAlertPanel) {
                [self setAlertTextField2:[NSString stringWithFormat:@"For Customer ID = [%@]",currentid]
                                  doWait:NO
                                 isError:NO];
                [ibAlertPanel orderFront:nil];
            }
            
        }
        
        
    }

    
}

-(void) setupWarningTimer1
{
    NSLog(@"2014-07-22steve Inside %@-%@......",self, NSStringFromSelector(_cmd));

    [self.fWarningTimer1 invalidate];
    self.fWarningTimer1 = nil;
    
    BOOL doWarnWithTimer = YES;
    if (doWarnWithTimer) {
        NSTimeInterval delay = 0.1; // 2014-08-09steve
        
        NSMutableDictionary* param = [NSMutableDictionary dictionaryWithCapacity:0];
        [param setObject:self.fCurrentTepsCustomerID forKey:@"currentid"];
        self.fWarningTimer1 = [NSTimer timerWithTimeInterval:delay  // 2014-08-09steve 5.0
                                                  target:self
                                                selector:@selector(warningTimerTrippered:)
                                                userInfo:param
                                                 repeats:NO];
        [[NSRunLoop mainRunLoop] addTimer:self.fWarningTimer1
                                  forMode:NSDefaultRunLoopMode];
    }

}



// send


-(void) sendRecordingStatusNotification // 2014-08-08steve
{
    
    BOOL interfaceWithTeps = [[NSUserDefaults standardUserDefaults] boolForKey:@"prefInterfaceWithTeps"];
    if (interfaceWithTeps) {

        NSString* status = @"statusUnknown";
        
        if (_mIsRecording) {
            status = @"statusIsRecording";
        }else {
            status = @"statusNotRecording";
        }
        
        // send notification to teps...
        NSDictionary* param2 = [NSDictionary dictionaryWithObjectsAndKeys: @"Teps-Video.app",@"senderApp",
                                [NSString stringWithFormat:@"%@",status],@"action",
                                nil];
        NSNotification* note = [NSNotification notificationWithName:sTepsXNotification // 2014-08-08steve
                                                             object:nil	// must be NSString ONLY
                                                           userInfo:param2];
        
        [[NSDistributedNotificationCenter defaultCenter] performSelectorOnMainThread:@selector(postNotification:)
                                                                          withObject:note
                                                                       waitUntilDone:NO];
        
        NSLog(@"Sending Notification = %@",note);
        
    } else {
        NSLog(@"Not interfacing with Teps... No Notificaiton Sent...");
    }
    
}



-(void) handleTepsVideoNotification:(NSNotification*) note	// 2014-07-14steve
{
	// if (gVerbose)
    NSLog (@ "%@->%@...........................................................................", self, NSStringFromSelector(_cmd) );
    NSLog (@ "%@->%@...with Notif = %@", self, NSStringFromSelector(_cmd), note );
    NSLog (@ "%@->%@...........................................................................", self, NSStringFromSelector(_cmd) );
    
	id param = [note userInfo];
	id action = [param objectForKey:@"action"];
    id senderApp = [param objectForKey:@"senderApp"]; // 2014-08-08steve
	
    
    if (senderApp && [senderApp caseInsensitiveCompare:@"Teps-Video.app"]==NSOrderedSame) { // 2014-08-08steve
        
        NSLog(@"SenderApp is Ourself... So Do Nothing...");
        return;
        
    }
    
	if (action)
	{
        
		if ( [action caseInsensitiveCompare:@"SetCustomerID"]==NSOrderedSame )
		{
            NSBeep();
            
            id customerid = [param objectForKey:@"customerID"];
            id folderpath = [param objectForKey:@"folderPath"];
            
            if ( customerid && folderpath && [customerid isKindOfClass:[NSString class]] && [folderpath isKindOfClass:[NSString class]] )
            {
                self.fCurrentTepsCustomerID = [NSString stringWithFormat:@"%@",customerid];
                self.fCurrentTepsDailyFolderPath =  folderpath;

                // may want to start a timer - to alert user if they do not start the video????
                
                [self setupWarningTimer1]; // 2014-07-15steve
                
                if (self.mVideoStatusWindowController)
                    [self.mVideoStatusWindowController setTextField2:[NSString stringWithFormat:@"CustomerID=[%@]",self.fCurrentTepsCustomerID]
                                                              doWait:NO
                                                           withColor:[NSColor magentaColor]]; // 2014-07-13steve

            }
            
			
		}
        else if ( [action caseInsensitiveCompare:@"GetStatus"]==NSOrderedSame ) // 2014-08-08steve
		{
            [self sendRecordingStatusNotification]; // 2014-08-08steve
        }
        else if ([action caseInsensitiveCompare:@"StopRecording"]==NSOrderedSame )
        { // 2014-08-08steve
            [self toggleRecording:nil]; // 2014-08-08steve
        }
        else if ([action caseInsensitiveCompare:@"StartRecording"]==NSOrderedSame )
        { // 2014-08-08steve
            [self toggleRecording:nil]; // 2014-08-08steve
        }
        
        else if ([action caseInsensitiveCompare:@"statusIsRecording"]==NSOrderedSame ) {
            // 2014-08-08steve do nothing - we sent this out probably
        }
        else if ([action caseInsensitiveCompare:@"statusNotRecording"]==NSOrderedSame ) {
            // 2014-08-08steve  do nothing - we sent this out probably
        }
        else if ([action caseInsensitiveCompare:@"statusMovieThumbnailCreated"]==NSOrderedSame ) {
            // 2014-08-08steve  do nothing - we sent this out probably
        }
        
        
        else if ([action caseInsensitiveCompare:@"burnVideo"]==NSOrderedSame )
        {
            // 2014-07-31steve - You MustSwitch to Sales Station Mode
            NSLog(@"Attempting to ReLaunch App into the Sales Station Mode to handle action=%@...",action);
            NSAlert * alert = [NSAlert alertWithMessageText:@"WARNING - WRONG Mode???"
                                              defaultButton:@"Cancel"
                                            alternateButton:@"Switch To Sales Station Mode"
                                                otherButton:nil
                                  informativeTextWithFormat:@"The action[%@] is Only Handled When Teps-Video.app is in the Sales Station Mode.\n\n",action];
            [self bringToFront:nil];
            NSInteger returnvalue = [alert runModal];
            if (returnvalue==NSAlertAlternateReturn) {
                NSInteger mode = 2; // sales station mode
                [[NSUserDefaults standardUserDefaults] setInteger:mode forKey: sPrefStartupMode];	// 5-8-03 use predefine
                NSLog (@ "Inside %@ => %@...will try to relaunch app for mode = [%i]", self, NSStringFromSelector(_cmd), mode );
                [[NSUserDefaults standardUserDefaults] synchronize];

                [[NSNotificationCenter defaultCenter] postNotificationName:@"AppDelegateNotification"
                                                                    object:nil
                                                                  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:@"relaunchApp",@"action", nil]];
            }
            
            
        }
        else {
            
            bool doAlert = YES; // 2014-07-21steve

            if (doAlert)
            {
                NSAlert* alert = [[NSAlert alloc] init];
                [alert setMessageText:[NSString stringWithFormat:@"Inside handleTepsVideoNotification ..." ]];
                [alert setInformativeText:[NSString stringWithFormat:@"Do NOT How To Handle This Notification:\n\n%@\n\n", note]];
                [alert addButtonWithTitle:@"OK"];
                
                [[NSApplication sharedApplication] activateIgnoringOtherApps: YES];
                [alert runModal];
                
                [alert release];
                alert = nil;
            }

        }
        
	}
	
}

-(void) handleTepsXNotification:(NSNotification*)note
{
    
    NSLog (@ "%@->%@...........................................................................", self, NSStringFromSelector(_cmd) );
    NSLog (@ "%@->%@...with Notif = %@", self, NSStringFromSelector(_cmd), note );
    NSLog (@ "%@->%@...........................................................................", self, NSStringFromSelector(_cmd) );
    
	id param = [note userInfo];
	id action = [param objectForKey:@"action"];
    id senderApp = [param objectForKey:@"senderApp"]; // 2014-08-08steve
	
    
    if (senderApp && [senderApp caseInsensitiveCompare:@"Teps-Video.app"]==NSOrderedSame) { // 2014-08-08steve
        
        NSLog(@"SenderApp is Ourself... So Do Nothing...");
        return;
        
    }

	if (action)
	{
		if ( [action caseInsensitiveCompare:@"newCustomerSittingStarted"]==NSOrderedSame )
		{
            id customerID = [param objectForKey:@"value"];
            NSLog(@"Found New CustomerID = [%@]",customerID);
            
            if ( self.fCurrentTepsCustomerID==nil) {
                self.fCurrentTepsCustomerID = [NSString stringWithFormat:@"%@",customerID];
            }
            
            if (self.fCurrentTepsCustomerID && [self.fCurrentTepsCustomerID caseInsensitiveCompare:customerID]==NSOrderedSame) {
                // they are the same do nothing
            } else {
                // we need to check - if a movie is still recording
                // we need autoStop it
                [self stopRecording:nil];
                
                self.fCurrentTepsCustomerID = [NSString stringWithFormat:@"%@",customerID]; // 2014-07-15steve
                
                [self setupWarningTimer1]; // 2014-07-15steve

            }
            

        }
        
        
    }
    

}


-(void) handleScannedBarcode:(NSNotification*) note	// 2014-11-05steve
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    
    NSLog (@ "Inside %@ => %@...note = %@", [self class], NSStringFromSelector(_cmd), note );
    
    
    NSString* newbarcode = [[note object] objectForKey:@"newBarcode"];
    NSLog (@ "Inside %@ => %@...newbarcode = %@", [self class], NSStringFromSelector(_cmd), newbarcode );

    newbarcode = [newbarcode stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
    
    if ( newbarcode && [newbarcode length]) {
        BOOL doContinue = NO;
        if (_mIsRecording) {
            // another recording is taking place
            NSString* currentcode = [[NSUserDefaults standardUserDefaults] stringForKey:cPrefRenameString];
            if ( currentcode && [currentcode caseInsensitiveCompare:newbarcode]==NSOrderedSame) {
                // scanned the same barcode as current recording video
                NSString* messageText = @"WARNING - Do YOU WANT TO STOP CURRENT RECORDING???";
                NSString* informativestring = @"\n\nPress \"STOP Recording\" OR \"Keep Recording\"\n\n";
                NSAlert * alert = [NSAlert alertWithMessageText:messageText
                                                  defaultButton:@"Keep Recording"
                                                alternateButton:nil
                                                    otherButton:@"STOP Recording"
                                      informativeTextWithFormat:@"%@",informativestring];
                NSInteger response = [alert runModal];
                if ( response == NSAlertDefaultReturn) {
                    doContinue = YES;
                } else {
                    doContinue = NO;
                }
                
                
            } else {
                NSString* messageText = @"WARNING - DO YOU WANT TO EXISTING RECORDING FIRST???";
                NSString* informativestring = @"some informative text";
                NSAlert * alert = [NSAlert alertWithMessageText:messageText
                                                  defaultButton:@"Start NEW Recording Session"
                                                alternateButton:nil
                                                    otherButton:@"Keep Recording"
                                      informativeTextWithFormat:@"%@",informativestring];
                NSInteger response = [alert runModal];
                if ( response == NSAlertOtherReturn) {
                    doContinue = YES;
                } else {
                    doContinue = NO;
                }

            }
        } else {
            doContinue = YES;
        }
        
        if (doContinue) {
            [[NSUserDefaults standardUserDefaults] setObject:[NSString stringWithFormat:@"%@", newbarcode] forKey:cPrefRenameString];
            [self performSelectorOnMainThread:@selector(preStartRecording2:) withObject:[NSString stringWithFormat:@"%@", newbarcode] waitUntilDone:NO];
        }
        
        
    }
    
    
    if ( pool )	[pool release];
    
}



@end
