//
//  TepsExtras.m
//  MyRecorder
//
//  Created by steve on 11/4/10.
//  Copyright 2010 TriPrism, Inc. All rights reserved.
//

#import "TepsExtras.h"
#import "MyAliasKit.h"

#include <sys/stat.h>
#include <pwd.h>
#include <grp.h>

NSString* sPrefFolderNameKey = @"prefFolderNameKey";
NSString* sPrefFolderNameIsValidKey = @"prefFolderNameIsValidKey";
NSString* sPrefFolderAliasDataKey = @"prefFolderAliasDataKey";


extern NSString* computeCurrentDaysFolderString2 ( short hoursToShift, short minsToShift ); // 2011-11-07steve



// latest (new-est) one first
NSInteger mySortByDateDescending(id path1, id path2, void *info)
{
	// compare 
	NSComparisonResult comp = [[path1 objectForKey:@"lastModDate"] compare:
							   [path2 objectForKey:@"lastModDate"]];
	// invert ordering
	if (comp == NSOrderedDescending) {
		comp = NSOrderedAscending;
	}
	else if(comp == NSOrderedAscending){
		comp = NSOrderedDescending;
	}
	return comp;                                
}

// earliest (oldest) one first
NSInteger mySortByDateAscending(id path1, id path2, void *info)
{
	// compare 
	NSComparisonResult comp = [[path2 objectForKey:@"lastModDate"] compare:
							   [path1 objectForKey:@"lastModDate"]];
	// invert ordering
	if (comp == NSOrderedDescending) {
		comp = NSOrderedAscending;
	}
	else if(comp == NSOrderedAscending){
		comp = NSOrderedDescending;
	}
	return comp;                                
}
// earliest (oldest) one first
NSInteger mySortByDate(id path1, id path2, void *info)
{
	return mySortByDateAscending(path1, path2, nil);
}


#pragma mark -

// 2010-04-21steve
// this runs a terminal Shell Script
// must separate the shell command and its parameters
NSString* runThisShellScript ( NSString*scriptName, NSArray*paramArray )
{	
	bool doEchoMesg = true;
	
	if ( scriptName==nil )
	{
		NSLog (@"\n\nrunThisShellScript FAILED => scriptName==nil!\n\n" );
		return nil;
	}
	
	if (doEchoMesg) NSLog ( @"runThisShellScript => scriptName = [%@]", scriptName );
	if ( paramArray )
	{
		if (doEchoMesg) NSLog ( @"runThisShellScript => param:\n%@", paramArray );		
	}
	else
	{
		if (doEchoMesg) NSLog ( @"runThisShellScript => NO param" );		
	}
	
	
	NSString *latexPath = scriptName;
	if ( latexPath==nil )
	{
		NSLog (@"\n\nrunThisShellScript FAILED => UNABLE to find script named: [%@]!\n\n", scriptName );
		return nil;
	}
	
    NSTask *task;
    task = [[NSTask alloc] init];
    [task setLaunchPath: latexPath];
	
    NSArray *arguments = nil;
	
	if ( paramArray )
		arguments = [NSArray arrayWithArray: paramArray];
	
    [task setArguments: arguments];
	
    NSPipe *pipe;
    pipe = [NSPipe pipe];
    [task setStandardOutput: pipe];
	
    NSFileHandle *file;
    file = [pipe fileHandleForReading];
	
    [task launch];
	
	[task waitUntilExit];	// 05-15-08steve
	
	if (![task isRunning])
	{
		int status = [task terminationStatus];
		if (status == 0)
		{
			if (doEchoMesg) NSLog(@"runThisShellScript Task succeeded.");			
		}
		else
		{
			NSLog(@"runThisShellScript Task failed.");
            [task release];
			return [NSString stringWithString:@"Task failed."]; // 2010-11-05
		}
	}
	
    NSData *data = [file readDataToEndOfFile];
	
    NSString *string = [[NSString alloc] initWithData: data encoding: NSUTF8StringEncoding];
    if (doEchoMesg) NSLog (@"runThisShellScript got back:\n%@\n", string);
	
	if ( task ) [task release];	// free memory
	
	return [string autorelease];
	
}

BOOL runShellScript ( NSString*scriptName, NSArray*paramArray )
{
	BOOL success = NO;
	
	if ( scriptName==nil )
	{
		NSLog (@"\n\nrunShellScript FAILED => scriptName==nil!\n\n" );
		return NO;
	}
	
	
	NSBundle *bundle = [NSBundle mainBundle];
	NSString *latexPath = [bundle pathForAuxiliaryExecutable:scriptName];
	if ( latexPath==nil )
	{
		NSLog (@"\n\nrunShellScript FAILED => UNABLE to find script named: [%@]!\n\n", scriptName );
		return NO;
	}
	
	
	NSString* result = runThisShellScript(latexPath, paramArray);
	
	
	return success;
	
}

// keys:
// @"scriptName"
// @"scriptParams"
// @"delegate"
BOOL launchScriptWithParameters ( id parameters )
{
	NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
	[parameters retain];
	
	@try
	{
		if (parameters==nil )
		{
			NSLog( @"2010-11-07steve - launchScriptWithParameters -> parameters is NIL?");
			NSException *e = [NSException exceptionWithName: @"launchScriptWithParameters - Bad Parameter..."
													 reason: @"Parameters is NIL???"
												   userInfo: nil];
			@throw e;
		}
		if (![parameters isKindOfClass:[NSDictionary class]] )
		{
			NSLog( @"2010-11-07steve - launchScriptWithParameters -> parameters is the wrong ClassType = [%@]...", [parameters class]);
			NSException *e = [NSException exceptionWithName: @"launchScriptWithParameters - Bad Parameter..."
													 reason: [NSString stringWithFormat: @"Parameters is the wrong class type (%@)", [parameters class]]
												   userInfo: nil];
			@throw e;
		}
	
		NSString* scriptName = [parameters objectForKey:@"scriptName" ];// @"playMovieInQTPlayer.scpt";
		if ( scriptName==nil )
		{
			NSLog( @"2010-11-07steve - launchScriptWithParameters -> parameters [scriptName] is NOT Found...");
			NSException *e = [NSException exceptionWithName: @"launchScriptWithParameters - Bad Parameter..."
													 reason: [NSString stringWithFormat: @"scriptName parameter not specified."]
												   userInfo: nil];
			@throw e;
		}
		
		id scriptParams = [parameters objectForKey:@"scriptParams"];
		id delegate = [parameters objectForKey:@"delegate"];
		
		
		NSBundle *bundle = [NSBundle mainBundle];
		NSString *scriptPath = [bundle pathForAuxiliaryExecutable:scriptName];
		if ( scriptPath==nil )
		{
			NSLog( @"2010-11-07steve - launchScriptWithParameters -> parameters [scriptName] is NOT Found...");
			NSException *e = [NSException exceptionWithName: @"launchScriptWithParameters - Bad Parameter..."
													 reason: [NSString stringWithFormat: @"scriptName (%@) NOT Found.", scriptName]
												   userInfo: nil];
			@throw e;
		}
		else
		{
			NSLog ( @"2010-11-04steve Found Script named: %@", scriptPath);
		}
		
		
		bool doOsaScript = true;
		if ( doOsaScript )
		{
			NSString *scriptName = @"/usr/bin/osascript";
			
			NSMutableArray *paramArray = [NSMutableArray arrayWithObjects:scriptPath,nil];
			
			if ( scriptParams )
			{
				NSLog ( @"2010-11-07steve scriptParams class at this point is: %@", [scriptParams class] );
				NSLog ( @"2010-11-07steve scriptParams at this point is: %@", scriptParams );
				
				if ([scriptParams isKindOfClass:[NSString class]])
				{
					[paramArray addObject:scriptParams];
				}
				else if ( [scriptParams isKindOfClass:[NSArray class]] )
				{
					short paramCount = [scriptParams count];
					if ( [scriptParams count] )
					{
						NSLog (@"2010-11-07steve scriptParams is an array with: %@", scriptParams );
						short i=0;
						for ( i=0; i<paramCount; i++ )
						{
							[paramArray addObject:[scriptParams objectAtIndex:i]];
						}
					}
					NSLog (@"2010-11-06steve paramArray is: %@", paramArray );
				}
				else
				{
					NSLog ( @"2010-11-07steve UKNOWN CLASS type for the Parameter????" );
				}
			}
			
			
			
			NSString* response2 = runThisShellScript ( scriptName, paramArray );
			NSLog ( @"2010-11-05steve response2 = %@", response2 );
			
			[delegate processResponseFromScript:response2];
			
		}
		
	}
	@catch( NSException* e)
	{
		NSLog (@"2010-11-07steve - NSException found: %@", e );
		// [parameters release];
		// parameters = nil;
		[pool release];
		return NO;
	}
	
	//[parameters release];
	//parameters = nil;
	
	[pool release];
	
}




#pragma mark -

// 2014-08-26steve
void printToCups (NSString* itemPath)
{
    
    NSLog (@ "Inside %s(%@) => %@...",__func__, itemPath );

    BOOL validPath = NO;
	NSString* itemToPrint = itemPath;
	if ( itemToPrint )
	{
		if ( [itemToPrint length] ) {
			itemToPrint = [itemToPrint stringByExpandingTildeInPath]; // 2009-08-12steve expand tilde
			BOOL isDirectory = NO;
			BOOL itemExists = [[NSFileManager defaultManager] fileExistsAtPath:itemToPrint isDirectory:&isDirectory];
			if ( itemExists && !isDirectory )
			{
				validPath = YES;
			}
        }
        
    } else {
        
    }

	if (validPath)
	{
		NSArray* paramArray = [NSArray arrayWithObjects:itemToPrint, nil];
		runShellScript ( @"printToCups.sh", paramArray );
	}
	else
	{
		NSBeep();
		NSLog ( @"User Needs to Select a Valid Image Item to Print..." );
	}
    
    
}

// 2009-08-12steve
void revealItemInFinder (NSString* itemPath)
{
	NSLog (@ "Inside revealItemInFinder...revealItemInFinder => %@...", itemPath );
	
	BOOL validPath = NO;
	NSString* pathToReveal = itemPath;
	if ( pathToReveal )
	{
		if ( [pathToReveal length] )
		{
			pathToReveal = [pathToReveal stringByExpandingTildeInPath]; // 2009-08-12steve expand tilde
			BOOL isDirectory = NO;
			BOOL itemExists = [[NSFileManager defaultManager] fileExistsAtPath:pathToReveal isDirectory:&isDirectory];
			if ( itemExists && isDirectory )
			{
				validPath = YES;
			}
		}
		
	}
	
	if (validPath)
	{
		NSArray* paramArray = [NSArray arrayWithObjects:pathToReveal, nil];
		runShellScript ( @"revealInFinder.sh", paramArray );
	}
	else
	{
		NSBeep();
		NSLog ( @"User Needs to Select a Valid Existing Folder..." );
	}
	
}


void moveFileToUserTrash (NSString *filePath)
{
    CFURLRef        trashURL = NULL;
    FSRef           trashFolderRef;
    CFStringRef     trashPath = NULL;
    OSErr           err = noErr;
    NSFileManager   *mgr = [NSFileManager defaultManager];
	
	// 2010-10-16steve make sure item is allowed to be deleted
	if ( NO==[mgr isDeletableFileAtPath:[filePath stringByStandardizingPath]] )
	{
		NSLog ( @"moveFileToUserTrash => item [%@] is NOT deletable..." );
		return;
	}
	
	
	
	
    err = FSFindFolder(kUserDomain, kTrashFolderType, kDontCreateFolder, &trashFolderRef);
	
    if (err == noErr)
	{
		trashURL = CFURLCreateFromFSRef(kCFAllocatorSystemDefault, &trashFolderRef);
		
		if (trashURL)
		{
			trashPath = CFURLCopyFileSystemPath (trashURL, kCFURLPOSIXPathStyle);
			
			NSString *destTrashFilePath = [(NSString *)trashPath stringByAppendingPathComponent:[filePath lastPathComponent]];
			if ( [mgr fileExistsAtPath:destTrashFilePath] )
			{
				// NSLog ( @"Trash Already has a file at [%@]", destTrashFilePath);
				NSDate *today = [NSDate date];	// get current date/time
				NSCalendarDate* calendarDate;
				calendarDate = [ today dateWithCalendarFormat: @" %Y%m%d_%H%M%S" timeZone:nil];
				NSString* formattedDate = [ calendarDate description ];
				destTrashFilePath = [destTrashFilePath stringByAppendingString:formattedDate];
				// NSLog ( @"Creating Unique Name to Trash at [%@]", destTrashFilePath);
			}
			
			// 08-17-07
			// if (![mgr movePath:filePath toPath:[(NSString *)trashPath stringByAppendingPathComponent:[filePath lastPathComponent]] handler:nil])
			if (![mgr movePath:filePath toPath:destTrashFilePath handler:nil])
			{
				NSLog(@"moveFileToUserTrash -> Move operation did not succeed! for [%@]", destTrashFilePath );
			}
			
        }
		
        if (trashPath)	CFRelease(trashPath);
        if (trashURL)	CFRelease(trashURL);
		
    }
	
}


#pragma mark -


bool setPermission ( NSString* objcPath, UInt32 permMode )
{
	const char* cPath = [objcPath cStringUsingEncoding:NSMacOSRomanStringEncoding];
	return setPermissionInC (cPath, permMode);
}

// permMode = OCTAL value
// result = setPermission ( itempPath, \777); NOTE: not \x777 nor 0x777
// true = successful in setting itemPath to permMode
// false = failed
// DOES NOT Create the itemPath does 
bool setPermissionInC ( const char *itemPath, UInt32 permMode )
{
	bool doEcho = false;	// 2009-08-19steve true;
	bool success = false;
	if (doEcho) printf ( "\n" );
	if (doEcho) printf ( "setPermission => itemPath to evaluate is [%s]\n", itemPath );
	
	uid_t prevMode = 0;
	struct stat statbuf;
	if ( 0==stat(itemPath, &statbuf) )
	{
		if (doEcho) printf ( "setPermission => Success in getting statbuf info for [%s]\n", itemPath );
		
		prevMode = statbuf.st_mode;
		
		uid_t newMode = ( (prevMode) & (~(0777)) );
		
		if (doEcho ) printf ( "prevMode = [0x%04x hex] = [%04o octal]\n", prevMode, prevMode );
		if (doEcho ) printf ( "newFolderMode  = [0x%04x hex] = [%04o octal]\n", newMode, newMode );
		newMode |= ( (permMode) & (0777) );
		if (doEcho ) printf ( "newFolderMode  = [0x%04x hex] = [%04o octal]\n", newMode, newMode );
		
		// chmod
		int result = chmod ( itemPath, newMode );
		if ( result !=0 )
		{
			printf ( "setPermission => chmod newFolderMode FAILED with errorCode = [%i] = [0x%04x]", result, result );
			success = false;
		}
		else
		{
			success = true;
			if (doEcho ) printf ("setPermission => SUCCESS IN chmod!!!\n" );
		}
	}
	else
	{
		printf ("\nsetPermission => Unable to get info on itemPath = [%s]!!!\n", itemPath );
	}
	
	return success;
}


BOOL createDirectoryIfMissing (NSString* toBeCreatedPath)
{
	NSLog (@ "Inside createDirectoryIfMissing -> withPath=[%@]", toBeCreatedPath );
	
	BOOL doesExists = NO;
	unsigned long permMode = 0777; // OCTAL

	NSFileManager* mgr = [NSFileManager defaultManager];
	BOOL isDirectory = NO;
	BOOL exists = [mgr fileExistsAtPath:toBeCreatedPath isDirectory:&isDirectory];
	if ( exists && isDirectory )
	{
		// just set the permission
		setPermission(toBeCreatedPath, permMode);
		
		doesExists = YES;
	}
	else
	{
		NSError* error = nil;
		// NSLog (@ "2010-11-04steve posixPermission is: %o = %i", [[NSNumber numberWithLong: permMode] longValue] );
		NSDictionary* attributes = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithLong: permMode], NSFilePosixPermissions,nil];
		BOOL didCreate = [mgr createDirectoryAtPath:toBeCreatedPath 
						withIntermediateDirectories:YES 
										 attributes:attributes
											  error:&error];
		if ( error )
			NSLog ( @"2010-11-04steve doCreateDirectoryIfMissing error returned: %@", error );
		
		if ( didCreate && !error )
			doesExists = YES;
	}
	
	return doesExists;
}



//NSString* folderPathKey = [parameters objectForKey:sPrefFolderNameKey];
//NSString* folderPathIsValidKey = [parameters objectForKey:sPrefFolderNameIsValidKey];
//NSString* folderAliasDataKey = [parameters objectForKey:sPrefFolderAliasDataKey];
BOOL validateFolderPrefWithParam ( NSDictionary* parameters)
{
	NSLog (@ "Inside validateFolderPrefWithParam withParameter = %@", parameters );
	
	
	BOOL isValid = NO;
	
	NSString* folderPathKey = [parameters objectForKey:sPrefFolderNameKey];
	NSString* folderPathIsValidKey = [parameters objectForKey:sPrefFolderNameIsValidKey];
	NSString* folderAliasDataKey = [parameters objectForKey:sPrefFolderAliasDataKey];
	
	
	NSString* path1 = [[NSUserDefaults standardUserDefaults] objectForKey:folderPathKey];
	id aliasData1 = [[NSUserDefaults standardUserDefaults] objectForKey:folderAliasDataKey];
	
    if (aliasData1==nil ) { // 2013-08-09steve
        // this is the prob the first time
        createDirectoryIfMissing(path1);
        BOOL isDirectory = NO;
		BOOL exists = [[NSFileManager defaultManager] fileExistsAtPath:path1 isDirectory:&isDirectory];
		if ( exists && isDirectory )
		{
			// great
			isValid = YES;
            [[NSUserDefaults standardUserDefaults] setObject:path1 forKey:folderPathKey];
            [[NSUserDefaults standardUserDefaults] setBool:YES forKey:folderPathIsValidKey ];
            [MyAliasKit savePath:path1 forKey:folderAliasDataKey];
		}
        
    }
    
	if ( path1 )
	{
		BOOL isDirectory = NO;
		BOOL exists = [[NSFileManager defaultManager] fileExistsAtPath:path1 isDirectory:&isDirectory];
		if ( exists && isDirectory )
		{
			// great
			isValid = YES;
		}
		else
		{
			NSString* resolvedPath = nil;
			BOOL didResolve = [MyAliasKit resolveAliasFromData:aliasData1 resolvedPath:&resolvedPath];
			if ( didResolve==false )
			{
				[[NSUserDefaults standardUserDefaults] setBool:NO forKey:folderPathIsValidKey];
				// need to openPref Tab
			}
			else
			{
				[[NSUserDefaults standardUserDefaults] setBool:YES forKey:folderPathIsValidKey];
				[[NSUserDefaults standardUserDefaults] setObject:resolvedPath forKey:folderPathKey];
				isValid = YES; // 2010-11-12steve
			}
		}
		
	}
	
	return isValid;
	
}



#pragma mark -

NSString* setToFactoryDateFormatString (void) // 2014-09-04
{
    [[NSUserDefaults standardUserDefaults] setObject:[NSString stringWithFormat:@"yyyyMMdd"] forKey:@"prefDailyDateFormat"];
    [[NSUserDefaults standardUserDefaults] synchronize];
    return [[NSUserDefaults standardUserDefaults] objectForKey:@"prefDailyDateFormat"];
    
}


// pass positive hoursToShift to shift back
NSString* computeCurrentDaysFolderString2 ( short hoursToShift, short minsToShift )
{
    
	long oneHourInSecs = 60*60;
	long oneMinInSecs = 60;
	long amHour = hoursToShift;		// 13;
	long amMinutes = minsToShift;	// 57;
	long amTimeInSeconds =  (amHour*oneHourInSecs) +  (amMinutes*oneMinInSecs);
	
	NSDate* hoursAgoDate = [NSDate dateWithTimeIntervalSinceNow:(-1*amTimeInSeconds)];
	//NSLog ( @"Shift Time By (HH::MM) = %02i:%02i => [%@]", amHour, amMinutes, hoursAgoDate );
	NSString *newString = nil;
	{
		NSDateFormatter *outputFormatter = [[NSDateFormatter alloc] init];
		[outputFormatter setFormatterBehavior:NSDateFormatterBehavior10_4];
		
		[outputFormatter setDateFormat:@"yyyy-MM-dd"]; // 2010-11-11steve changed to dash as separators
        
        NSString* dateFormatString = [[NSUserDefaults standardUserDefaults] stringForKey:@"prefDailyDateFormat"]; // 2014-09-04
        if (dateFormatString) {
            dateFormatString = [dateFormatString stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
            if ([dateFormatString length]) {
                [outputFormatter setDateFormat:dateFormatString]; // 2014-09-04steve
            }
        }
        
        
        
        
        
		
		// [outputFormatter setDateStyle:NSDateFormatterFullStyle]; // Use Constants from System Prefs
		// NSLog ( @"2010-10-14steve The Newly Shifted Date String is: %@", [outputFormatter stringFromDate:hoursAgoDate] );
		
		newString = [NSString stringWithString:[outputFormatter stringFromDate:hoursAgoDate]];
		
        newString = [[NSString alloc] initWithFormat:@"%@",[outputFormatter stringFromDate:hoursAgoDate]]; // 2014-09-23steve
        
		// NSLog ( @"2010-10-14steve newString is %@", newString );
		// NSLog ( @"2010-10-14steve newString is %i", [newString length] );
		
		// success = [newString getCString:buffer maxLength:[newString length]+1 encoding:NSMacOSRomanStringEncoding];
		// NSLog ( @"2010-10-14steve didGet is %@", didGet?@"True":@"False" );
		// NSLog ( @"2010-10-14steve as buffer => %s", buffer );
		
		[outputFormatter release];
	}
	
    
	return [newString autorelease]; // 2014-09-23steve need to return as autorelease nsstring
	// return success;
}

// this function is to make sure that there is NO other item that already exists in the new path???
NSString* createSafeFilePathFromBase ( NSString *creationPath )
{
	NSLog (@ "Inside createSafeFilePathFromBase for %@...", creationPath );
	
	const cMaxLoop = 1000; // 2011-12-05steve
	int numberWithName = 1;
	BOOL isDir;
	NSString *safePath;
	safePath = [creationPath retain];
	
	// if the item already exist = we want to rename it
	while ([[NSFileManager defaultManager] fileExistsAtPath:safePath
												isDirectory:&isDir])
    {
        [safePath release];
        safePath = [[NSString alloc] initWithFormat:@"%@ %d.%@",
					[creationPath stringByDeletingPathExtension],
					numberWithName,
					[creationPath pathExtension]];
        
		NSLog (@"2011-12-05steve %s - new safePathLoop is: %@", __func__,safePath ); // 2011-12-05steve
		
        numberWithName++;
		
		if ( numberWithName > cMaxLoop ) // 2011-12-05steve - dont loop forever
		{
			NSLog (@"2011-12-05steve %s - looping too many times...", __func__ ); // 2011-12-05steve
			// 2012-11-13steve failed = true;
			[safePath autorelease];
			return creationPath; // return back original path????
			break;
		}
    }
	
	
	return [safePath autorelease];
}



NSString* getTodaysDateAsString(void)
{
	return [NSString stringWithFormat:@"%@",computeCurrentDaysFolderString2(3,0)];

//	NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
//	// anytime before 3AM is considered the dayBefore
//	NSString* newDateString = computeCurrentDaysFolderString2(3,0);
//
//	[newDateString retain];
//	[pool release];
//	if ( newDateString==nil )
//	{
//		NSLog (@"2011-11-07steve computeCurrentDaysFolderString2 returned NIL????" );
//	}
//	
//	return [newDateString autorelease];
}


// will create folder with "localPathName/YYYY-MM-DD" under the current app
// 2010-11-10steve
NSString* getWorkingDirectoryPath ( NSString* localPathName )
{
	// create the local working directory
	NSString* curBundlePath = [[NSBundle mainBundle]bundlePath];
	NSLog ( @"2010-11-11steve current curExecutablePath = %@", curBundlePath );
	NSString* curWorkingPath = [[curBundlePath stringByDeletingLastPathComponent]  stringByAppendingPathComponent:localPathName];
	curWorkingPath = [curWorkingPath stringByAppendingPathComponent:getTodaysDateAsString()];
	
	return curWorkingPath;
}



#pragma mark -

@implementation TepsExtras

@end
