//
//  TepsXmlWriter.m
//  Teps-HotFolder
//
//  Created by steve on 5/18/12.
//  Copyright 2012 TriPrism, Inc. All rights reserved.
//

#import "TepsXmlWriter.h"



NSString* XmlVersion1_0Header = @"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\" ?>";
NSString* mainXmlKeyName = @"data";
NSString* genericItemNameFormat = @"item%i";





@implementation TepsXmlWriter

@synthesize mString;
@synthesize mDictionary;
@synthesize mLookupArrayName;
@synthesize _indentCount;


#pragma mark -


-(id) init
{
	self = [super init];
	
	if ( self ) {
		self.mString = [NSMutableString stringWithCapacity:0];
		self.mDictionary = [NSMutableDictionary dictionaryWithCapacity:0];
		self._indentCount = 0;
		
		
		self.mLookupArrayName = [NSMutableDictionary dictionaryWithCapacity:0];
		[self.mLookupArrayName setObject:@"user" forKey:@"userdataxx"];
		
		
	}
	
	return self;
	
}

-(void) dealloc
{
	
	self.mString = nil;
	self.mDictionary = nil;
	self._indentCount = 0;

	[super dealloc];
}

#pragma mark -


-(BOOL) putTabsWithIndexCount:(NSInteger) indentCount
{
	if ( indentCount ) {
		NSInteger i = 0;
		for ( i=0; i<indentCount; i++ ) {
			[self.mString appendFormat:@"\t"];
		}
	}
	
	return YES;
	
}

-(BOOL) processEachObject:(id) obj forKey:(NSString*)objectName
{
	if ( obj==nil ) {
		NSLog (@ "Inside %@ => %@...obj is NIL ???", [self class], NSStringFromSelector(_cmd) );
		return NO; // 2013-03-26steve
	}
	
	[self putTabsWithIndexCount:self._indentCount];
	
	if ([obj isKindOfClass:[NSString class]] ) {
		[self.mString appendFormat:@"<%@>%@</%@>\n",objectName,obj,objectName];
	}
	
	if ([obj isKindOfClass:[NSDictionary class]] || [obj isKindOfClass:[NSMutableDictionary class]]) {
		[self convertDictionaryToXml:obj forKey:objectName];
	}
	
	if ([obj isKindOfClass:[NSArray class]] ) {
		if ( [obj count] ) {
			[self convertArrayToXml:obj forKey:objectName];
		}
	}
	
	return YES;
	
}


-(BOOL) convertArrayToXml:(NSArray*) arrInfo forKey:(NSString*)keyname 
{
	NSString* itemFormatString = nil;
	id arrayItemName = [self.mLookupArrayName objectForKey:keyname];
	if ( arrayItemName==nil ) {
		itemFormatString = [NSString stringWithString:genericItemNameFormat];
	}
		
		
	if ( keyname ) {
		
		[self.mString appendFormat:@"<%@>\n",keyname];
		++(self._indentCount);

		short xx = 0;
		for ( id obj in arrInfo ) {
			if ( itemFormatString ) {
				arrayItemName = [NSString stringWithFormat:itemFormatString,xx++];
			}

			[self processEachObject:obj forKey:arrayItemName];
			
		}
		
		--(self._indentCount);
		[self putTabsWithIndexCount:self._indentCount];
		[self.mString appendFormat:@"</%@>\n",keyname];

	}
	
	return YES;
}



-(BOOL) convertDictionaryToXml:(NSMutableDictionary*) dictInfo forKey:(NSString*)keyname
{
	
	if (keyname) {
		
		[self.mString appendFormat:@"<%@>\n",keyname];
		++(self._indentCount);
		
		
		NSLog ( @"all the keys = %@", [dictInfo allKeys] );
		NSArray* allKeys = [dictInfo allKeys];
		NSInteger totalKeys = [allKeys count];
		short i = 0;
		for ( i=0; i<totalKeys; i++ ) {
			
			id elementName = [allKeys objectAtIndex:i];
			id obj = [dictInfo objectForKey:elementName];
			
			[self processEachObject:obj forKey:elementName];
			
		}
		
		
		--(self._indentCount);
		[self putTabsWithIndexCount:self._indentCount];
		[self.mString appendFormat:@"</%@>\n",keyname];
		
	}
	
	
	
	return YES;
}

-(BOOL) convertDictionaryToXml3:(NSMutableDictionary*) dictInfo
{
	id keyname = [dictInfo objectForKey:@"mainXmlKeyName"];
	id attributes = [dictInfo objectForKey:@"attributes"];
	id keys = [dictInfo objectForKey:@"dictionary"];
	
	NSLog( @"2013-03-26steve dictInfo = %@", dictInfo );
	NSLog( @"2013-03-26steve keys = %@", keys );
	NSLog( @"2013-03-26steve keys = %@", keys );
	
	if (keys==nil ) {
		//BAD
		NSLog (@ "Inside %@ => %@...NO keys Passed...", self, NSStringFromSelector(_cmd) );
		return NO;
	}
	
	if (keyname) {
		
		NSString* attrString = @"";
		if (attributes) {
			NSArray* allkeys = [attributes allKeys];
			short count = 0;
			for ( id obj in allkeys ) {
				id value = [attributes objectForKey:obj];
				if ( count++) 
					attrString = [attrString stringByAppendingFormat:@" "];
				attrString = [attrString stringByAppendingFormat:@"%@=%@", obj, value];
			}
		}
		
		if ( attributes ) {
			[self.mString appendFormat:@"<%@ %@>\n",keyname,attrString];
		} else {
			[self.mString appendFormat:@"<%@>\n",keyname];
		}
		++(self._indentCount);
		
		
		NSLog ( @"all the keys = %@", [keys allKeys] );
		NSArray* allKeys = [keys allKeys];
		NSInteger totalKeys = [allKeys count];
		short i = 0;
		for ( i=0; i<totalKeys; i++ ) {
			
			id elementName = [allKeys objectAtIndex:i];
			id obj = [keys objectForKey:elementName];
			
			[self processEachObject:obj forKey:elementName];
			
		}
		
		
		--(self._indentCount);
		[self putTabsWithIndexCount:self._indentCount];
		[self.mString appendFormat:@"</%@>\n",keyname];
		
	}
	else {
		NSLog (@ "Inside %@ => %@...NO keyName set...", self, NSStringFromSelector(_cmd) );
	}
	
	
	return YES;
}




#pragma mark -
/*
-(BOOL) convertArrayToXmlOld:(NSArray*) arrInfo forKey:(NSString*)keyname withArrayItemName:(NSString*)arrayItemName
{
	
	
	if ( keyname && arrayItemName ) {
		
		[self.mString appendFormat:@"<%@>\n",keyname];
		++(self._indentCount);
		
		
		for ( id obj in arrInfo ) {
			
			[self putTabsWithIndexCount:self._indentCount];
			
			if ([obj isKindOfClass:[NSString class]] ) {
				[self.mString appendFormat:@"<%@>%@</%@>\n",arrayItemName,obj,arrayItemName];
			}
			
			if ([obj isKindOfClass:[NSDictionary class]] || [obj isKindOfClass:[NSMutableDictionary class]]) {
				[self convertDictionaryToXmlOld:obj forKey:arrayItemName];
			}
			
			if ([obj isKindOfClass:[NSArray class]] ) {
				if ( [obj count] ) {
					[self convertArrayToXmlOld:obj forKey:arrayItemName withArrayItemName:@"user"];
				}
			}
			
			
		}
		
		
		
		--(self._indentCount);
		[self putTabsWithIndexCount:self._indentCount];
		[self.mString appendFormat:@"</%@>\n",keyname];
		
	}
	
	return YES;
}


-(BOOL) convertDictionaryToXmlOld:(NSMutableDictionary*) dictInfo forKey:(NSString*)keyname
{
	
	if (keyname) {
		
		[self.mString appendFormat:@"<%@>\n",keyname];
		++(self._indentCount);

		
		NSLog ( @"all the keys = %@", [dictInfo allKeys] );
		NSArray* allKeys = [dictInfo allKeys];
		NSInteger totalKeys = [allKeys count];
		for ( short i=0; i<totalKeys; i++ ) {
			
			id elementName = [allKeys objectAtIndex:i];
			id obj = [dictInfo objectForKey:elementName];
			
			[self putTabsWithIndexCount:self._indentCount];

			if ([obj isKindOfClass:[NSString class]] ) {
				[self.mString appendFormat:@"<%@>%@</%@>\n",elementName,obj,elementName];
			}
			
			if ([obj isKindOfClass:[NSDictionary class]] || [obj isKindOfClass:[NSMutableDictionary class]]) {
				[self convertDictionaryToXmlOld:obj forKey:elementName];
			}
			
			if ([obj isKindOfClass:[NSArray class]] ) {
				if ( [obj count] ) {
					[self convertArrayToXmlOld:obj forKey:elementName withArrayItemName:@"user"];
				}
			}
			
		}
		
		
		--(self._indentCount);
		[self putTabsWithIndexCount:self._indentCount];
		[self.mString appendFormat:@"</%@>\n",keyname];

	}
	
	
	NSError* error = nil;
	[self.mString writeToFile:@"/xmlFile.xml" atomically:YES encoding:NSUTF8StringEncoding error:&error];
	
	return YES;
}
*/


#pragma mark -


-(BOOL) createDummyInfo
{
	self.mDictionary = [NSMutableDictionary dictionaryWithCapacity:0];
	
	[self.mDictionary setObject:@"ID12345" forKey:@"ids"];
	[self.mDictionary setObject:@"steve@triprism.com" forKey:@"emails"];
	[self.mDictionary setObject:@"triprism" forKey:@"account"];
	[self.mDictionary setObject:@"1234-1234" forKey:@"locationid"];
	
	// BOOL didWrite1 = [self.mDictionary writeToFile:@"/testDictionary1.plist" atomically:YES];
	// NSLog (@"2012-05-18steve didWrite1 = %@",didWrite1?@"YES":@"NO");
	
	
	NSMutableDictionary* userInfo1 = [NSMutableDictionary dictionaryWithCapacity:0];
	[userInfo1 setObject:@"s1@triprism.com" forKey:@"email"];
	[userInfo1 setObject:@"todd" forKey:@"name"];
	[userInfo1 setObject:@"12/06" forKey:@"dob"];
	[userInfo1 setObject:@"123-456-7890" forKey:@"phone"];
	
	[self.mDictionary setObject:[NSArray arrayWithObjects:userInfo1,userInfo1,userInfo1,nil] forKey:@"userdata" ];
	//[self.mDictionary setObject:userInfo1 forKey:@"userdata" ];
	
	BOOL didWrite2 = [self.mDictionary writeToFile:@"/testDictionary2.plist" atomically:YES];
	NSLog (@"2012-05-18steve didWrite2 = %@",didWrite2?@"YES":@"NO");
	
	return YES;
}



-(BOOL) doStartConversion:(NSMutableDictionary*) dictionary
{
	[self.mString setString:XmlVersion1_0Header];
	[self.mString appendFormat:@"\n"];

	[self convertDictionaryToXml:dictionary forKey:[NSString stringWithString:mainXmlKeyName]];
	
	// NSError* error = nil;
	// [self.mString writeToFile:@"/xmlFile.xml" atomically:YES encoding:NSUTF8StringEncoding error:&error];
	
	return YES;
}


-(BOOL) doStartConversion2:(NSMutableDictionary*) param	// 2013-03-19steve
{
	
	NSString* mainXmlKeyName = [param objectForKey:@"mainXmlKeyName"];
	id dictionary = [param objectForKey:@"dictionary"];
	
	[self.mString setString:XmlVersion1_0Header];
	[self.mString appendFormat:@"\n"];
	
	[self convertDictionaryToXml:dictionary forKey:[NSString stringWithString:mainXmlKeyName]];
	
	NSError* error = nil;
	[self.mString writeToFile:@"/xmlFile.xml" atomically:YES encoding:NSUTF8StringEncoding error:&error];
	
	return YES;
}

-(BOOL) doStartConversion3:(NSMutableDictionary*) param	// 2013-03-26steve
{
	
	NSLog( @"2013-03-26steve param = %@", param );

	NSString* mainXmlKeyName = [param objectForKey:@"mainXmlKeyName"];
	id attributes = [param objectForKey:@"attributes"];
	id dictionary = [param objectForKey:@"dictionary"];
	
	[self.mString setString:XmlVersion1_0Header];
	[self.mString appendFormat:@"\n"];
	
	[self convertDictionaryToXml3:param];
	
	NSError* error = nil;
	[self.mString writeToFile:@"/xmlFile.xml" atomically:YES encoding:NSUTF8StringEncoding error:&error];
	
	return YES;
}



-(BOOL) doTest
{
	
	[self createDummyInfo];
	
	[self doStartConversion:self.mDictionary];
	

	return YES;
}


-(BOOL) doTest2 // 2013-03-19steve
{

	NSMutableDictionary* printstationInfo = [NSMutableDictionary dictionaryWithCapacity:1];
	[printstationInfo setObject:@"some name" forKey:@"name"];
	[printstationInfo setObject:@"myMacAddress" forKey:@"mac"];
	[printstationInfo setObject:@"5" forKey:@"printlocationid"];
	
	[self.mString setString:XmlVersion1_0Header];
	[self.mString appendFormat:@"\n"];
	[self convertDictionaryToXml:printstationInfo forKey:[NSString stringWithString:@"printstation"]];
	
	NSLog (@"2013-03-19steve converted xmlData = %@", self.mString );
	
	if ( self.mString ) {
		NSMutableDictionary* param = [NSMutableDictionary dictionaryWithCapacity:1];
		[param setObject:@"mylongtokenstring" forKey:@"t"];
		[param setObject:[NSString stringWithString:self.mString] forKey:@"data"];
		
		NSLog (@"2013-03-19steve converted param = %@", param );

	} else {
		
	}
	

	
	
	
	return YES;
}



@end
