//
//  ATMPPR_Task.m
//
//  Created by tyler on 10/5/14.
//  Copyright (c) 2014 Atomic Pepper LLC. All rights reserved.
//

#import "ATMPPR_Task.h"

@implementation ATMPPR_Task
+(void)removeFilesInDirectory:(NSString *)searchPath ofKind:(NSArray *)fileExtensions minDaysOld:(NSNumber *)age
{
    [self removeFilesInDirectory:searchPath ofKind:fileExtensions daysOld:age maxDepth:nil];
}

+(void)removeFilesInDirectory:(NSString *)path ofKind:(NSArray *)fileExtensions daysOld:(NSNumber *)age maxDepth:(NSNumber *)maxDepth;
{
    NSMutableArray * patterns = [NSMutableArray array];
    for (NSString * arg in fileExtensions)
    {
        [patterns addObject:[@"*." stringByAppendingString:arg]];
    }

    [self removeFilesInDirectory:path matchingPatterns:patterns daysOld:age maxDepth:maxDepth];
}
+(void)removeFilesInDirectory:(NSString *)path matchingPatterns:(NSArray *)patterns daysOld:(NSNumber *)age maxDepth:(NSNumber *)maxDepth;
{
    [self findAtPath:path fileType:@"f" matchingPatterns:patterns daysOld:age maxDepth:maxDepth performingCommand:@"rm"];
}
+(void)removeSubdirectoriesAtPath:(NSString *)path matchingPatterns:(NSArray *)patterns daysOld:(NSNumber *)age maxDepth:(NSNumber *)maxDepth
{
    [self findAtPath:path fileType:@"d" matchingPatterns:patterns daysOld:age maxDepth:maxDepth performingCommand:@"rmdir"];
}
+(void)findAtPath:(NSString *)path fileType:(NSString *)fileType matchingPatterns:(NSArray *)patterns daysOld:(NSNumber *)age maxDepth:(NSNumber *)maxDepth performingCommand:(NSString *)toolName
{
    //find <path> -not -newerct '2 days ago' -and -type f -and '(' -name '*.mov' -or -name '*.m4v' -or -name '*.mp4' -or -name '*.jpg' ')' -exec ls '{}' +
    
    if(0 == path.length) return;
    while(path.length > 1 && [path hasSuffix:@"/"])
    {
        path = [path substringToIndex:path.length - 1];
    }
    
    NSMutableArray * args = [NSMutableArray array];
    [args addObjectsFromArray:@[@"-H",path]];
    if(fileType && [@[@"f",@"d",@"b",@"c",@"l",@"p",@"s"] containsObject:fileType])
    {
        [args addObject:@"-type"];
        [args addObject:fileType];
    }
    if(age)
    {
        [args addObject:@"-and"];
        [args addObject:@"-not"];
        // 2014-10-15steve [args addObject:@"-newerct"];
        [args addObject:@"-newerBt"]; // 2014-10-15steve want creation date - not change date
        [args addObject:[NSString stringWithFormat:@"%u day%s ago",age.unsignedIntValue,[@1 isEqualToNumber: age] ? "" :"s"]];
    }
    if (1 < patterns.count)
    {
        [args addObject:@"-and"];
        [args addObject:@"("];
        int i = 0;
        for (NSString * arg in patterns)
        {
            [args addObject:@"-iname"];
            [args addObject:arg];
            if(++i < patterns.count)
            {
                [args addObject:@"-or"];
            }
        }
        [args addObject:@")"];
    }
    else if (patterns.count)
    {
        [args addObject:@"-iname"];
        [args addObject:patterns.lastObject];
    }
    
    if(maxDepth)
    {
        [args addObject:@"-maxdepth"];
        [args addObject:[NSString stringWithFormat:@"%u",maxDepth.unsignedIntValue]];
    }
    [args addObject:@"-mindepth"];
    [args addObject:@"1"];
    if(toolName)
    {
        [args addObject:@"-exec"];
        [args addObject:toolName];
        [args addObject:@"{}"];
        [args addObject:@"+"];
    }
    
    NSString * tool = @"/usr/bin/find";
    [self launchToolAtPath:tool withArguments:args completionHandler:^(NSTask *task) {
        NSLog(@"%@ [%d] did complete with exit status %d",tool,task.processIdentifier,task.terminationStatus);
    }];
}
+(NSTask *)launchToolAtPath:(NSString *)launchPath withArguments:(NSArray *)args completionHandler:(void (^)(NSTask * task))completionBlock
{
    NSLog(@"starting task %@ %@",launchPath,[args componentsJoinedByString:@" "]);
    
    NSTask * task = [[NSTask alloc] init];
    [task setLaunchPath:launchPath];
    [task setArguments:args];
    task.terminationHandler = completionBlock;
    [task launch];
    NSLog(@"started task %@ [%d]",launchPath,task.processIdentifier);
    [task autorelease];
    return  task;
}

@end
