//
//  borderlessWindow.m
//  Teps-AppleEvents
//
//  Created by Steve on 8/6/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "borderlessWindow.h"
#include <string>



extern NSString* prefQuitTypedStringKey; // 2010-09-11steve
NSString* prefQuitTypedStringKey = @"prefQuitTypedString"; // 2010-09-11steve


@implementation borderlessWindow



- (void)timerFireMethod:(NSTimer*)theTimer
{
	NSLog (@ "Inside %@ => %@...", self, NSStringFromSelector(_cmd) );

	[fBuffer setString:@""];
	
	fTimer = nil;
}


#pragma mark -


//In Interface Builder we set CustomWindow to be the class for our window, so our own initializer is called here.
- (id)initWithContentRect:(NSRect)contentRect styleMask:(unsigned int)aStyle backing:(NSBackingStoreType)bufferingType defer:(BOOL)flag {
	
	NSLog (@ "Inside %@->%@...\n\n", self, NSStringFromSelector(_cmd) );
	
    //Call NSWindow's version of this function, but pass in the all-important value of NSBorderlessWindowMask
    //for the styleMask so that the window doesn't have a title bar
    
	// IF we still want to be able to drag the window
	// NSWindow* result = [super initWithContentRect:contentRect styleMask:aStyle backing:NSBackingStoreBuffered defer:NO];
    
	// 2008-08-04steve - truly BorderLESS
	// NSWindow* result = [super initWithContentRect:contentRect styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:NO];
	NSWindow* result = [super initWithContentRect:contentRect styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:NO];
   
	//Set the background color to clear so that (along with the setOpaque call below) we can see through the parts
    //of the window that we're not drawing into
    
	[result setBackgroundColor: [NSColor darkGrayColor]];
	//[result setBackgroundColor: [NSColor clearColor]];
  
	//This next line pulls the window up to the front on top of other system windows.  This is how the Clock app behaves;
    //generally you wouldn't do this for windows unless you really wanted them to float above everything.
    // [result setLevel: NSStatusWindowLevel];	
	// [result setLevel: NSFloatingWindowLevel];	// 2008-10-20steve
    // [result setLevel: NSModalPanelWindowLevel];	
    // [result setLevel: NSPopUpMenuWindowLevel];	
    // [result setLevel: NSScreenSaverWindowLevel];	 // prevents command-tab from being visible but still active behind

    //Let's start with no transparency for all drawing into the window
    [result setAlphaValue:1.0];
    
	//but let's turn off opaqueness so that we can see through the parts of the window that we're not drawing into
    [result setOpaque:NO];
    
	//and while we're at it, make sure the window has a shadow, which will automatically be the shape of our custom content.
    [result setHasShadow: YES];
    [result setHasShadow: NO];

    
	fCanDragWindow = NO;	// 2008-10-14steve YES;
	
    
	return result;
}

-(void)awakeFromNib
{
	NSLog (@ "%@->%@...", self, NSStringFromSelector(_cmd) );
		
    [self makeKeyAndOrderFront:nil]; // 2014-10-20steve
	
}

// added 2014-10-20steve
-(void) dealloc
{
    [super dealloc];
}



// Custom windows that use the NSBorderlessWindowMask can't become key by default.  Therefore, controls in such windows
// won't ever be enabled by default.  Thus, we override this method to change that.
- (BOOL) canBecomeKeyWindow
{
	NSLog (@ "%@->%@...", self, NSStringFromSelector(_cmd) );
    return YES;
}
- (void)windowDidResignKey:(NSNotification *)notification
{
	NSLog (@ "%@->%@...", self, NSStringFromSelector(_cmd) );
	
	// 2011-01-07steve [self makeKeyWindow];
	
}
- (void)windowDidResignMain:(NSNotification *)notification
{
	NSLog (@ "%@->%@...", self, NSStringFromSelector(_cmd) );
}



//Once the user starts dragging the mouse, we move the window with it. We do this because the window has no title
//bar for the user to drag (so we have to implement dragging ourselves)
- (void)mouseDragged:(NSEvent *)theEvent
{
	if ( fCanDragWindow==NO )
		return;
	
	NSPoint currentLocation;
	NSPoint newOrigin;
	NSRect  screenFrame = [[NSScreen mainScreen] frame];
	NSRect  windowFrame = [self frame];
	
	
	//grab the current global mouse location; we could just as easily get the mouse location 
	//in the same way as we do in -mouseDown:
    currentLocation = [self convertBaseToScreen:[self mouseLocationOutsideOfEventStream]];
    newOrigin.x = currentLocation.x - fInitialLocation.x;
    newOrigin.y = currentLocation.y - fInitialLocation.y;
    
    // Don't let window get dragged up under the menu bar
    if( (newOrigin.y+windowFrame.size.height) > (screenFrame.origin.y+screenFrame.size.height) ){
		newOrigin.y=screenFrame.origin.y + (screenFrame.size.height-windowFrame.size.height);
    }
    
    //go ahead and move the window to the new location
    [self setFrameOrigin:newOrigin];
}

//We start tracking the a drag operation here when the user first clicks the mouse,
//to establish the initial location.
- (void)mouseDown:(NSEvent *)theEvent
{    	
    NSRect  windowFrame = [self frame];
	
    //grab the mouse location in global coordinates
	fInitialLocation = [self convertBaseToScreen:[theEvent locationInWindow]];
	fInitialLocation.x -= windowFrame.origin.x;
	fInitialLocation.y -= windowFrame.origin.y;
	
	[super mouseDown:theEvent];
}


@end
