//
//  watcher.m
//  FTPUpload
//
//  Created by steve on 6/15/09.
//  Copyright 2009 TriPrism,Inc. All rights reserved.
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <dirent.h>
#include <assert.h>

#import "watcher.h"
// #import "NotificationHeader.h"
#import <CoreServices/CoreServices.h>

static off_t get_directory_size(const char *path, int add, int recursive);
static off_t get_total_size(void);


CFRunLoopRef gWatcherCFRunLoopRef = nil;


struct settings_t {
    FSEventStreamEventId sinceWhen;
    CFAbsoluteTime       latency;
    int                  flags;
    int                  num_paths;
    const char         **array_of_paths;
    bool                 print_settings;
    bool                 verbose;
    int                  flush_seconds;
} _settings;

static struct settings_t *settings = &_settings;



//
//--------------------------------------------------------------------------------
// Routines to keep track of the size of the directory hierarchy 
// we are watching.
//
// This code is not exemplary in any way.  It should definitely
// not be used in production code as it is inefficient.
//

typedef struct dir_item {
    char *dirname;
    off_t size;
} dir_item;

dir_item *dir_items=NULL;
int       num_dir_items=0;
int       max_dir_items=0;

#define DIR_ITEM_INCR  128

static int
add_dir_item(const char *name, off_t size)
{
    if (num_dir_items+1 >= max_dir_items) {
        dir_item *newitem;
		
        newitem = (dir_item *)realloc(dir_items, (max_dir_items+DIR_ITEM_INCR)*sizeof(dir_item));
        if (newitem == NULL) {
            return ENOSPC;
        }
        dir_items = newitem;
        max_dir_items += DIR_ITEM_INCR;
    }
	
    dir_items[num_dir_items].dirname = strdup(name);
    dir_items[num_dir_items].size    = size;
    
    num_dir_items++;
    return 0;
}


static int
update_dir_item(const char *name, off_t size)
{
    int i;
	
    for(i=0; i < num_dir_items; i++) {
        if (strcmp(name, dir_items[i].dirname) == 0) {
            dir_items[i].size = size;
            return 0;
        }
    }
	
    return ENOENT;
}


// note: this returns zero if the directory exists
//       otherwise it returns one (i.e. true).
static int
dir_does_not_exist(const char *name)
{
    int i;
	
    for(i=0; i < num_dir_items; i++) {
        if (strcmp(name, dir_items[i].dirname) == 0 && dir_items[i].size != 0) {
            return 0;
        }
    }
	
    return 1;
}


static off_t
get_total_size(void)
{
    int   i;
    off_t size=0;
	
    for(i=0; i < num_dir_items; i++) {
        size += dir_items[i].size;
    }
	
    return size;
}


static off_t
iterate_subdirs(const char *dirname, int add, int recursive)
{
    char          *fullpath;
    DIR           *dir;
    struct dirent *dirent;
    struct stat    st;
    off_t          size=0, result=0;
    
    fullpath = (char*)malloc(PATH_MAX);
    if (fullpath == NULL) {
        return -1;
    }
    
    if (add) {
        add_dir_item(dirname, 0);
    }
	
    dir = opendir(dirname);
    if (dir == NULL) {
        if(fullpath){
            free(fullpath);
        }
        if (errno == ENOENT) {             // it may have been deleted.
            update_dir_item(dirname, 0);
            return 0;
        }
		
        printf("failed to opendir(%s) (%s)\n", dirname, strerror(errno));
        return -1;
    }
	
    dirent = NULL;
    while ((dirent = readdir(dir)) != NULL) {
        if (strcmp(dirent->d_name, ".") == 0 || strcmp(dirent->d_name, "..") == 0)
            continue;
		
        snprintf(fullpath, PATH_MAX, "%s/%s", dirname, dirent->d_name);
        if (lstat(fullpath, &st) != 0) {
            printf("Error stating %s : %s\n", fullpath, strerror(errno));
            continue;
        }
		
        size += st.st_size;
        
        if (S_ISDIR(st.st_mode) && (recursive || dir_does_not_exist(fullpath))) {
            result = get_directory_size(fullpath, add, 1);
            if (result < 0) {
                printf("error getting size for %s\n", fullpath);
            }
        }
    }
	
    closedir(dir);
    free(fullpath);
	
    if (update_dir_item(dirname, size) == ENOENT) {
        add_dir_item(dirname, size);
    }
	
    return size;
}


static void
check_for_deleted_dirs(const char *dirname)
{
    int   i;
    struct stat st;
	
    for(i=0; i < num_dir_items; i++) {
        if (stat(dir_items[i].dirname, &st) != 0) {
            dir_items[i].size = 0;
        }
    }
}

static off_t
get_directory_size(const char *dirname, int add, int recursive)
{
    off_t sz;
	
    check_for_deleted_dirs(dirname);
	
    sz = iterate_subdirs(dirname, add, recursive);
	
    return sz;
}



//
//--------------------------------------------------------------------------------
//

// For things that should be logged only when -verbose is turned on.
static void
LogV(const char *format, ...)
{
    if (!settings->verbose) {
        return;
    }

//	NSLog (@"%s",format);
	
//    
//    va_list ap;
//    va_start(ap, format);
//    (void) vfprintf(stderr, format, ap);
//    va_end(ap);
}

// For logging errors.
static void
LogError(const char *format, ...)
{

//	NSLog (@"%s",format);
	
	
//    va_list ap;
//    va_start(ap, format);
//    (void) vfprintf(stderr, format, ap);
//    va_end(ap);
}

// For things that could go to stdout, but currently go to stderr to prevent out-of-order output with stderr output.
static void
Print(const char *format, ...)
{
 //	NSLog (@"%s",format);

//	va_list ap;
//    va_start(ap, format);
//    if (settings->verbose) {
//        (void) vfprintf(stderr, format, ap);
//    } else {
//        (void) vfprintf(stdout, format, ap);
//    }
//    va_end(ap);
}

//
//--------------------------------------------------------------------------------
// Callback routines
//

static void
timer_callback(CFRunLoopRef timer, void *info)
{
    FSEventStreamRef streamRef = (FSEventStreamRef)info;
	
    LogV("%s: CFAbsoluteTimeGetCurrent() => %.3f\n", __FUNCTION__, CFAbsoluteTimeGetCurrent());
    LogV("%s: FSEventStreamFlushAsync(streamRef = %p)...\n", __FUNCTION__, streamRef);
    FSEventStreamFlushAsync(streamRef);
}

static void
fsevents_callback(FSEventStreamRef streamRef, void *clientCallBackInfo, int numEvents, const char *const eventPaths[], const FSEventStreamEventFlags *eventMasks, const uint64_t *eventIDs)
{
    int   len, recursive;
    // off_t new_size;
    char  path_buff[PATH_MAX];
    const char *full_path = (const char *)clientCallBackInfo;
    
    LogV("%s(streamRef = %p, clientCallBackInfo = %p, numEvents = %d)\n", __FUNCTION__, streamRef, clientCallBackInfo, numEvents);
    // 2009-06-09steve LogV("%s: FSEventStreamGetSinceWhen(streamRef) => %lld\n", __FUNCTION__, FSEventStreamGetSinceWhen(streamRef));
	
    int i;
    for (i=0; i < numEvents; i++) {
		
        strcpy(path_buff, eventPaths[i]);
        len = strlen(path_buff);
        if (path_buff[len-1] == '/') {
            // chop off a trailing slash so that get_directory_size() works
            path_buff[--len] = '\0';
        }
		
        if (eventMasks[i] & kFSEventStreamEventFlagMustScanSubDirs) {
            recursive = 1;
        } else if (eventMasks[i] & kFSEventStreamEventFlagUserDropped) {
            printf("BAD NEWS! We dropped events.\n");
            printf("Forcing a full rescan.\n");
            recursive = 1;
            strlcpy(path_buff, full_path, sizeof(path_buff));
        } else if (eventMasks[i] & kFSEventStreamEventFlagKernelDropped) {
            printf("REALLY BAD NEWS! The kernel dropped events.\n");
            printf("Forcing a full rescan.\n");
            recursive = 1;
            strlcpy(path_buff, full_path, sizeof(path_buff));
        } else {
            recursive = 0;
        }
		
		{
			NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
			// NSBeep();
			NSLog ( @"=====>>>>  FOUND EVENT Inside fsevents_callback  <<<<<=======" );
			NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
			[nc postNotificationName:cNotification_WatcherItemChanged object:nil userInfo:nil];
			if (pool) [pool release];
		}
		
		
		/* 2009-06-30steve
	
        new_size = get_directory_size(path_buff, 0, recursive);
        if (new_size < 0) {
            printf("Could not update size on %s\n", path_buff);
        } else {
            printf("New total size: %lld (change made to: %s) for path: %s\n",
                   get_total_size(), path_buff, full_path);
        }
		
		*/
		
		
    }
}


//
//--------------------------------------------------------------------------------
//  Simple wrapper to create an FSEventStream
//
static FSEventStreamRef my_FSEventStreamCreate(const char *path)
{
    void                 *info_pointer = (void *)path;
    FSEventStreamContext  context = {0, info_pointer, NULL, NULL, NULL};
    FSEventStreamRef      streamRef = NULL;
    CFMutableArrayRef     cfArray;
	
    cfArray = CFArrayCreateMutable(kCFAllocatorDefault, 1, &kCFTypeArrayCallBacks);
    if (NULL == cfArray) {
        LogError("%s: ERROR: CFArrayCreateMutable() => NULL\n", __FUNCTION__);
        // goto Return;
		return nil;	// 2009-08-13 snow
    }
	
    CFStringRef cfStr = CFStringCreateWithCString(kCFAllocatorDefault, path, kCFStringEncodingUTF8);
    if (NULL == cfStr) {
        CFRelease(cfArray);
        // goto Return;
		return nil;	// 2009-08-13 snow
    }
	
    CFArraySetValueAtIndex(cfArray, 0, cfStr);
    CFRelease(cfStr);
	
    if (settings->verbose) {
        CFShow(cfArray);
    }
	
    streamRef = FSEventStreamCreate(kCFAllocatorDefault,
                                    (FSEventStreamCallback)&fsevents_callback,
                                    &context,
                                    cfArray,
                                    settings->sinceWhen,
                                    settings->latency,
                                    settings->flags);
    CFRelease(cfArray);
    if (NULL == streamRef) {
        LogError("%s: ERROR: FSEventStreamCreate() => NULL\n", __FUNCTION__);
        //goto Return;
		return nil;	// 2009-08-13 snow
    }
	
    if (settings->verbose) {
        FSEventStreamShow(streamRef);
    }
	
Return:
    return streamRef;
}

#pragma mark -
void doInstallTheWatcher (const char* fullpath)
{
	
	// set the settings
	memset(settings, 0, sizeof(struct settings_t));
    settings->latency = 1.0;
	settings->sinceWhen = kFSEventStreamEventIdSinceNow;
	settings->flush_seconds = 3;
		
	
    FSEventStreamRef streamRef;
    Boolean startedOK;

	streamRef = my_FSEventStreamCreate(fullpath);
	
    FSEventStreamScheduleWithRunLoop(streamRef, CFRunLoopGetCurrent(), kCFRunLoopDefaultMode);
	
    startedOK = FSEventStreamStart(streamRef);
    if (!startedOK) {
        LogError("%s: failed to start the FSEventStream\n");
        // goto out;
		if ( streamRef ) // 2009-08-13 snow clean up
		{
			FSEventStreamInvalidate(streamRef);
			FSEventStreamRelease(streamRef);
		}
		return;	// 2009-08-13 snow
    }
	
	
    //
    // NOTE: we get the initial size *after* we start the
    //       FSEventStream so that there is no window
    //       during which we would miss events.
    //
	// off_t dir_sz = get_directory_size(fullpath, 1, 1);
    // printf("Initial total size is: %lld for path: %s\n", get_total_size(), fullpath);
//    NSLog(@"Initial total size is: %lld for path: %s\n", get_total_size(), fullpath);
    
	if (settings->flush_seconds >= 0) {
        // LogV("%s: CFAbsoluteTimeGetCurrent() => %.3f\n", __FUNCTION__, CFAbsoluteTimeGetCurrent());
//        NSLog(@"%s: CFAbsoluteTimeGetCurrent() => %.3f\n", __FUNCTION__, CFAbsoluteTimeGetCurrent());
		
        CFAllocatorRef allocator = kCFAllocatorDefault;
        CFAbsoluteTime fireDate = CFAbsoluteTimeGetCurrent() + settings->flush_seconds;
        CFTimeInterval interval = settings->flush_seconds;
        CFOptionFlags flags = 0;
        CFIndex order = 0;
        CFRunLoopTimerCallBack callback = (CFRunLoopTimerCallBack)timer_callback;
        CFRunLoopTimerContext context = { 0, streamRef, NULL, NULL, NULL };
        CFRunLoopTimerRef timer = CFRunLoopTimerCreate(allocator, fireDate, interval, flags, order, callback, &context);
        CFRunLoopAddTimer(CFRunLoopGetCurrent(), timer, kCFRunLoopDefaultMode);
    }
	

	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	NSMutableDictionary* statusInfo = [NSMutableDictionary dictionaryWithObject:(NSRunLoop*) CFRunLoopGetCurrent() forKey:@"CFRunLoopRef"];
	[nc postNotificationName:cNotification_WatcherStarted object:nil userInfo:statusInfo];	
	NSLog ( @"doInstallTheWatcher is running inside NSRunLoop = %@\n", CFRunLoopGetCurrent() );
	
    //
    // Run
    //
    CFRunLoopRun();
	
	NSLog (@"Done With CFRunLoopRun inside doInstallTheWatcher..." );
	
	
    //
    // Stop / Invalidate / Release
    //
    FSEventStreamStop(streamRef);
	out:
    FSEventStreamInvalidate(streamRef);
    FSEventStreamRelease(streamRef);
}





#pragma mark -
@implementation watcher
-(id) init
{
	NSLog (@ "Inside %@ => %@...", [self class], NSStringFromSelector(_cmd) );
	
	if ((self = [super init]))
	{	// superclass may return nil

	}

	return self;
}


-(void) doInstall:(char*)fullpath
{
    Boolean startedOK;
	
	
	// set the settings
    memset(settings, 0, sizeof(struct settings_t));
    settings->latency = 3.0;
    settings->sinceWhen = kFSEventStreamEventIdSinceNow;
    settings->flush_seconds = -1;
	
	
	
	fFSEventStreamRef = my_FSEventStreamCreate(fullpath);
	
    FSEventStreamScheduleWithRunLoop(fFSEventStreamRef, CFRunLoopGetCurrent(), kCFRunLoopDefaultMode);
	
    startedOK = FSEventStreamStart(fFSEventStreamRef);
    if (!startedOK) {
        LogError("%s: failed to start the FSEventStream\n");
		if ( fFSEventStreamRef )
		{
			FSEventStreamInvalidate(fFSEventStreamRef);
			FSEventStreamRelease(fFSEventStreamRef);
		}
		
        // goto out;
		return;	// 2009-08-13 snow
	}
	
    //
    // NOTE: we get the initial size *after* we start the
    //       FSEventStream so that there is no window
    //       during which we would miss events.
    //
	// off_t dir_sz = get_directory_size(fullpath, 1, 1);
    // printf("Initial total size is: %lld for path: %s\n", get_total_size(), fullpath);
	//    NSLog(@"Initial total size is: %lld for path: %s\n", get_total_size(), fullpath);
    
	if (settings->flush_seconds >= 0) {
        // LogV("%s: CFAbsoluteTimeGetCurrent() => %.3f\n", __FUNCTION__, CFAbsoluteTimeGetCurrent());
		//        NSLog(@"%s: CFAbsoluteTimeGetCurrent() => %.3f\n", __FUNCTION__, CFAbsoluteTimeGetCurrent());
		
        CFAllocatorRef allocator = kCFAllocatorDefault;
        CFAbsoluteTime fireDate = CFAbsoluteTimeGetCurrent() + settings->flush_seconds;
        CFTimeInterval interval = settings->flush_seconds;
        CFOptionFlags flags = 0;
        CFIndex order = 0;
        CFRunLoopTimerCallBack callback = (CFRunLoopTimerCallBack)timer_callback;
        CFRunLoopTimerContext context = { 0, fFSEventStreamRef, NULL, NULL, NULL };
        CFRunLoopTimerRef timer = CFRunLoopTimerCreate(allocator, fireDate, interval, flags, order, callback, &context);
        CFRunLoopAddTimer(CFRunLoopGetCurrent(), timer, kCFRunLoopDefaultMode);
    }
	
    //
    // Run
    //
    CFRunLoopRun();
	
	// Stop / Invalidate / Release
    //
    FSEventStreamStop(fFSEventStreamRef);
out:
    FSEventStreamInvalidate(fFSEventStreamRef);
    FSEventStreamRelease(fFSEventStreamRef);
	
}

-(void) doStop
{
	if ( fFSEventStreamRef )
	{
		FSEventStreamStop(fFSEventStreamRef);
		FSEventStreamInvalidate(fFSEventStreamRef);
		FSEventStreamRelease(fFSEventStreamRef);
	}
	
}


@end


